package org.wow.autodkp.model;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.PointSystemFactory;
import org.wow.autodkp.persistence.ItemMasterDB;


/**
 * Stores central state: all raids and point systems.
 * 
 * @author jsandusky
 */
public class AutoDKPModel {
    
    private static AutoDKPModel mInstance = new AutoDKPModel();
    
    private RaidLookup mRaidLookup;
    private List mRaids;
    private IPointSystem[] mPointSystems;
    private Map mLoot;
    private List mUniqueLoot;
    private Raid mSelectedRaid;
    
    
    private AutoDKPModel() {
        mPointSystems = PointSystemFactory.loadConstraints();
    }
    
    
    public static AutoDKPModel getInstance() {
        return mInstance;
    }
    
    
    public ModelLoadStatus load(String raidFile) {
        mPointSystems = null;
        mRaidLookup = null;
        mRaids = null;
        mLoot = new HashMap();
        mUniqueLoot = null;
        mSelectedRaid = null;
        
        System.gc();
        
        ModelLoadStatus loadStatus = new ModelLoadStatus();
        
        mPointSystems = PointSystemFactory.loadConstraints();
        
        mRaidLookup = new RaidLookup();
        mRaids = mRaidLookup.initialize(raidFile, loadStatus);
        if (!loadStatus.isRaidsLoaded()) {
            return loadStatus;
        }
        
        // clear update flag
        ItemMasterDB.getDefault().setUpdateFlag(false);
        
        loadStatus.setMissingLoot(resolveMissingItems());
        
        return loadStatus;
    }
    
    private List resolveMissingItems() {
        mUniqueLoot = new ArrayList();
        final List missingLoots = new ArrayList();
        for (Iterator iter=mLoot.values().iterator(); iter.hasNext();) {
            Loot loot = (Loot) iter.next();
            mUniqueLoot.add(loot);
            if (!ItemMasterDB.getDefault().hasItem(loot.getID())) {
                missingLoots.add(loot);
            }
        }
        Comparator comparator = new Comparator() {
            public int compare(Object arg0, Object arg1) {
                Loot a = (Loot) arg0;
                Loot b = (Loot) arg1;
                return a.getName().compareTo(b.getName());
            }
        };
        Collections.sort(mUniqueLoot, comparator);
        Collections.sort(missingLoots, comparator);
        
        return missingLoots;
    }
    
    public List getAllLoot() {
        return mUniqueLoot;
    }

    public IPointSystem[] getConstraints() {
        return mPointSystems;
    }
    
    public void setSelectedRaid(Raid raid) {
        mSelectedRaid = raid;
    }
    
    public Raid getSelectedRaid() {
        return mSelectedRaid;
    }
    
    public List getRaidItems() {
        return mRaids;
    }
    
    public Loot findLoot(String id) {
        return (Loot) mLoot.get(id);
    }
    
    public Loot createLoot(String id) {
        Loot loot = (Loot) mLoot.get(id);
        if (loot == null) {
            loot = new Loot();
            loot.setID(id);
            loot.loadFromDB();
            mLoot.put(id, loot);
        }
        return loot;
    }
    
    public Loot createLoot(String id, 
                           String name, 
                           String color,
                           String bossName, 
                           String instanceName) {
        Loot loot = (Loot) mLoot.get(id);
        if (loot == null) {
            loot = new Loot(id, name, color, bossName, instanceName);
            mLoot.put(id, loot);
        } else {
            // synchronize boss and instance info, because some loot objects
            // may not have this info while others do
            if (
                    loot.getBossName() == LootConstants.BOSS_NAME_NONE &&
                    bossName           != LootConstants.BOSS_NAME_NONE) {
                loot.setBossName(bossName);
            }
            if (
                    loot.getInstanceName() == LootConstants.INSTANCE_NAME_NONE &&
                    instanceName           != LootConstants.INSTANCE_NAME_NONE) {
                loot.setInstanceName(instanceName);
            }
        }
        
        return loot;
    }
}
