package org.wow.autodkp.model;

import java.awt.Color;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.persistence.ItemMasterDB;
import org.wow.autodkp.persistence.ItemPointsDB;


/**
 * The representation of an item within AutoDKP.
 * 
 * @author jsandusky
 */
public class Loot implements Comparable {
    
    private static final String COLOR_STR_POOR      = "9d9d9d";
    private static final String COLOR_STR_COMMON    = "000000";
    private static final String COLOR_STR_UNCOMMON  = "1eff00";
    private static final String COLOR_STR_RARE      = "0070dd";
    private static final String COLOR_STR_EPIC      = "a335ee";
    private static final String COLOR_STR_LEGENDARY = "ff8000";
    private static final String COLOR_STR_ARTIFACT  = "ff0000";
    
    public static final Color COLOR_POOR      = new Color(157, 157, 157);
    public static final Color COLOR_COMMON    = new Color(  0,   0,   0);
    public static final Color COLOR_UNCOMMON  = new Color( 30, 255,   0);
    public static final Color COLOR_RARE      = new Color(  0, 112, 221);
    public static final Color COLOR_EPIC      = new Color(163,  53, 238);
    public static final Color COLOR_LEGENDARY = new Color(255, 128,   0);
    public static final Color COLOR_ARTIFACT  = new Color(255,   0,   0);
    
    private String mID;
    private String mName;
    private int mQuality;
    private String mColor;
    private String mBossName;
    private String mInstanceName;
    
    private Map mLootConstraintsToValuesMap = new HashMap();

    
    public Loot() {
    }
    
    public Loot(String id, 
                String name, 
                String color, 
                String bossName, 
                String instanceKey) {
        mID = id;
        mName = name;
        mBossName = bossName;
        mInstanceName = instanceKey;
        
        // fix name
        if (name.indexOf(":") >= 0) {
            mName = name.replace(':', '-');
        }
        
        // fix color
        if (color.length() > 6) {
            mColor = color.toLowerCase().substring(2);
            if (mColor.equals("ffffff")) {
                mColor = "000000";
            }
        } else {
            mColor = color;
        }
        
        loadFromDB();
    }
    
    
    public void putValueForConstraint(IPointSystem constraint, String value) {
        mLootConstraintsToValuesMap.put(constraint.getID(), value);
    }
    
    public String getValueForConstraint(IPointSystem constraint) {
        return (String) mLootConstraintsToValuesMap.get(constraint.getID());
    }
    
    public static Cost findLootCost(PlayerData playerData, IPointSystem pointSystem, Loot loot) {
        if (pointSystem.isAuctionedLootValues()) {
            // For point systems that support auctioned loot values, we grab
            // the cost of the loot from the recorded value in the player data.
            return playerData.getAuctionedLootValue(loot);
        }
        
        // Otherwise, the value of the loot is predetermined.
        String lootValue = loot.getValueForConstraint(pointSystem);
        if (lootValue != null) {
            if (LootConstants.isLootCategoryID(lootValue)) {
                // loot value is a loot category ID
                List lootCosts = pointSystem.getLootConstraintCosts();
                int index = pointSystem.getLootConstraintIndex(lootValue);
                if (index >= 0) {
                    return (Cost) lootCosts.get(index);
                }
            } else {
                // loot value is a direct hard value
                return Cost.parseCost(lootValue);
            }
        }
        return null;
    }
    
    public void setID(String id) {
        mID = id;
    }
    
    public void loadFromDB() {
        if (ItemMasterDB.getDefault().hasItem(mID)) {
            mQuality  = ItemMasterDB.getDefault().getQuality(mID);
            if (mColor == null) {
                mColor = getColorStringFromQuality(mQuality);
            }
            if (mName == null) {
                mName = ItemMasterDB.getDefault().getName(mID);
            }
            if (mBossName == null) {
                mBossName = ItemMasterDB.getDefault().getBossName(mID);
            } else {
                String dbBoss = ItemMasterDB.getDefault().getBossName(mID);
                if (dbBoss == null || dbBoss.length() == 0) {
                    // must update database
                    ItemMasterDB.getDefault().addLoot(this);
                    ItemMasterDB.getDefault().setUpdateFlag(true);
                }
            }
            if (mInstanceName == null) {
                mInstanceName = ItemMasterDB.getDefault().getInstanceName(mID);
            } else {
                String dbInstance = ItemMasterDB.getDefault().getInstanceName(mID);
                if (dbInstance == null || dbInstance.length() == 0) {
                    // must update database
                    ItemMasterDB.getDefault().addLoot(this);
                    ItemMasterDB.getDefault().setUpdateFlag(true);
                }
            }
        } else {
            if (mColor != null) {
                mQuality = getQualityFromColorString(mColor);
            }
            if (mBossName == null) {
                mBossName = LootConstants.BOSS_NAME_NONE;
            }
            if (mInstanceName == null) {
                mInstanceName = LootConstants.INSTANCE_NAME_NONE;
            }
        }
        
        IPointSystem constraints[] = AutoDKPModel.getInstance().getConstraints();
        for (int i=0; i < constraints.length; i++) {
            ItemPointsDB constraintDB = (ItemPointsDB) constraints[i].getItemDB();
            if (constraintDB.hasItem(mID)) {
                mLootConstraintsToValuesMap.put(constraints[i].getID(), constraintDB.getValue(mID));
            }
        }
    }
    
    public String getID() {
        return mID;
    }
    
    public String getName() {
        return mName;
    }

    public String getBossName() {
        return mBossName;
    }
    
    public void setBossName(String name) {
        mBossName = name;
    }
    
    public void setInstanceName(String name) {
        mInstanceName = name;
    }
    
    public String getInstanceName() {
        return mInstanceName;
    }
    
    public String getColor() {
        return mColor;
    }

    public int getQuality() {
        return mQuality;
    }
    
    public int compareTo(Object arg0) {
        Loot other = (Loot) arg0;
        return this.mName.compareTo(other.mName);
    }
    
    public static int getQualityFromColorString(String colorString) {
        int quality = LootConstants.QUALITY_NONE;
        if (       colorString.equals(COLOR_STR_POOR)) {
            quality = LootConstants.QUALITY_POOR;
        } else if (colorString.equals(COLOR_STR_COMMON)) {
            quality = LootConstants.QUALITY_COMMON;
        } else if (colorString.equals(COLOR_STR_UNCOMMON)) {
            quality = LootConstants.QUALITY_UNCOMMON;
        } else if (colorString.equals(COLOR_STR_RARE)) {
            quality = LootConstants.QUALITY_RARE;
        } else if (colorString.equals(COLOR_STR_EPIC)) {
            quality = LootConstants.QUALITY_EPIC;
        } else if (colorString.equals(COLOR_STR_LEGENDARY)) {
            quality = LootConstants.QUALITY_LEGENDARY;
        } else {
            quality = LootConstants.QUALITY_ARTIFACT;
        }
        return quality;
    }
    
    public static String getColorStringFromQuality(int quality) {
        switch (quality) {
        case LootConstants.QUALITY_POOR:
            return COLOR_STR_POOR;
        case LootConstants.QUALITY_COMMON:
            return COLOR_STR_COMMON;
        case LootConstants.QUALITY_UNCOMMON:
            return COLOR_STR_UNCOMMON;
        case LootConstants.QUALITY_RARE:
            return COLOR_STR_RARE;
        case LootConstants.QUALITY_EPIC:
            return COLOR_STR_EPIC;
        case LootConstants.QUALITY_LEGENDARY:
            return COLOR_STR_LEGENDARY;
        case LootConstants.QUALITY_ARTIFACT:
            return COLOR_STR_ARTIFACT;
        }
        return null;
    }
    
    public static Color getColorFromQuality(int quality) {
        switch (quality) {
        case LootConstants.QUALITY_POOR:
            return COLOR_POOR;
        case LootConstants.QUALITY_COMMON:
            return COLOR_COMMON;
        case LootConstants.QUALITY_UNCOMMON:
            return COLOR_UNCOMMON;
        case LootConstants.QUALITY_RARE:
            return COLOR_RARE;
        case LootConstants.QUALITY_EPIC:
            return COLOR_EPIC;
        case LootConstants.QUALITY_LEGENDARY:
            return COLOR_LEGENDARY;
        case LootConstants.QUALITY_ARTIFACT:
            return COLOR_ARTIFACT;
        }
        return Color.BLACK;
    }
    
    public static Color getColorFromColorString(String colorString) {
        if (colorString == null || colorString.equals("")) {
            return null;
        }
        if        (colorString.equals(COLOR_STR_POOR)) {
            return COLOR_POOR;
        } else if (colorString.equals(COLOR_STR_UNCOMMON)) {
            return COLOR_UNCOMMON;
        } else if (colorString.equals(COLOR_STR_COMMON)) {
            return COLOR_COMMON;
        } else if (colorString.equals(COLOR_STR_RARE)) {
            return COLOR_RARE;
        } else if (colorString.equals(COLOR_STR_EPIC)) {
            return COLOR_EPIC;
        } else if (colorString.equals(COLOR_STR_LEGENDARY)) {
            return COLOR_LEGENDARY;
        }
        return COLOR_ARTIFACT;
    }
    
    public static String getColorStringFromColor(Color color) {
        if (color == null) {
            return "";
        }
        if        (color.equals(COLOR_POOR)) {
            return COLOR_STR_POOR;
        } else if (color.equals(COLOR_UNCOMMON)) {
            return COLOR_STR_UNCOMMON;
        } else if (color.equals(COLOR_COMMON)) {
            return COLOR_STR_COMMON;
        } else if (color.equals(COLOR_RARE)) {
            return COLOR_STR_RARE;
        } else if (color.equals(COLOR_EPIC)) {
            return COLOR_STR_EPIC;
        } else if (color.equals(COLOR_LEGENDARY)) {
            return COLOR_STR_LEGENDARY;
        }
        return COLOR_STR_ARTIFACT;
    }
}
