package org.wow.autodkp.model;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.List;


/**
 * 
 * 
 * @author jsandusky
 */
public class PlayTime {
    
    private List mTimeSpans;
    
    public PlayTime(List joinDates, List leaveDates) {
        // We may sometimes have a duplicate of a join date
        // (and possibly leave date?), that the date compare on
        // PlayerData addTimeJoin/addTimeLeave fails on (i.e.
        // because of a difference in only a second or two.
        // We must remove these to get accurate time span info.
        List allDates = new ArrayList();
        for (Iterator iter=joinDates.iterator(); iter.hasNext();) {
            Date iterDate = (Date) iter.next();
            allDates.add(new PlayDate(iterDate, true));
        }
        for (Iterator iter=leaveDates.iterator(); iter.hasNext();) {
            Date iterDate = (Date) iter.next();
            allDates.add(new PlayDate(iterDate, false));
        }
        Collections.sort(allDates);
        
        List fixedDateList = new ArrayList();
        
        // These algorithms have grown to be somewhat complex, therefore I have
        // added a JUnit test case that verifies dozens of test cases. Run that
        // test case each time a change is made to this class.
        
        // Now we go through all the dates and fix their arrangement.
        // The purpose of this is to only have valid join/leave pairs in the array.
        // If a join follows a join, and leave follows a leave, we must skip or insert
        // join times to fix the arrangement.
        for (int i=0; i < allDates.size(); i++) {
            PlayDate playDate = (PlayDate) allDates.get(i);
            
            if (i == 0 && i + 1 >= allDates.size()) {
                // Only item in the entire list
                if (!playDate.mIsJoin) {
                    // Only date is a leave, insert a join with no start time.
                    fixedDateList.add(new PlayDate(null, true));
                    fixedDateList.add(playDate);
                } else {
                    // Only date is a join, append a leave with no end time.
                    fixedDateList.add(playDate);
                    fixedDateList.add(new PlayDate(null, false));
                }
            }
            
            else if (i == 0) {
                // First date
                if (!playDate.mIsJoin) {
                    if (checkNext(fixedDateList, playDate, allDates, i)) {
                        // First date is a leave, insert a join of any time.
                        fixedDateList.add(new PlayDate(null, true));
                        fixedDateList.add(playDate);
                    } else {
                        // First date is a leave, followed by another leave.
                        // Strip the first leave off as if it were never there.
                        allDates.remove(i);
                        i -= 1;
                    }
                } else {
                    // First date is a join, check next date.
                    if (checkNext(fixedDateList, playDate, allDates, i)) {
                        fixedDateList.add(playDate);
                    } else {
                        // First date is a join, followed by another join.
                        // Strip the second join as if it were never there.
                        allDates.remove(i + 1);
                        i -= 1;
                    }
                }
            }

            else if (i + 1 >= allDates.size()) {
                // Last date
                if (!playDate.mIsJoin) {
                    // Last date is a leave, no change.
                    fixedDateList.add(playDate);
                } else {
                    // Last date is a join, append a leave with no end time.
                    fixedDateList.add(playDate);
                    fixedDateList.add(new PlayDate(null, false));
                }
            }
            
            else if (i > 0 && i + 1 < allDates.size()) {
                // Somewhere in the middle.
                if (checkNext(fixedDateList, playDate, allDates, i)) {
                    fixedDateList.add(playDate);
                }
            }
        }
        
        // Now we should have a list divisible by 2, that always
        // consists of join-then-leave pairings.
        mTimeSpans = new ArrayList();
        for (Iterator iter=fixedDateList.iterator(); iter.hasNext();) {
            PlayDate playDate = (PlayDate) iter.next();
            if (iter.hasNext()) {
                PlayDate nextDate = (PlayDate) iter.next();
                mTimeSpans.add(new TimeSpan(playDate.mDate, nextDate.mDate));
            }
        }
    }
    
    // Checks the next date in the list to determine what to do with the current date.
    private boolean checkNext(List fixedDateList, PlayDate playDate, List allDates, int index) {
        PlayDate nextDate = (PlayDate) allDates.get(index + 1);
        if (playDate.mIsJoin) {
            if (nextDate.mIsJoin) {
                // Two join times in a row, skip the next join time.
                // We do this by essentially moving the current join time
                // to be the next join time, and skipping it this iteration.
                allDates.set(index + 1, playDate);
            } else {
                // Join followed by Leave, no change.
                return true;
            }
        } else {
            if (!nextDate.mIsJoin) {
                // Two leave times in a row, skip this leave time.
            } else {
                // Leave followed by Join, no change.
                return true;
            }
        }
        return false;
    }
    
    public boolean includes(Date date) {
        for (Iterator iter=mTimeSpans.iterator(); iter.hasNext();) {
            TimeSpan timeSpan = (TimeSpan) iter.next();
            if (timeSpan.includes(date)) {
                return true;
            }
        }
        return false;
    }
    
    public int getMinutes(Date startDate, Date endDate) {
        int totalMinutes = 0;
        for (Iterator iter=mTimeSpans.iterator(); iter.hasNext();) {
            TimeSpan timeSpan = (TimeSpan) iter.next();
            totalMinutes += timeSpan.getMinutes(startDate, endDate);
        }
        return totalMinutes;
    }
    
    
    /**
     * Wraps around a date that may represent either a 
     * join or leave time. This allows join and leave dates
     * to be merged into the same list and sorted, while
     * preserving whether the date is join or leave.
     */
    private static class PlayDate implements Comparable {
        private Date mDate;
        private boolean mIsJoin;
        public PlayDate(Date date, boolean isJoin) {
            mDate = date;
            mIsJoin = isJoin;
        }
        public int compareTo(Object arg0) {
            PlayDate other = (PlayDate) arg0;
            if (other.mDate == null) {
                return -1;
            } else if (mDate == null) {
                return 0;
            }
            return mDate.compareTo(other.mDate);
        }
    }
    
    
    private static class TimeSpan {
        private Date mStart;
        private Date mEnd;
        public TimeSpan(Date start, Date end) {
            mStart = start;
            mEnd = end;
        }
        public boolean includes(Date date) {
            if (mStart == null && mEnd == null) {
                return true;
            } else if (mStart == null) {
                if (date.compareTo(mEnd) <= 0) {
                    return true;
                }
            } else if (mEnd == null) {
                if (date.compareTo(mStart) >= 0) {
                    return true;
                }
            } else {
                if (
                        date.compareTo(mStart) >= 0 &&
                        date.compareTo(mEnd)   <= 0) {
                    return true;
                }
            }
            return false;
        }
        public int getMinutes(Date startDate, Date endDate) {
            int minutes = 0;
            Date actualStart = null;
            Date actualEnd = null;
            if (mStart != null && mEnd != null) {
                actualStart = mStart;
                actualEnd = mEnd;
            } else if (mStart == null) {
                actualStart = startDate;
                actualEnd = mEnd;
            } else if (mEnd == null) {
                actualStart = mStart;
                actualEnd = endDate;
            }
            if (actualStart != null && actualEnd != null) {
                if (actualStart.compareTo(startDate) < 0) {
                    actualStart = startDate;
                }
                if (actualEnd.compareTo(endDate) > 0) {
                    actualEnd = endDate;
                }
                minutes = (int) (actualEnd.getTime() - actualStart.getTime());
                minutes /= 60000;
            }
            if (minutes < 0) {
                minutes = 0;
            }
            return minutes;
        }
    }
}
