package org.wow.autodkp.model;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.IdentityHashMap;
import java.util.Iterator;
import java.util.List;



/**
 * Represents raid data for a player.
 * 
 * @author jsandusky
 */
public class PlayerData implements Comparable {
    
    public static final String[] CLASSES = {
        "Druid",
        "Hunter",
        "Mage",
        "Paladin",
        "Priest",
        "Rogue",
        "Shaman",
        "Warlock",
        "Warrior"
    };
    
    // Fake player names that may appear in Raid Tracker.
    public static final String UNKNOWN_ENTITY = "Unknown Entity";
    public static final String UNKNOWN = "Unknown";
    
    protected String mPlayerName;
    protected String mPlayerClass;
    
    private List mJoinDates;
    private List mLeaveDates;
    private List mLoots;
    private List mLootTimes;
    private List mAuctionedLootValues;
    private PlayTime mPlayTime;
    
    
    public PlayerData() {
    }
    
    public PlayerData(String playerName, String playerClass) {
        mPlayerName = playerName;
        mPlayerClass = playerClass;
    }
    
    public String getPlayerClass() {
        return mPlayerClass;
    }
    
    public void setPlayerClass(String playerClass) {
        if (playerClass != null) {
            mPlayerClass = playerClass.toLowerCase();
        } else {
            mPlayerClass = "";
        }
    }
    
    public void setPlayerName(String name) {
        mPlayerName = name;
    }
    
    public void addTimeJoin(Date date) {
        if (mJoinDates == null) {
            mJoinDates = new ArrayList();
        }
        if (!mJoinDates.contains(date)) {
            mJoinDates.add(date);
        }
    }
    
    public void addTimeLeave(Date date) {
        if (mLeaveDates == null) {
            mLeaveDates = new ArrayList();
        }
        if (!mLeaveDates.contains(date)) {
            mLeaveDates.add(date);
        }
    }

    public List getJoinTimes() {
        return mJoinDates;
    }
    
    public List getLeaveTimes() {
        return mLeaveDates;
    }
    
    public Date getFirstJoinTime() {
        if (mJoinDates != null && mJoinDates.size() > 0) {
            return (Date) mJoinDates.get(0);
        }
        return null;
    }
    
    public boolean hasTimeLeave() {
        if (mLeaveDates != null) {
            return mLeaveDates.size() > 0;
        }
        return false;
    }
    
    public boolean isNonRaidMember() {
        if (
                (mJoinDates  == null || mJoinDates.size() == 0) &&
                (mLeaveDates == null || mLeaveDates.size() == 0)) {
            return true;
        }
        return false;
    }

    public List getLootTimes() {
        return mLootTimes;
    }
    
    public void addLoot(Loot loot, Date date, Cost auctionValue) {
        if (mLoots == null) {
            mLoots = new ArrayList();
            mLootTimes = new ArrayList();
            mAuctionedLootValues = new ArrayList();
        }
        mLoots.add(loot);
        mLootTimes.add(date);
        mAuctionedLootValues.add(auctionValue);
    }

    public Date getLootTimeForLoot(Loot loot) {
        if (mLoots != null) {
            for (int i=0; i < mLoots.size(); i++) {
                Loot iterLoot = (Loot) mLoots.get(i);
                if (iterLoot == loot) {
                    return (Date) mLootTimes.get(i);
                }
            }
        }
        return null;
    }
    
    public List getAuctionedLootValues() {
        return mAuctionedLootValues;
    }
    
    public Cost getAuctionedLootValue(Loot loot) {
        if (mLoots != null) {
            for (int i=0; i < mLoots.size(); i++) {
                Loot iterLoot = (Loot) mLoots.get(i);
                if (iterLoot == loot) {
                    return (Cost) mAuctionedLootValues.get(i);
                }
            }
        }
        return null;
    }
    
    public void setAuctionedLootValue(Loot loot, Cost cost) {
        if (mLoots != null) {
            for (int i=0; i < mLoots.size(); i++) {
                Loot iterLoot = (Loot) mLoots.get(i);
                if (iterLoot == loot) {
                    mAuctionedLootValues.set(i, cost);
                }
            }
        }
    }
    
    public int getLootCount() {
        if (mLoots != null) {
            return mLoots.size();
        }
        return 0;
    }

    public String getPlayerName() {
        return mPlayerName;
    }
    
    public List getLoots() {
        return mLoots;
    }
    
    public void sortLootByTimeLooted() {
        
        class LootSortItem {
            private Loot mLoot;
            private Cost mAuctionedValue;
            public LootSortItem(Loot loot, Cost auctionedValue) {
                mLoot = loot;
                mAuctionedValue = auctionedValue;
            }
        }
        
        if (mLoots != null) {
            IdentityHashMap identityMap = new IdentityHashMap();
            for (int i=0; i < mLoots.size(); i++) {
                Loot loot = (Loot) mLoots.get(i);
                Date time = (Date) mLootTimes.get(i);
                Cost cost = (Cost) mAuctionedLootValues.get(i);
                identityMap.put(time, new LootSortItem(loot, cost));
            }
            Collections.sort(mLootTimes);
            List newLootList           = new ArrayList();
            List newAuctionedValueList = new ArrayList();
            for (int i=0; i < mLootTimes.size(); i++) {
                Date time             = (Date) mLootTimes.get(i);
                LootSortItem sortItem = (LootSortItem) identityMap.get(time);
                newLootList.add(sortItem.mLoot);
                newAuctionedValueList.add(sortItem.mAuctionedValue);
            }
            mLoots = newLootList;
            mAuctionedLootValues = newAuctionedValueList;
        }
    }
    
    public PlayTime getPlayTime() {
        if (mPlayTime == null) {
            if (mJoinDates == null) {
                mJoinDates = new ArrayList();
            }
            if (mLeaveDates == null) {
                mLeaveDates = new ArrayList();
            }
            mPlayTime = new PlayTime(mJoinDates, mLeaveDates);
        }
        return mPlayTime;
    }
    
    public int compareTo(Object arg0) {
        PlayerData other = (PlayerData) arg0;
        return this.getPlayerName().compareTo(other.getPlayerName());
    }
    
    public void merge(PlayerData data, String playerName, String playerClass) {
        // Override player name and class.
        setPlayerName(playerName);
        setPlayerClass(playerClass);
        
        merge(data);
    }
    
    public void merge(PlayerData data) {
        if (mPlayerName == null) {
            mPlayerName = data.getPlayerName();
        }
        if (mPlayerClass == null) {
            mPlayerClass = data.getPlayerClass();
        }
        if (data.getJoinTimes() != null) {
            if (mJoinDates == null) {
                mJoinDates = new ArrayList();
                mJoinDates.addAll(data.getJoinTimes());
            } else {
                addUniqueDates(mJoinDates, data.getJoinTimes());
            }
        }
        if (data.getLeaveTimes() != null) {
            if (mLeaveDates == null) {
                mLeaveDates = new ArrayList();
                mLeaveDates.addAll(data.getLeaveTimes());
            } else {
                addUniqueDates(mLeaveDates, data.getLeaveTimes());
            }
        }
        mPlayTime = null;
        if (data.getLoots() != null && data.getLootTimes() != null) {
            if (mLoots == null) {
                mLoots = new ArrayList();
                mLootTimes = new ArrayList();
                mAuctionedLootValues = new ArrayList();
                mLoots.addAll(data.getLoots());
                mLootTimes.addAll(data.getLootTimes());
                mAuctionedLootValues.addAll(data.getAuctionedLootValues());
            } else {
                addUniqueLoot(mLoots, mLootTimes, mAuctionedLootValues, data);
            }
        }
    }
    
    private static void addUniqueLoot(
            List myLoots, 
            List myTimes, 
            List myLootValues,
            PlayerData otherData) {
        List otherLoots = otherData.getLoots();
        List otherTimes = otherData.getLootTimes();
        List otherValues = otherData.getAuctionedLootValues();
        for (int otherCount = 0; otherCount < otherLoots.size(); otherCount++) {
            Loot otherLoot = (Loot) otherLoots.get(otherCount);
            Date otherTime = (Date) otherTimes.get(otherCount);
            boolean isFound = false;
            for (int myCount = 0; myCount < myLoots.size(); myCount++) {
                Loot myLoot = (Loot) myLoots.get(myCount);
                Date myTime = (Date) myTimes.get(myCount);
                if (
                        (myLoot == otherLoot) &&
                        (myTime.equals(otherTime))) {
                    isFound = true;
                    break;
                }
            }
            if (!isFound) {
                myLoots.add(otherLoot);
                myTimes.add(otherTime);
                myLootValues.add(otherValues.get(otherCount));
            }
        }
    }
    
    private static void addUniqueDates(List originalDates, List addedDates) {
        for (Iterator otherIter=addedDates.iterator(); otherIter.hasNext();) {
            Date otherDate = (Date) otherIter.next();
            if (otherDate != null) {
                boolean isFound = false;
                for (Iterator myIter=originalDates.iterator(); myIter.hasNext();) {
                    Date myDate = (Date) myIter.next();
                    if (myDate != null && myDate.equals(otherDate)) {
                        isFound = true;
                        break;
                    }
                }
                if (!isFound) {
                    originalDates.add(otherDate);
                }
            }
        }
    }
}
