package org.wow.autodkp.model;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.wow.autodkp.persistence.Persistence;
import org.wow.autodkp.persistence.xml.XMLRaidWriter;


/**
 * 
 * 
 * @author jsandusky
 */
public class Raid {
    
    private String mID;
    private File mSaveFile;
    private Date mStartDate;
    private Date mEndDate;
    private PlayerData[] mPlayerData;
    private Map mBossNameToKillDateMap;
    private List mInstanceNames;
    
    
    public Raid(String id, 
                File saveFile, 
                Date startDate, 
                Date endDate, 
                Map bossNameToKillDateMap,
                List instanceNames,
                PlayerData[] playerData) {
        mID = id;
        mSaveFile = saveFile;
        mStartDate = startDate;
        mEndDate = endDate;
        mBossNameToKillDateMap = bossNameToKillDateMap;
        mInstanceNames = instanceNames;
        mPlayerData = playerData;
    }
    
    
    public String getID() {
        return mID;
    }
    
    public PlayerData[] getPlayerData() {
        return mPlayerData;
    }
    
    public void setPlayerData(PlayerData[] playerData) {
        mPlayerData = playerData;
    }
    
    public void addPlayerData(List newPlayerData) {
        if (newPlayerData != null && newPlayerData.size() > 0) {
            for (int i=0; i < mPlayerData.length; i++) {
                newPlayerData.add(mPlayerData[i]);
            }
            Collections.sort(newPlayerData);
            mPlayerData = (PlayerData[]) newPlayerData.toArray(new PlayerData[0]);
        }
    }
    
    public Date getEndTime() {
        return mEndDate;
    }
    
    public Date getStartTime() {
        return mStartDate;
    }
    
    public Map getBossNameToKillDateMap() {
        return mBossNameToKillDateMap;
    }
    
    public void setBossKill(BossKillInfo bossKillInfo) {
        if (mBossNameToKillDateMap == null) {
            mBossNameToKillDateMap = new HashMap();
        }
        mBossNameToKillDateMap.put(
                bossKillInfo.getBossName(), 
                bossKillInfo.getKillDate());
    }
    
    public void removeBossKill(String bossName) {
        if (mBossNameToKillDateMap != null) {
            mBossNameToKillDateMap.remove(bossName);
        }
    }

    public List getInstanceNames() {
        return mInstanceNames;
    }
    
    public void setInstanceNames(List instanceNames) {
        mInstanceNames = instanceNames;
    }
    
    /**
     * Returns whether the raid file exists
     * (i.e. whether it is promoted above the Raid Tracker log)
     */
    public boolean isPromoted() {
        return mSaveFile.exists();
    }
    
    /**
     * Saves the raid file and promotes it, which
     * causes it to override the Raid Tracker raid log
     * while this raid file exists.
     */
    public void promote() {
        new XMLRaidWriter().writeXML(mSaveFile, mStartDate, mEndDate, mBossNameToKillDateMap, mInstanceNames, mPlayerData);
    }
    
    public void delete() {
        if (isPromoted()) {
            mSaveFile.delete();
        }
    }
    
    public File export(File saveDir) {
        File saveFile = new File(saveDir.getPath() + File.separator + mID + Persistence.XML_EXTENSION);
        new XMLRaidWriter().writeXML(saveFile, mStartDate, mEndDate, mBossNameToKillDateMap, mInstanceNames, mPlayerData);
        return saveFile;
    }
    
    public Raid merge(Raid otherRaid) {
        Date myStart = getStartTime();
        Date otherStart = otherRaid.getStartTime();
        if (myStart == null || otherStart == null) {
            return null;
        }
        Raid laterRaid = this;
        long newStartTime = otherStart.getTime();
        if (myStart.compareTo(otherStart) < 0) {
            newStartTime = myStart.getTime();
            laterRaid = otherRaid;
        }
        newStartTime -= 1000;
        
        Date newStart = new Date(newStartTime);
        String newID = FormatConstants.FORMAT_ID.format(newStart);
        Date newEnd = laterRaid.getEndTime();
        File newSaveFile = new File(Persistence.INSTANCES_DIR + 
                File.separator + 
                Persistence.SAVE_RAID_DIR + 
                File.separator +
                newID + 
                Persistence.XML_EXTENSION);
        
        Map myBossKills = getBossNameToKillDateMap();
        Map otherBossKills = otherRaid.getBossNameToKillDateMap();
        Map newBossKills = new HashMap(otherBossKills);
        for (Iterator iter=myBossKills.keySet().iterator(); iter.hasNext();) {
            String myBossName = (String) iter.next();
            if (!newBossKills.containsKey(myBossName)) {
                newBossKills.put(myBossName, myBossKills.get(myBossName));
            }
        }
        
        List myInstances = getInstanceNames();
        List otherInstances = otherRaid.getInstanceNames();
        List newInstances = new ArrayList(otherInstances);
        for (Iterator iter=myInstances.iterator(); iter.hasNext();) {
            String myInstance = (String) iter.next();
            if (!newInstances.contains(myInstance)) {
                newInstances.add(myInstance);
            }
        }
        
        PlayerData[] myPlayerData = getPlayerData();
        PlayerData[] otherPlayerData = otherRaid.getPlayerData();
        List newPlayerData = new ArrayList();
        for (int i=0; i < myPlayerData.length; i++) {
            PlayerData newDatum = new PlayerData();
            newDatum.merge(myPlayerData[i]);
            newPlayerData.add(newDatum);
        }
        // Merge players from the other raid into this raid.
        for (Iterator iter=newPlayerData.iterator(); iter.hasNext();) {
            PlayerData newDatum = (PlayerData) iter.next();
            for (int otherCount=0; otherCount < otherPlayerData.length; otherCount++) {
                PlayerData otherDatum = otherPlayerData[otherCount];
                if (newDatum.getPlayerName().equals(otherDatum.getPlayerName())) {
                    newDatum.merge(otherDatum);
                    break;
                }
            }
        }
        // Add players from the other raid that do not exist in this raid.
        for (int otherCount=0; otherCount < otherPlayerData.length; otherCount++) {
            PlayerData otherDatum = otherPlayerData[otherCount];
            boolean isFound = false;
            for (Iterator iter=newPlayerData.iterator(); iter.hasNext();) {
                PlayerData newDatum = (PlayerData) iter.next();
                if (otherDatum.getPlayerName().equals(newDatum.getPlayerName())) {
                    isFound = true;
                    break;
                }
            }
            if (!isFound) {
                PlayerData newDatum = new PlayerData();
                newDatum.merge(otherDatum);
                newPlayerData.add(newDatum);
            }
        }
        
        Raid newRaid = new Raid(
                newID, 
                newSaveFile, 
                newStart, 
                newEnd, 
                newBossKills, 
                newInstances, 
                (PlayerData[]) newPlayerData.toArray(new PlayerData[newPlayerData.size()]));
        
        return newRaid;
    }
}
