package org.wow.autodkp.model;

import java.io.File;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.wow.autodkp.model.graph.LuaGraphGenerator;
import org.wow.autodkp.model.graph.LuaGraphItem;
import org.wow.autodkp.model.graph.RaidGraphItem;
import org.wow.autodkp.persistence.AlternatesManager;
import org.wow.autodkp.persistence.Persistence;
import org.wow.autodkp.persistence.xml.XMLRaidReader;


/**
 * 
 * 
 * @author jsandusky
 */
public class RaidLookup {
    
    public List initialize(String raidFile, ModelLoadStatus loadStatus) {
        LuaGraphItem raidRoot = null;
        if (!raidFile.equals(Persistence.IGNORE_RT_FILE)) {
            // If we have not disconnected AutoDKP from Raid Tracker, read the log.
            LuaGraphItem graphRoot = null;
            try {
                graphRoot = new LuaGraphGenerator().generateGraph(new File(raidFile));
            } catch (Throwable t) {
                Logger.log(t);
                loadStatus.setThrowable(t);
                loadStatus.setRaidsLoaded(false);
                return null;
            }
            if (graphRoot != null) {
                LuaGraphItem[] children = graphRoot.getChildren();
                if (children != null && children.length > 0) {
                    LuaGraphItem raidInfo = children[0];
                    if (raidInfo.getType() == LuaGraphItem.RAID_ROOT) {
                        raidRoot = raidInfo;
                    }
                }
            }
            if (raidRoot == null) {
                loadStatus.setRaidsLoaded(false);
                return null;
            }
        }
        
        // Compile a list of both all graph items and saved items.
        // If a saved item exists for the same graph item, the saved item
        // takes the place of the graph item.
        Map raidIDToSourceMap = new HashMap();
        if (raidRoot != null) {
            LuaGraphItem[] graphChildren = raidRoot.getChildren();
            if (graphChildren != null) {
                for (int i=0; i < graphChildren.length; i++) {
                    RaidGraphItem raidGraphItem = (RaidGraphItem) graphChildren[i];
                    // generate: start date from graph
                    Date startDate = generateDate(raidGraphItem, LuaGraphItem.KEY_ITEM);
                    raidGraphItem.setStartDate(startDate);
                    String raidID = FormatConstants.FORMAT_ID.format(startDate);
                    raidIDToSourceMap.put(raidID, raidGraphItem);
                }
            }
        }
        
        File raidDir = new File(Persistence.INSTANCES_DIR + File.separator + Persistence.SAVE_RAID_DIR);
        if (raidDir.exists()) {
            File[] saveFiles = raidDir.listFiles();
            for (int i=0; i < saveFiles.length; i++) {
                String raidID = saveFiles[i].getName();
                if (raidID.endsWith(Persistence.XML_EXTENSION)) {
                    raidID = raidID.substring(0, raidID.indexOf(Persistence.XML_EXTENSION));
                }
                raidIDToSourceMap.put(raidID, saveFiles[i]);
            }
        } else {
            raidDir.mkdirs();
        }
        
        loadStatus.setRaidsLoaded(true);
        return loadRaids(raidIDToSourceMap, loadStatus);
    }
    
    private Date generateDate(LuaGraphItem graphItem, byte itemID) {
        LuaGraphItem dateItem = graphItem.getItem(itemID);
        if (dateItem != null) {
            try {
                Date date = FormatConstants.DATE_PARSE_FORMAT.parse((String) dateItem.getValue());
                return date;
            } catch (ParseException e) {
                Logger.log(e);
            }
        }
        return null;
    }
    
    private Map generateBossNameToKillDateMap(LuaGraphItem graphItem) {
        Map bossNameToKillDateMap = new HashMap();
        LuaGraphItem bossKillsItem = graphItem.getItem(LuaGraphItem.BOSS_KILLS_GROUP);
        if (bossKillsItem != null) {
            LuaGraphItem[] bossChildren = bossKillsItem.getChildren();
            if (bossChildren != null) {
                for (int i=0; i < bossChildren.length; i++) {
                    LuaGraphItem child = bossChildren[i];
                    // There can be two different types of boss graph items.
                    if (child.getType() == LuaGraphItem.BOSS_KILLS_BOSS) {
                        Date date = parseDate((String) child.getValue());
                        if (date != null) {
                            bossNameToKillDateMap.put(child.getName(), date);
                        }
                    } else if (child.getType() == LuaGraphItem.BOSS_KILLS_ITEM) {
                        LuaGraphItem bossItem = child.getItem(LuaGraphItem.BOSS_KILLS_ITEM_BOSS);
                        LuaGraphItem dateItem = child.getItem(LuaGraphItem.BOSS_KILLS_ITEM_TIME);
                        if (dateItem.getValue() == null) {
                            // Possible corrupted boss kill date, attempt to recover by assuming
                            // that the boss kill name and date info is inside the dateItem.
                            // This may have been a CT Raid Tracker bug.
//                            ["BossKills"] = {
//                                    [1] = {
//                                        ["boss"] = 1,
//                                        ["time"] = {
//                                            ["boss"] = "High Priestess Jeklik",
//                                            ["time"] = "05/29/06 19:25:59",
//                                            ["attendees"] = {
//                                            },
                            bossItem = dateItem.getItem(LuaGraphItem.BOSS_KILLS_ITEM_BOSS);
                            dateItem = dateItem.getItem(LuaGraphItem.BOSS_KILLS_ITEM_TIME);
                            if (bossItem != null && dateItem != null) {
                                Date date = parseDate((String) dateItem.getValue());
                                if (date != null) {
                                    bossNameToKillDateMap.put(bossItem.getValue(), date);
                                }
                            }
                        } else {
                            Date date = parseDate((String) dateItem.getValue());
                            if (date != null) {
                                bossNameToKillDateMap.put(bossItem.getValue(), date);
                            }
                        }
                    }
                }
            }
        }
        return bossNameToKillDateMap;
    }
    
    private Date parseDate(String value) {
        try {
            Date date = FormatConstants.DATE_PARSE_FORMAT.parse(value);
            return date;
        } catch (ParseException e) {
            Logger.log(e);
        }
        return null;
    }
    
    private List generateInstanceNames(LuaGraphItem graphItem) {
        List instanceNames = new ArrayList();
        
        LuaGraphItem raidZoneItem = graphItem.getItem(LuaGraphItem.ZONE_ITEM);
        if (raidZoneItem != null) {
            String instanceName = (String) raidZoneItem.getValue();
            instanceNames.add(instanceName);
        }
        
        LuaGraphItem lootGroup = graphItem.getItem(LuaGraphItem.LOOT_GROUP);
        if (lootGroup != null) {
            LuaGraphItem[] lootChildren = lootGroup.getChildren();
            if (lootChildren != null) {
                for (int i=0; i < lootChildren.length; i++) {
                    LuaGraphItem lootItem = lootChildren[i];
                    LuaGraphItem zoneItem = lootItem.getItem(LuaGraphItem.LOOT_ITEM_ZONE);
                    if (lootItem != null && zoneItem != null) {
                        String instanceName = (String) zoneItem.getValue();
                        if (!instanceNames.contains(instanceName)) {
                            instanceNames.add(instanceName);
                        }
                    }
                }
            }
        }
        
        return instanceNames;
    }
    
    private Raid loadRaidFromGraphItem(String raidID, RaidGraphItem raidGraphItem) {
        // generate: end date from graph
        Date endDate = generateDate(raidGraphItem, LuaGraphItem.RAID_END_ITEM);
        raidGraphItem.setEndDate(endDate);
        
        // generate: boss kill date map from graph
        Map bossKeyToKillDateMap = generateBossNameToKillDateMap(raidGraphItem);
        
        // generate: instances
        List instanceNames = generateInstanceNames(raidGraphItem);
        
        // generate: player data from graph
        PlayerDataGenerator generator = new PlayerDataGenerator();
        PlayerData[] playerData = generator.generatePlayerData(raidGraphItem);
        
        File saveFile = new File(Persistence.INSTANCES_DIR + 
                                 File.separator + 
                                 Persistence.SAVE_RAID_DIR + 
                                 File.separator +
                                 raidID + 
                                 Persistence.XML_EXTENSION);
        Raid raid = new Raid(raidID, 
                         saveFile, 
                         raidGraphItem.getStartDate(), 
                         endDate, 
                         bossKeyToKillDateMap, 
                         instanceNames, 
                         playerData);
        
        AlternatesManager.mergeAlternates(raid);

        return raid;
    }
    
    private List loadRaids(Map raidIDToSourceMap, ModelLoadStatus loadStatus) {
        List raids = new ArrayList();
        
        List invalidRaids = new ArrayList();
        for (Iterator iter=raidIDToSourceMap.keySet().iterator(); iter.hasNext();) {
            String raidID = (String) iter.next();
            try {
                Object source = raidIDToSourceMap.get(raidID);
                if (source instanceof RaidGraphItem) {
                    Raid raid = loadRaidFromGraphItem(raidID, (RaidGraphItem) source);
                    raids.add(raid);
                    
                } else if (source instanceof File) {
                    File saveFile = (File) source;
                    XMLRaidReader raidReader = new XMLRaidReader();
                    Raid raid = raidReader.readXML(raidID, saveFile);
                    AlternatesManager.mergeAlternates(raid);
                    raids.add(raid);
                }
                
            } catch (Throwable t) {
                Logger.log(t);
                invalidRaids.add(raidID);
            }
        }
        
        if (invalidRaids.size() > 0) {
            loadStatus.setInvalidRaids(invalidRaids);
        }
        
        if (raids.size() >= 2) {
            Collections.sort(raids, new Comparator() {
                public int compare(Object arg0, Object arg1) {
                    Raid raid0 = (Raid) arg0;
                    Raid raid1 = (Raid) arg1;
                    return raid1.getStartTime().compareTo(raid0.getStartTime());
                }
            });
        }
        return raids;
    }
}
