package org.wow.autodkp.model.graph;




/**
 * Represents a name/value pairing from a .lua variables file.
 * For the sake of memory conservation, the only type of information
 * that is saved is pertinent raid info.
 * 
 * .lua saved variable files from CT Raid Tracker can grow to be
 * enormous (10Mb+). Because AutoDKP keeps data from ALL raids
 * cached in memory, it is very important to reduce the memory footprint
 * of this class.
 * 
 * TODO consider keeping data for only 1 raid in memory.
 * 
 * @author jsandusky
 */
public class LuaGraphItem {
    
    public static final byte TYPE_NONE               = -1;
    public static final byte RAID_ITEM               =  0;
    public static final byte KEY_ITEM                =  1;
    public static final byte ZONE_ITEM               =  2;
    public static final byte PLAYER_INFOS_GROUP      =  3;
    public static final byte PLAYER_INFOS_PLAYER     =  4;
    public static final byte PLAYER_INFOS_CLASS      =  5;
    public static final byte LEAVE_GROUP             =  6;
    public static final byte LEAVE_GROUP_ITEM        =  7;
    public static final byte LEAVE_GROUP_ITEM_PLAYER =  8;
    public static final byte LEAVE_GROUP_ITEM_TIME   =  9;
    public static final byte BOSS_KILLS_GROUP        = 10;
    public static final byte BOSS_KILLS_BOSS         = 11;
    public static final byte JOIN_GROUP              = 12;
    public static final byte JOIN_GROUP_ITEM         = 13;
    public static final byte JOIN_GROUP_ITEM_PLAYER  = 14;
    public static final byte JOIN_GROUP_ITEM_TIME    = 15;
    public static final byte LOOT_GROUP              = 16;
    public static final byte LOOT_ITEM               = 17;
    public static final byte LOOT_ITEM_BOSS          = 18;
    public static final byte LOOT_ITEM_ZONE          = 19;
    public static final byte LOOT_ITEM_TIME          = 20;
    public static final byte LOOT_ITEM_PLAYER        = 21;
    public static final byte LOOT_ITEM_DETAIL        = 22;
    public static final byte LOOT_ITEM_DETAIL_COLOR  = 23;
    public static final byte LOOT_ITEM_DETAIL_ID     = 24;
    public static final byte LOOT_ITEM_DETAIL_NAME   = 25;
    public static final byte RAID_END_ITEM           = 26;
    public static final byte RAID_ROOT               = 27;
    public static final byte BOSS_KILLS_ITEM         = 28;
    public static final byte BOSS_KILLS_ITEM_BOSS    = 29;
    public static final byte BOSS_KILLS_ITEM_TIME    = 30;
    public static final byte LOOT_ITEM_COSTS         = 31;
    public static final byte VERSION                 = 32;
    
    public  static final String RAID_LOG_KEY         = "CT_RaidTracker_RaidLog";
    public  static final String RAID_TRACKER_VERSION = "CT_RaidTracker_Version";
    private static final String GRAPH_ID_JOIN        = "[\"Join\"]";
    private static final String GRAPH_ID_LEAVE       = "[\"Leave\"]";
    private static final String GRAPH_ID_LOOT        = "[\"Loot\"]";
    private static final String GRAPH_ID_LOOT_ID     = "[\"id\"]";
    private static final String GRAPH_ID_LOOT_NAME   = "[\"name\"]";
    private static final String GRAPH_ID_LOOT_COLOR  = "[\"c\"]";
    private static final String GRAPH_ID_PLAYER      = "[\"player\"]";
    private static final String GRAPH_ID_ITEM        = "[\"item\"]";
    private static final String GRAPH_ID_TIME        = "[\"time\"]";
    private static final String GRAPH_ID_INSTANCE    = "[\"zone\"]";
    private static final String GRAPH_ID_BOSS        = "[\"boss\"]";
    private static final String GRAPH_ID_PLAYERINFOS = "[\"PlayerInfos\"]";
    private static final String GRAPH_ID_CLASS       = "[\"class\"]";
    private static final String GRAPH_ID_START       = "[\"key\"]";
    private static final String GRAPH_ID_END         = "[\"End\"]";
    private static final String GRAPH_ID_BOSSKILLS   = "[\"BossKills\"]";
    private static final String GRAPH_ID_COSTS       = "[\"costs\"]";
    
    private static final String BRACKET_START = "[\"";
    private static final String BRACKET_END   = "\"]";
    
    protected byte mType = TYPE_NONE;
    private LuaGraphItem mParent;
    private String mName;
    private Object mValue;
    private boolean mIsLeaf;
    private LuaGraphItem[] mChildren;

    
    public LuaGraphItem(LuaGraphItem parent, String name, Object value, boolean isLeaf) {
        mParent = parent;
        mName = name;
        mValue = value;
        mIsLeaf = isLeaf;
    }

    public void resolveType() {
        if (mName.equals(RAID_LOG_KEY)) {
            mType = RAID_ROOT;
            mName = null;
            return;
        } else if (mName.equals(RAID_TRACKER_VERSION)) {
            mType = VERSION;
            mName = null;
            return;
        }
        
        if (mParent != null) {
            if        (mParent.getType() == RAID_ITEM) {
                if        (mName.equals(GRAPH_ID_PLAYERINFOS)) {
                    mType = PLAYER_INFOS_GROUP;
                } else if (mName.equals(GRAPH_ID_INSTANCE)) {
                    mType = ZONE_ITEM;
                } else if (mName.equals(GRAPH_ID_LEAVE)) {
                    mType = LEAVE_GROUP;
                } else if (mName.equals(GRAPH_ID_JOIN)) {
                    mType = JOIN_GROUP;
                } else if (mName.equals(GRAPH_ID_BOSSKILLS)) {
                    mType = BOSS_KILLS_GROUP;
                } else if (mName.equals(GRAPH_ID_START)) {
                    mType = KEY_ITEM;
                } else if (mName.equals(GRAPH_ID_LOOT)) {
                    mType = LOOT_GROUP;
                } else if (mName.equals(GRAPH_ID_END)) {
                    mType = RAID_END_ITEM;
                }
            } else if (mParent.getType() == PLAYER_INFOS_GROUP) {
                mType = PLAYER_INFOS_PLAYER;
            } else if (mParent.getType() == PLAYER_INFOS_PLAYER) {
                if        (mName.equals(GRAPH_ID_CLASS)) {
                    mType = PLAYER_INFOS_CLASS;
                }
            } else if (mParent.getType() == LEAVE_GROUP) {
                mType = LEAVE_GROUP_ITEM;
            } else if (mParent.getType() == LEAVE_GROUP_ITEM) {
                if        (mName.equals(GRAPH_ID_PLAYER)) {
                    mType = LEAVE_GROUP_ITEM_PLAYER;
                } else if (mName.equals(GRAPH_ID_TIME)) {
                    mType = LEAVE_GROUP_ITEM_TIME;
                }
            } else if (mParent.getType() == JOIN_GROUP) {
                mType = JOIN_GROUP_ITEM;
            } else if (mParent.getType() == JOIN_GROUP_ITEM) {
                if        (mName.equals(GRAPH_ID_PLAYER)) {
                    mType = JOIN_GROUP_ITEM_PLAYER;
                } else if (mName.equals(GRAPH_ID_TIME)) {
                    mType = JOIN_GROUP_ITEM_TIME;
                }
            } else if (mParent.getType() == BOSS_KILLS_GROUP) {
                mType = BOSS_KILLS_BOSS;
            } else if (mParent.getType() == BOSS_KILLS_BOSS) {
                // Add backwards support for the alternative way 
                // CT Raid Tracker logs boss info.
                mParent.mType = BOSS_KILLS_ITEM;
                mParent.mName = null;
                if        (mName.equals(GRAPH_ID_BOSS)) {
                    mType = BOSS_KILLS_ITEM_BOSS;
                } else if (mName.equals(GRAPH_ID_TIME)) {
                    mType = BOSS_KILLS_ITEM_TIME;
                }
            } else if (mParent.getType() == BOSS_KILLS_ITEM) {
                if        (mName.equals(GRAPH_ID_BOSS)) {
                    mType = BOSS_KILLS_ITEM_BOSS;
                } else if (mName.equals(GRAPH_ID_TIME)) {
                    mType = BOSS_KILLS_ITEM_TIME;
                }
            } else if (mParent.getType() == BOSS_KILLS_ITEM_TIME) {
                // Add support for a corrupted way that CT Raid Tracker
                // might contain boss kill info:
//              ["BossKills"] = {
//              [1] = {
//                  ["boss"] = 1,
//                  ["time"] = {
//                      ["boss"] = "High Priestess Jeklik",
//                      ["time"] = "05/29/06 19:25:59",
//                      ["attendees"] = {
//                      },
                if        (mName.equals(GRAPH_ID_BOSS)) {
                    mType = BOSS_KILLS_ITEM_BOSS;
                } else if (mName.equals(GRAPH_ID_TIME)) {
                    mType = BOSS_KILLS_ITEM_TIME;
                }
            } else if (mParent.getType() == LOOT_GROUP) {
                mType = LOOT_ITEM;
            } else if (mParent.getType() == LOOT_ITEM) {
                if        (mName.equals(GRAPH_ID_INSTANCE)) {
                    mType = LOOT_ITEM_ZONE;
                } else if (mName.equals(GRAPH_ID_PLAYER)) {
                    mType = LOOT_ITEM_PLAYER;
                } else if (mName.equals(GRAPH_ID_BOSS)) {
                    mType = LOOT_ITEM_BOSS;
                } else if (mName.equals(GRAPH_ID_TIME)) {
                    mType = LOOT_ITEM_TIME;
                } else if (mName.equals(GRAPH_ID_ITEM)) {
                    mType = LOOT_ITEM_DETAIL;
                } else if (mName.equals(GRAPH_ID_COSTS)) {
                    mType = LOOT_ITEM_COSTS;
                }
            } else if (mParent.getType() == LOOT_ITEM_DETAIL) {
                if        (mName.equals(GRAPH_ID_LOOT_COLOR)) {
                    mType = LOOT_ITEM_DETAIL_COLOR;
                } else if (mName.equals(GRAPH_ID_LOOT_ID)) {
                    mType = LOOT_ITEM_DETAIL_ID;
                } else if (mName.equals(GRAPH_ID_LOOT_NAME)) {
                    mType = LOOT_ITEM_DETAIL_NAME;
                }
            }
        }
        
        if (
                mType == PLAYER_INFOS_PLAYER ||
                mType == BOSS_KILLS_BOSS) {
            String actualName = mName;
            if (actualName.startsWith(BRACKET_START)) {
                actualName = actualName.substring(2);
            }
            if (actualName.endsWith(BRACKET_END)) {
                actualName = actualName.substring(0, actualName.length() - 2);
            }
            mName = actualName;

        } else {
            // usually, we don't need to store the name, so save on memory
            // because CT_RaidTracker.lua files can be HUGE
            mName = null;
        }
    }
    
    public LuaGraphItem getItem(byte type) {
        if (mChildren != null) {
            for (int i=0; i < mChildren.length; i++) {
                LuaGraphItem child = mChildren[i];
                if (child.getType() == type) {
                    return child;
                }
            }
        }
        return null;
    }
    
    public void addItem(LuaGraphItem item) {
        mIsLeaf = false;
        if (mChildren == null) {
            mChildren = new LuaGraphItem[1];
            mChildren[0] = item;
        } else {
            int newSize = mChildren.length + 1;
            LuaGraphItem[] newChildren = new LuaGraphItem[newSize];
            int i = 0;
            while (i < mChildren.length) {
                newChildren[i] = mChildren[i];
                i++;
            }
            newChildren[i] = item;
            mChildren = newChildren;
        }
    }

    public String getName() {
        return mName;
    }

    public Object getValue() {
        return mValue;
    }

    public boolean isLeaf() {
        return mIsLeaf;
    }
    
    public LuaGraphItem[] getChildren() {
        return mChildren;
    }

    public LuaGraphItem getParent() {
        return mParent;
    }
    
    public byte getType() {
        return mType;
    }
    
    public boolean isData() {
        return mType != TYPE_NONE;
    }
}
