package org.wow.autodkp.model.points;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.wow.autodkp.model.Cost;
import org.wow.autodkp.model.LootConstants;
import org.wow.autodkp.model.PlayerData;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.model.points.dkp.DefaultDKPCalculator;
import org.wow.autodkp.model.points.dkp.IDKPCalculator;
import org.wow.autodkp.persistence.AlternatesManager;
import org.wow.autodkp.persistence.IItemDB;
import org.wow.autodkp.persistence.ItemPointsDB;
import org.wow.autodkp.persistence.Persistence;
import org.wow.autodkp.persistence.xml.XMLRaidConstraintsReader;
import org.wow.autodkp.persistence.xml.XMLRaidConstraintsWriter;


/**
 * A point system within AutoDKP.
 * Contains the logic necessary to automatically calculate points
 * for the specified raid.
 * 
 * @author jsandusky
 */
public class DefaultPointSystem implements IPointSystem {

    private IPointSystemRaidData mCurrentRaidData;
    private IItemDB mItemDB;
    private IDKPCalculator mCalculator;
    private String mID;
    private boolean mIsAuctionedLootValues;
    
    // convenience arrays to contain version of
    // filtered raid player data
    private PlayerData[] mNonRemovedPlayerData;
    private PlayerData[] mNonRemovedRaidMemberPlayerData;
    
    // set to allow direct edits by default
    private boolean mIsDirectLootEditing = true;
    
    protected String mName;
    
    private boolean mDKPType_bossKill;
    private boolean mDKPType_zeroSum;
    private boolean mDKPType_zeroSumExcludeLooter;
    private boolean mDKPType_timeBased;

    protected float mStandardBossPoints;
    
    protected List mLootConstraintKeys  = new ArrayList();
    protected List mLootConstraintCosts = new ArrayList();
    protected List mLootConstraintNames = new ArrayList();
    
    protected List mBossConstraintCosts = new ArrayList();
    protected List mBossConstraintNames = new ArrayList();
    
    private float mDurationPoints;
    private int mDurationMinutes;
    private TimeThreshold mThreshold;
    private List mRollCalls;

    private LinkedHashMap mAdjustmentIDToNameMap = new LinkedHashMap();
    
    
    // This constructor called upon persistence, 
    // (loading a previously saved system).
    public DefaultPointSystem() {
        mLootConstraintKeys.add(LootConstants.LOOT_CONSTRAINT_KEY_NONE);
        mLootConstraintCosts.add(null);
        mLootConstraintNames.add(LootConstants.LOOT_CONSTRAINT_NAME_NONE);
        
        mBossConstraintCosts.add(null);
        mBossConstraintNames.add(LootConstants.BOSS_NAME_NONE);
    }
    
    // This constructor called upon manual creation of a new system.
    public DefaultPointSystem(String id, String name) {
        this();
        setID(id);
        setName(name);
        setBossKill(true);
        addRollCall(new RollCall(RollCall.RAID_START_NAME, 0.0f, 0));
        addRollCall(new RollCall(RollCall.RAID_END_NAME,   0.0f, 0));
    }

    
    public LinkedHashMap getAdjustmentMap() {
        return mAdjustmentIDToNameMap;
    }

    public IPointSystemRaidData getCurrentRaidData() {
        return mCurrentRaidData;
    }
    
    public String getID() {
        return mID;
    }

    public void setID(String id) {
        mID = id;
    }
    
    public IItemDB getItemDB() {
        if (mItemDB == null) {
            mItemDB = new ItemPointsDB(this);
        }
        return mItemDB;
    }

    public String getName() {
        return mName;
    }

    public void setName(String name) {
        mName = name;
    }
    
    public void importRaid(Raid raidToImport, 
                           IPointSystemRaidData importedRaidData, 
                           Raid currentRaid) {
        
        // Remove non-raid members that may still be in the raid,
        // i.e. from a previously linked raid.
        removeNonRaidMembers(currentRaid, getCurrentRaidData().getPointSystemPlayerDataMap());

        List importedPoints = new ArrayList();
        
        // First, build a list of all applicable data from the previous raid,
        // these are the carryovers into the new raid.
        PlayerData[] importedPlayerData = raidToImport.getPlayerData();
        Map importedConstraintsPlayerData = importedRaidData.getPointSystemPlayerDataMap();
        for (int i=0; i < importedPlayerData.length; i++) {
            String playerName = importedPlayerData[i].getPlayerName();
            String playerClass = importedPlayerData[i].getPlayerClass();
            IPointSystemPlayerData importedData = 
                (IPointSystemPlayerData) importedConstraintsPlayerData.get(playerName);
            if (importedData != null) {
                Carryover carryover = new Carryover(
                        playerName, 
                        playerClass, 
                        importedData.getOutgoingTotal());
                importedPoints.add(carryover);
            }
        }
        
        // If the player is in this raid, update our data.
        // Otherwise the player is carried over.
        List newPlayerData = new ArrayList();
        IPointSystemRaidData currentRaidData = getCurrentRaidData();
        for (Iterator iter=importedPoints.iterator(); iter.hasNext();) {
            Carryover carryover = (Carryover) iter.next();
            String playerName  = carryover.getPlayerName();
            IPointSystemPlayerData myData = 
                (IPointSystemPlayerData) currentRaidData.getPointSystemPlayerDataMap().get(playerName);
            if (myData != null) {
                // player from imported raid is in this raid
                myData.setIncomingTotal(carryover.getPlayerPoints());
            } else {
                // Include the imported player's points so that any
                // raid that imports this raid has the player's information.
                // We update the point system player data here.
                IPointSystemPlayerData newData = new DefaultPointSystemPlayerData();
                newData.setPlayerName(carryover.getPlayerName());
                newData.setPlayerClass(carryover.getPlayerClass());
                newData.setIncomingTotal(carryover.getPlayerPoints());
                newData.setOutgoingTotal(carryover.getPlayerPoints());
                newData.setFormula(getCalculator().generateFormula(null));
                currentRaidData.getPointSystemPlayerDataMap().put(carryover.getPlayerName(), newData);
                PlayerData playerData = new PlayerData(
                        carryover.getPlayerName(), 
                        carryover.getPlayerClass());
                newPlayerData.add(playerData);
            }
        }
        
        // Some players were carried over, which means we need to update 
        // the raid player data in addition to the point system player data.
        // We update the raid player data here.
        currentRaid.addPlayerData(newPlayerData);
        
        currentRaidData.setImportedRaidID(raidToImport.getID());
        
        saveRaid(currentRaid);
    }

    public void loadFromRaidData(Raid raid) {
        mCurrentRaidData = loadConstraintsForRaid(
                this, 
                raid,
                getAdjustmentMap());
        
        // create filtered player data
        PlayerData basePlayerData[] = raid.getPlayerData();
        List nonRemovedPlayerData = new ArrayList();
        List nonRemovedRaidMemberPlayerData = new ArrayList();
        Map constraintsPlayerData = getCurrentRaidData().getPointSystemPlayerDataMap();
        for (int i = 0; i < basePlayerData.length; i++) {
            PlayerData iterData = basePlayerData[i];
            IPointSystemPlayerData constraintData = 
                (IPointSystemPlayerData) constraintsPlayerData.get(iterData.getPlayerName());
            if (constraintData != null) {
                nonRemovedPlayerData.add(iterData);
                if (!iterData.isNonRaidMember()) {
                    nonRemovedRaidMemberPlayerData.add(iterData);
                }
            }
        }
        Collections.sort(nonRemovedRaidMemberPlayerData);
        
        mNonRemovedPlayerData = (PlayerData[]) nonRemovedPlayerData.toArray(
                new PlayerData[nonRemovedPlayerData.size()]);
        mNonRemovedRaidMemberPlayerData = (PlayerData[]) nonRemovedRaidMemberPlayerData.toArray(
                new PlayerData[nonRemovedRaidMemberPlayerData.size()]);
        
        getCalculator().loadFromRaidData(raid);
        getCalculator().calculate();
    }
    
    // Returns an array of player data that contains only players that
    // have not been removed from points.
    public PlayerData[] getNonRemovedPlayerData() {
        return mNonRemovedPlayerData;
    }
    
    // Returns an array of player data that contains only players that
    // are actually in the raid and who have not been removed from points.
    public PlayerData[] getNonRemovedRaidMemberPlayerData() {
        return mNonRemovedRaidMemberPlayerData;
    }
    
    private static String getSaveDirectoryPathForRaidID(String id) {
        return 
            Persistence.INSTANCES_DIR + 
            File.separator + 
            Persistence.CONSTRAINTS_DIR + 
            File.separator + 
            id;
    }
    
    private static File getSaveFileForRaid(String path, Raid raid) {
        return new File(
                path + 
                File.separator + 
                raid.getID() + 
                Persistence.XML_EXTENSION);
    }
    
    /**
     * Loads a saved raid constraints file.
     * Pass in a null calculator when loading an raid to be imported into the current raid.
     */
    public static IPointSystemRaidData loadConstraintsForRaid(IPointSystem pointSystem, 
                                                              Raid raid,
                                                              Map adjustmentMap) {
        IPointSystemRaidData raidConstraints = null;
        
        String saveDirectoryPath = getSaveDirectoryPathForRaidID(pointSystem.getID());
        File saveFile = getSaveFileForRaid(saveDirectoryPath, raid);
        if (saveFile.exists()) {
            XMLRaidConstraintsReader reader = new XMLRaidConstraintsReader();
            raidConstraints = reader.readXML(pointSystem.getCalculator(), saveFile);
        }
        
        if (raidConstraints == null) {
            raidConstraints = new DefaultPointSystemRaidData();
        }
        initializeConstraintData(
                pointSystem, 
                raid, 
                raidConstraints,
                adjustmentMap);
        
        mergeAlternates(raidConstraints);        
        
        return raidConstraints;
    }
    
    private static void mergeAlternates(IPointSystemRaidData raidData) {
        Map playerData = raidData.getPointSystemPlayerDataMap();
        List playerKeys = new ArrayList(playerData.keySet());
        for (Iterator iter=playerKeys.iterator(); iter.hasNext();) {
            String iterName = (String) iter.next();
            IPointSystemPlayerData iterData = (IPointSystemPlayerData) playerData.get(iterName);
            if (AlternatesManager.getDefault().isAlternate(iterName)) {
                String primaryName = AlternatesManager.getDefault().getPrimaryForAlternate(iterName);
                IPointSystemPlayerData primaryData = (IPointSystemPlayerData) playerData.get(primaryName);
                if (primaryData != null) {
                    // must merge
                    primaryData.merge(iterData);
                    playerData.remove(iterName);
                } else {
                    // just change key name
                    playerData.remove(iterName);
                    playerData.put(primaryName, iterData);
                }
            }
        }
    }
    
    private static void removeNonRaidMembers(Raid raid, Map playerConstraints) {
        // Remove any player that is not a member of the raid.
        List newRaidData = new ArrayList();
        PlayerData raidData[] = raid.getPlayerData();
        for (int i = 0; i < raidData.length; i++) {
            PlayerData iterData = raidData[i];
            if (!iterData.isNonRaidMember()) {
                newRaidData.add(iterData);
            } else {
                if (playerConstraints != null) {
                    playerConstraints.remove(iterData.getPlayerName());
                }
            }
        }
        raid.setPlayerData((PlayerData[]) newRaidData.toArray(new PlayerData[newRaidData.size()]));
    }
    
    private static void initializeConstraintData(IPointSystem pointSystem, 
                                                 Raid raid, 
                                                 IPointSystemRaidData raidData,
                                                 Map adjustmentMap) {

        Map playerConstraints = raidData.getPointSystemPlayerDataMap();
        
        if (playerConstraints.size() == 0) {
            // Fill empty/missing raid constraints with default values.
            removeNonRaidMembers(raid, null);
            PlayerData playerData[] = raid.getPlayerData();
            for (int i = 0; i < playerData.length; i++) {
                PlayerData iterDatum = playerData[i];
                String playerName = iterDatum.getPlayerName();
                IPointSystemPlayerData constraintDatum = 
                    (IPointSystemPlayerData) playerConstraints.get(playerName);
                if (constraintDatum == null) {
                    constraintDatum = new DefaultPointSystemPlayerData();
                    constraintDatum.setPlayerName(playerName);
                    constraintDatum.setPlayerClass(iterDatum.getPlayerClass());
                    constraintDatum.setIncomingTotal(0);
                    IDKPCalculator calculator = pointSystem.getCalculator();
                    if (calculator != null) {
                        constraintDatum.setFormula(calculator.generateFormula(null));
                    }
                    constraintDatum.setOutgoingTotal(0);
                    
                    if (adjustmentMap != null && !iterDatum.isNonRaidMember()) {
                        for (Iterator iter=adjustmentMap.keySet().iterator(); iter.hasNext();) {
                            String adjustmentID = (String) iter.next();
                            Adjustment adjustment = (Adjustment) adjustmentMap.get(adjustmentID);
                            if (adjustment.getInitialValue() > 0) {
                                constraintDatum.setAdjustment(adjustmentID, adjustment.getInitialValue());
                            }
                        }
                    }
                    
                    playerConstraints.put(playerName, constraintDatum);
                }
            }
            
        } else {
            // Add non-raid members to the raid if any exist.
            // Non-raid members are those who exist in the points file but
            // do not exist in the raid file. This can happen if we have 
            // carried over people from previous raids who are not in the current raid.
            List newPlayerData = new ArrayList();
            PlayerData playerData[] = raid.getPlayerData();
            Map raidPlayerMap = new HashMap();
            for (int i=0; i < playerData.length; i++) {
                raidPlayerMap.put(playerData[i].getPlayerName(), playerData[i].getPlayerClass());
            }
            for (Iterator iter=playerConstraints.keySet().iterator(); iter.hasNext();) {
                String playerName = (String) iter.next();
                IPointSystemPlayerData iterPlayerDatum = 
                    (IPointSystemPlayerData) playerConstraints.get(playerName);
                if (!raidPlayerMap.containsKey(playerName)) {
                    // Non-raid member.
                    PlayerData newDatum = new PlayerData(
                            playerName, 
                            iterPlayerDatum.getPlayerClass());
                    newPlayerData.add(newDatum);
                } else {
                    // Raid member, fix player class.
                    // Legacy point files may have an empty player class field.
                    if (
                            iterPlayerDatum.getPlayerClass() == null ||
                            iterPlayerDatum.getPlayerClass().length() == 0) {
                        String playerClass = (String) raidPlayerMap.get(playerName);
                        iterPlayerDatum.setPlayerClass(playerClass);
                    }
                }
            }
            raid.addPlayerData(newPlayerData);
        }
        
        if (pointSystem.isTimeBased()) {
            // Initialize roll calls.
            Map rollCallDateMap = raidData.getRollCallDateMap();
            for (Iterator iter=pointSystem.getRollCalls().iterator(); iter.hasNext();) {
                RollCall rollCall = (RollCall) iter.next();
                Date date = (Date) rollCallDateMap.get(rollCall.getName());
                rollCallDateMap.put(rollCall.getName(), date);
            }
        }
    }
    
    public boolean isTimeBased() {
        return mDKPType_timeBased;
    }
    
    public boolean isBossKill() {
        return mDKPType_bossKill;
    }
    
    public boolean isZeroSum() {
        return mDKPType_zeroSum;
    }
    
    public boolean isZeroSumExcludeLooter() {
        return mDKPType_zeroSumExcludeLooter;
    }
    
    public void setBossKill(boolean isBossKill) {
        mDKPType_bossKill = isBossKill;
    }
    
    public void setZeroSum(boolean isZeroSum, boolean isExcludeLooter) {
        mDKPType_zeroSum = isZeroSum;
        mDKPType_zeroSumExcludeLooter = isExcludeLooter;
    }

    public void setTimeBased(boolean isTimeBased) {
        mDKPType_timeBased = isTimeBased;
    }
    
    public IDKPCalculator getCalculator() {
        if (mCalculator == null) {
            mCalculator = new DefaultDKPCalculator(this);
        }
        return mCalculator;
    }

    public float getStandardPointsForBoss() {
        return mStandardBossPoints;
    }
    
    public void setStandardPointsForBoss(float points) {
        mStandardBossPoints = points;
    }
    
    public float getCustomPointsForBoss(String bossName) {
        int index = mBossConstraintNames.indexOf(bossName);
        Cost cost = (Cost) mBossConstraintCosts.get(index);
        if (cost != null) {
            return cost.getCost();
        }
        return 0;
    }

    public List getLootConstraintCosts() {
        return mLootConstraintCosts;
    }

    public List getLootConstraintNames() {
        return mLootConstraintNames;
    }
    
    public boolean isPointsStarted(Raid raid) {
        String saveDirectoryPath = getSaveDirectoryPathForRaidID(getID());
        File saveFile = getSaveFileForRaid(saveDirectoryPath, raid);
        return saveFile.exists();
    }
    
    public void deleteRaid(Raid raid) {
        String saveDirectoryPath = getSaveDirectoryPathForRaidID(getID());
        File saveFile = getSaveFileForRaid(saveDirectoryPath, raid);
        if (saveFile.exists()) {
            saveFile.delete();
        }
    }
    
    public void saveRaid(Raid raid) {
        String saveDirectoryPath = getSaveDirectoryPathForRaidID(getID());
        File saveDirectory = new File(saveDirectoryPath);
        if (!saveDirectory.exists()) {
            saveDirectory.mkdirs();
        }
        
        File saveFile = getSaveFileForRaid(saveDirectoryPath, raid);
        new XMLRaidConstraintsWriter().writeXML(saveFile, mCurrentRaidData);
    }
    
    public List getBossConstraintCosts() {
        return mBossConstraintCosts;
    }
    
    public List getBossConstraintNames() {
        return mBossConstraintNames;
    }
    
    public void removeBossConstraint(String name) {
        int index = mBossConstraintNames.indexOf(name);
        mBossConstraintNames.remove(index);
        mBossConstraintCosts.remove(index);
    }
    
    public void addBossConstraint(String name, Cost cost) {
        mBossConstraintNames.add(name);
        mBossConstraintCosts.add(cost);
    }
    
    public void setBossConstraintCost(String name, Cost cost) {
        int index = mBossConstraintNames.indexOf(name);
        mBossConstraintCosts.set(index, cost);
    }
    
    public void removeLootCategory(String id) {
        int index = mLootConstraintKeys.indexOf(id);
        mLootConstraintKeys.remove(index);
        mLootConstraintNames.remove(index);
        mLootConstraintCosts.remove(index);
    }
    
    public void addLootCategory(String id, String name, Cost cost) {
        mLootConstraintKeys.add(id);
        mLootConstraintNames.add(name);
        mLootConstraintCosts.add(cost);
    }
    
    public void setLootCategory(String id, String name, Cost cost) {
        int index = mLootConstraintKeys.indexOf(id);
        mLootConstraintNames.set(index, name);
        mLootConstraintCosts.set(index, cost);
    }
    
    public List getLootConstraintKeys() {
        return mLootConstraintKeys;
    }
    
    public String getLootConstraintKey(int index) {
        List keys = getLootConstraintKeys();
        return (String) keys.get(index);
    }
    
    public int getLootConstraintIndex(String key) {
        List keys = getLootConstraintKeys();
        int index = keys.indexOf(key);
        if (index < 0) {
            return LootConstants.LOOT_CONSTRAINT_NONE;
        }
        return index;
    }
    
    public String toString() {
        return mName;
    }
    
    public boolean isDirectLootValueEditing() {
        return mIsDirectLootEditing;
    }
    
    public void setDirectLootValueEditing(boolean isDirect) {
        mIsDirectLootEditing = isDirect;
    }
    
    public boolean isAuctionedLootValues() {
        return mIsAuctionedLootValues;
    }
    
    public void setAuctionedLootValues(boolean isAuctioned) {
        mIsAuctionedLootValues = isAuctioned;
    }
    
    public float getDurationPoints() {
        return mDurationPoints;
    }

    public void setDurationPoints(float points) {
        mDurationPoints = points;
    }
    
    public int getDurationMinutes() {
        return mDurationMinutes;
    }

    public void setDurationMinutes(int minutes) {
        mDurationMinutes = minutes;
    }
    
    public RollCall getRollCall(String name) {
        if (mRollCalls != null) {
            for (Iterator iter=mRollCalls.iterator(); iter.hasNext();) {
                RollCall iterRollCall = (RollCall) iter.next();
                if (iterRollCall.getName().equals(name)) {
                    return iterRollCall;
                }
            }
        }
        return null;
    }
    
    public void addRollCall(RollCall rollCall) {
        if (mRollCalls == null) {
            mRollCalls = new ArrayList();
        }
        mRollCalls.add(rollCall);
    }
    
    public void addRollCall(int index, RollCall rollCall) {
        if (mRollCalls == null) {
            mRollCalls = new ArrayList();
        }
        mRollCalls.add(index, rollCall);
    }
    
    public void removeRollCall(String name) {
        RollCall rollCallToRemove = getRollCall(name);
        if (rollCallToRemove != null) {
            mRollCalls.remove(rollCallToRemove);
        }
    }
    
    public List getRollCalls() {
        return mRollCalls;
    }
    
    public TimeThreshold getTimeThreshold() {
        return mThreshold;
    }
    
    public void setTimeThreshold(TimeThreshold threshold) {
        mThreshold = threshold;
    }
}
