package org.wow.autodkp.model.points.dkp;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.wow.autodkp.model.Loot;
import org.wow.autodkp.model.PlayTime;
import org.wow.autodkp.model.PlayerData;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.model.points.CostItem;
import org.wow.autodkp.model.points.CustomFormulaItem;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.IPointSystemPlayerData;
import org.wow.autodkp.model.points.IPointSystemRaidData;
import org.wow.autodkp.model.points.RollCall;


/**
 * A calculator for DKP systems that knows how to determine a player's
 * points for a particular raid. Subclasses define DKP-specifics.
 * 
 * @author jsandusky
 */
public abstract class AbstractDKPCalculator implements IDKPCalculator {
    
    private Raid mCurrentRaid;
    protected IPointSystem mConstraints;

    
    public AbstractDKPCalculator(IPointSystem constraints) {
        mConstraints = constraints;
    }

    
    public abstract IFormula generateFormula(String s);

    protected abstract void fillCosts(PlayerData playerData, IFormula iformula, List list);
    
    
    private void resetFormulas(PlayerData playerData[], Map raidConstraintsData) {
        if (!mConstraints.getCurrentRaidData().isLocked()) {
            for (int i = 0; i < playerData.length; i++) {
                String playerName = playerData[i].getPlayerName();
                IPointSystemPlayerData constraintData = 
                    (IPointSystemPlayerData) raidConstraintsData.get(playerName);
                IFormula formula = constraintData.getFormula();
                formula.reset(constraintData.getIncomingTotal());
            }
        }
    }
    
    private void zeroSumTotal(PlayerData playerData[], Map raidConstraintsData) {
        if (!mConstraints.getCurrentRaidData().isLocked()) {
            for (int i = 0; i < playerData.length; i++) {
                String playerName = playerData[i].getPlayerName();
                IPointSystemPlayerData constraintData = 
                    (IPointSystemPlayerData) raidConstraintsData.get(playerName);
                
                float totalAdjustment = 0;
                for (Iterator iter = mConstraints.getAdjustmentMap().keySet().iterator(); iter.hasNext();) {
                    String adjustmentID = (String) iter.next();
                    totalAdjustment += constraintData.getAdjustment(adjustmentID);
                }
                
                IFormula formula = constraintData.getFormula();
                if (!playerData[i].isNonRaidMember()) {
                    formula.addLineItem(
                            formula.getDistributions(), 
                            IFormula.ID_ZERO_SUM,
                            IFormula.TYPE_NONE,
                            IFormula.ID_ZERO_SUM);
                }
                constraintData.setFormulaTotal(formula.getRunningTotal());
                constraintData.setOutgoingTotal(constraintData.getFormulaTotal() + totalAdjustment);
            }
        }
    }
    
    public void calculate() {
        if (mCurrentRaid != null) {
            PlayerData playerData[] = mConstraints.getNonRemovedPlayerData();
            Map raidConstraintsData = mConstraints.getCurrentRaidData().getPointSystemPlayerDataMap();
            
            // First reset all formulas.
            resetFormulas(playerData, raidConstraintsData);
            
            TimeDKPCalculation timeDKPCalculation = new TimeDKPCalculation();

            // Then calculate points for each player.
            for (int i = 0; i < playerData.length; i++) {
                String playerName = playerData[i].getPlayerName();
                IPointSystemPlayerData constraintData = (IPointSystemPlayerData)raidConstraintsData.get(playerName);
                calculateForPlayer(playerData[i], constraintData, timeDKPCalculation);
            }
            
            // For Zero Sum, we calculate totals only after all player calculations.
            if (mConstraints.isZeroSum()) {
                zeroSumTotal(playerData, raidConstraintsData);
            }
        }
    }

    public void calculateLineItem(PlayerData playerData, IPointSystemPlayerData constraintData) {
        IFormula formula = constraintData.getFormula();
        formula.reset(constraintData.getIncomingTotal());
        TimeDKPCalculation timeDKPCalculation = new TimeDKPCalculation();
        calculateForPlayer(playerData, constraintData, timeDKPCalculation);
    }
    
    private void calculateForPlayer(
            PlayerData playerData, 
            IPointSystemPlayerData constraintData,
            TimeDKPCalculation timeDKPCalculation)
    {
        float totalAdjustment = 0;
        for (Iterator iter = mConstraints.getAdjustmentMap().keySet().iterator(); iter.hasNext();) {
            String adjustmentID = (String) iter.next();
            totalAdjustment += constraintData.getAdjustment(adjustmentID);
        }

        if (!mConstraints.getCurrentRaidData().isLocked()) {
            IFormula formula = constraintData.getFormula();
            calculateCosts(formula, constraintData, playerData);
            
            if (mConstraints.isBossKill()) {
                calculateBossPoints(formula, constraintData, playerData);
            }
            
            if (mConstraints.isTimeBased()) {
                calculateTimePoints(
                        formula, 
                        mConstraints.getCurrentRaidData(), 
                        constraintData,
                        playerData,
                        timeDKPCalculation);
            }

            // If not Zero Sum, we can calculate the final total at this point.
            if (!mConstraints.isZeroSum()) {
                constraintData.setFormulaTotal(formula.getRunningTotal());
                constraintData.setOutgoingTotal(constraintData.getFormulaTotal() + totalAdjustment);
            }
            
        } else {
            float total = constraintData.getFormulaTotal() + totalAdjustment;
            constraintData.setOutgoingTotal(total);
        }
    }

    public void loadFromRaidData(Raid raid) {
        mCurrentRaid = raid;
    }

    protected void calculateCosts(IFormula formula, IPointSystemPlayerData constraintData, PlayerData playerData) {
        List costItems = new ArrayList();
        
        // Add loots to the cost items list first
        List loots = playerData.getLoots();
        if (loots != null) {
            for (Iterator iter=loots.iterator(); iter.hasNext();) {
                Loot item = (Loot) iter.next();
                costItems.add(new CostItem(item));
            }
        }

        // Then merge in custom formula items based upon their saved indexes
        List customFormulaItems = constraintData.getCustomFormulaItems();
        if (customFormulaItems != null) {
            for (Iterator iter=customFormulaItems.iterator(); iter.hasNext();) {
                CustomFormulaItem item = (CustomFormulaItem) iter.next();
                int index = item.getIndex();
                if (index > costItems.size()) {
                    index = 0;
                }
                costItems.add(index, new CostItem(item));
            }
        }
        
        // Filter loot items that are excluded
        List alreadyExcludedIDs = new ArrayList();
        List excludedIDs = constraintData.getExcludedLootIDs();
        if (excludedIDs != null) {
            for (Iterator excludedIter = excludedIDs.iterator(); excludedIter.hasNext();) {
                String excludedLootID = (String)excludedIter.next();
                for (Iterator costItemsIter = costItems.iterator(); costItemsIter.hasNext();) {
                    CostItem costItem = (CostItem)costItemsIter.next();
                    if (alreadyExcludedIDs.contains(costItem)) {
                        // already excluded, skip to next
                        continue;
                    }
                    if (costItem.isLoot() && excludedLootID.equals(costItem.getLootID())) {
                        // exclude the loot from the cost
                        alreadyExcludedIDs.add(costItem);
                        costItem.setFiltered(true);
                        break;
                    }
                }
            }
        }
        
        formula.setCostItems(costItems);
        
        fillCosts(playerData, formula, costItems);
    }

    private void calculateBossPoints(IFormula formula, 
                                     IPointSystemPlayerData constraintData, 
                                     PlayerData playerData) {
        if (playerData.isNonRaidMember()) {
            // No boss points for non raid members.
            return;
        }
        
        Map bossKillDetailMap = new HashMap();
        List bossExceptions   = constraintData.getBossKillExceptions();
        List customBossPoints = mConstraints.getBossConstraintNames();
        float bossPoints = 0;
        for (Iterator iter = mCurrentRaid.getBossNameToKillDateMap().keySet().iterator(); iter.hasNext();) {
            String bossName = (String) iter.next();
            Date bossKillTime = (Date) mCurrentRaid.getBossNameToKillDateMap().get(bossName);
            PlayTime playTime = playerData.getPlayTime();
            boolean isIncluding = playTime.includes(bossKillTime);
            if (bossExceptions != null && bossExceptions.contains(bossName)) {
                // An exception exists for this boss, flip the bit.
                isIncluding = !isIncluding;
            }
            float iterPoints = 0;
            if (isIncluding) {
                if (customBossPoints.contains(bossName)) {
                    iterPoints = mConstraints.getCustomPointsForBoss(bossName);;
                } else {
                    iterPoints = mConstraints.getStandardPointsForBoss();
                }
                bossPoints += iterPoints;
            }
            bossKillDetailMap.put(bossName, new BossKillPointDetail(isIncluding, iterPoints));
        }

        formula.addLineItem(
                bossPoints, 
                IFormula.ID_BOSS_POINTS, 
                IFormula.TYPE_NONE, 
                IFormula.ID_BOSS_POINTS);
        
        formula.setBossKillDetailMap(bossKillDetailMap);
    }
    
    private void calculateTimePoints(IFormula formula,
                                     IPointSystemRaidData raidData,
                                     IPointSystemPlayerData constraintData,
                                     PlayerData playerData,
                                     TimeDKPCalculation dkp)
    {
        if (playerData.isNonRaidMember()) {
            // No time points for non raid members.
            return;
        }

        Map timeDetailMap = new HashMap();
        List timeExceptions = constraintData.getTimeExceptions();
        
        Date startDate = (Date) raidData.getRollCallDateMap().get(RollCall.RAID_START_NAME);
        Date endDate   = (Date) raidData.getRollCallDateMap().get(RollCall.RAID_END_NAME);
        if (startDate == null || endDate == null) {
            // No start and end times entered, cannot calculate time points.
            return;
        }
        if (!dkp.isInitialized()) {
            dkp.initialize(mConstraints, startDate, endDate);
        }

        PlayTime playTime = playerData.getPlayTime();
        int minutesInRaid = playTime.getMinutes(startDate, endDate);

        float totalTimePoints = 0.0f;
        
        // Calculate duration points.
        totalTimePoints = calculateDurationPoints(
                dkp, 
                playerData.getPlayerName(), 
                startDate, 
                endDate,
                minutesInRaid,
                timeDetailMap,
                timeExceptions);
        
        // Calculate roll call points.
        Map rollCallDateMap = raidData.getRollCallDateMap();
        for (Iterator iter=mConstraints.getRollCalls().iterator(); iter.hasNext();) {
            RollCall rollCall = (RollCall) iter.next();
            if (rollCall.getRecurrence() > 0) {
                for (
                        int minutes    = rollCall.getRecurrence(),
                            recurrence = 0;
                        minutes <= dkp.getRaidMinutes();
                        minutes += rollCall.getRecurrence(),
                        recurrence++)
                {
                    Date rollCallDate = new Date(startDate.getTime() + minutes * 60000);
                    totalTimePoints += calculateRollCallPoints(
                            rollCall, 
                            rollCallDate, 
                            playerData, 
                            timeDetailMap, 
                            timeExceptions,
                            recurrence);
                }
            } else {
                Date rollCallDate = (Date) rollCallDateMap.get(rollCall.getName());
                if (rollCallDate == null) {
                    continue;
                }
                totalTimePoints += calculateRollCallPoints(
                        rollCall, 
                        rollCallDate, 
                        playerData, 
                        timeDetailMap, 
                        timeExceptions,
                        -1);
            }
        }
        
        formula.addLineItem(totalTimePoints, IFormula.ID_TIME_POINTS, IFormula.TYPE_NONE, IFormula.ID_TIME_POINTS);

        formula.setTimeDetailMap(timeDetailMap);
    }
    
    private float calculateRollCallPoints(
            RollCall rollCall, 
            Date rollCallDate,
            PlayerData playerData,
            Map timeDetailMap,
            List timeExceptions,
            int recurrence)
    {
        String timeID = getTimeDetailID(
                playerData.getPlayerName(), 
                true,
                rollCall.getName(), 
                recurrence);
        boolean isIncluding = playerData.getPlayTime().includes(rollCallDate);
        if (timeExceptions != null && timeExceptions.contains(timeID)) {
            // An exception exists for this boss, flip the bit.
            isIncluding = !isIncluding;
        }
        float rollCallPoints = 0.0f;
        if (isIncluding) {
            rollCallPoints = rollCall.getPoints();
        }
        TimePointDetail rollCallDetail = new TimePointDetail(
                timeID,
                isIncluding, 
                rollCallPoints,
                true,
                rollCall.getName(),
                rollCallDate,
                recurrence);
        timeDetailMap.put(timeID, rollCallDetail);
        return rollCallPoints;
    }
    
    private float calculateDurationPoints(
            TimeDKPCalculation dkp, 
            String playerName, 
            Date startDate, 
            Date endDate,
            int minutesInRaid,
            Map timeDetailMap,
            List timeExceptions)
    {
        String timeID = getTimeDetailID(playerName, false, null, 0);
        boolean isIncluding = true;
        if (timeExceptions != null && timeExceptions.contains(timeID)) {
            // An exception exists for this boss, flip the bit.
            isIncluding = !isIncluding;
        }
        float durationPoints = 0.0f;
        if (isIncluding) {
            durationPoints = dkp.calculateDurationPoints(minutesInRaid);
        }
        timeDetailMap.put(timeID, new TimePointDetail(timeID, isIncluding, durationPoints));
        return durationPoints;
    }
    
    public String getTimeDetailID(
            String playerName, 
            boolean isRollCall, 
            String rollCallName, 
            int recurrence) {
        if (!isRollCall) {
            return "duration";
        } else {
            return rollCallName + ":" + recurrence;
        }
    }
}