package org.wow.autodkp.model.points.dkp;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.wow.autodkp.model.Cost;
import org.wow.autodkp.model.Loot;
import org.wow.autodkp.model.PlayTime;
import org.wow.autodkp.model.PlayerData;
import org.wow.autodkp.model.points.CostItem;
import org.wow.autodkp.model.points.CustomFormulaItem;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.IPointSystemPlayerData;


/**
 * Defines the calculations needed for each type of DKP.
 * 
 * @author jsandusky
 */
public class DefaultDKPCalculator extends AbstractDKPCalculator
{

    public DefaultDKPCalculator(IPointSystem constraints)
    {
        super(constraints);
    }
    
    public IFormula generateFormula(String str)
    {
        if (str == null) {
            return new DefaultDKPFormula(0);
        }
        else {
            return new DefaultDKPFormula(str);
        }
    }

    protected void fillCosts(PlayerData playerData, IFormula formula, List costItems)
    {
        List distributionItems = null;
        
        for (Iterator iter = costItems.iterator(); iter.hasNext();) {
            CostItem costItem = (CostItem)iter.next();
            if (!costItem.isFiltered()) {
                if (costItem.isLoot()) {
                    costItem.setCost(Loot.findLootCost(
                            playerData, 
                            mConstraints, 
                            costItem.getLoot()));
                }
                Cost cost = costItem.getCost();
                if (cost != null) {
                    String id = null;
                    String type = null;
                    String name = null;
                    if (costItem.isLoot()) {
                        id = costItem.getLootID();
                        type = costItem.getLoot().getColor();
                    } else {
                        CustomFormulaItem item = costItem.getFormulaItem();
                        id = IFormula.ID_CUSTOM;
                        type = IFormula.TYPE_NONE;
                        if (item.isLoot()) {
                            type = Loot.getColorStringFromColor(item.getLootColor());
                        }
                        name = item.getItemText();
                    }
                    
                    if (cost.isPercent()) {
                        float costRatio = cost.getCost() / 100F;
                        int roundedResult = Math.round(formula.getRunningTotal() * costRatio);
                        formula.addLineItem(-roundedResult, id, type, name);
                        
                    } else {
                        
                        // For Zero Sum, we must build a list of all loot that needs
                        // its cost value to be distributed among the raid.
                        if (mConstraints.isZeroSum()) {
                            if (distributionItems == null) {
                                distributionItems = new ArrayList();
                            }
                            // Record distribution points.
                            if (costItem.isLoot()) {
                                distributionItems.add(costItem);
                            } else {
                                // Cost is a custom cost, apply to distribution points
                                // only if the custom cost contains an item type.
                                CustomFormulaItem item = costItem.getFormulaItem();
                                if (item.getLootColor() != null) {
                                    distributionItems.add(costItem);
                                }
                            }
                        }
                        
                        // Charge the cost of the loot to the player.
                        formula.addLineItem(-cost.getCost(), id, type, name);
                    }
                }
            }
        }
        
        // For Zero Sum, award distribution points to the other players in the raid.
        if (mConstraints.isZeroSum()) {
            if (
                    distributionItems != null    &&
                    distributionItems.size() > 0 &&
                    !playerData.isNonRaidMember()) {
                zeroSum_DistributePoints(distributionItems, playerData);
            }
        }
    }
    
    private void zeroSum_DistributePoints(List distributionItems, PlayerData playerData)
    {
        Map raidConstraintsData = mConstraints.getCurrentRaidData().getPointSystemPlayerDataMap();
        String playerName = playerData.getPlayerName();
        for (Iterator iter=distributionItems.iterator(); iter.hasNext();) {
            CostItem costItem = (CostItem) iter.next();
            String distributionID = zeroSum_getDistributionID(costItem, playerName, playerData);
            float distributionPoints = costItem.getCost().getCost();
            PlayerData[] distributionTargets = zeroSum_getDistributionTargets(
                    costItem, 
                    playerName, 
                    playerData, 
                    raidConstraintsData, 
                    distributionID);
            if (distributionTargets.length > 0) {
                distributionPoints = distributionPoints / (float) distributionTargets.length;
            }
            for (int i = 0; i < distributionTargets.length; i++) {
                String iterName = distributionTargets[i].getPlayerName();
                IPointSystemPlayerData constraintData = 
                    (IPointSystemPlayerData) raidConstraintsData.get(iterName);
                IFormula iterFormula = constraintData.getFormula();
                iterFormula.updateDistribution(distributionPoints);
                Map distributionDetailMap = iterFormula.getDistributionDetailMap();
                DistributionPointDetail iterDetail = (DistributionPointDetail) distributionDetailMap.get(distributionID);
                iterDetail.setCurrentPoints(distributionPoints);
            }
        }
    }
    
    private PlayerData[] zeroSum_getDistributionTargets(CostItem costItem, 
                                                        String playerName, 
                                                        PlayerData playerData, 
                                                        Map raidConstraintsData,
                                                        String distributionID)
    {
        List targetPlayers = new ArrayList();
        Date timeLooted = null;
        if (costItem.isLoot()) {
            timeLooted = playerData.getLootTimeForLoot(costItem.getLoot());
        } else {
            CustomFormulaItem customItem = costItem.getFormulaItem();
            timeLooted = customItem.getLootTime();
        }
        if (timeLooted != null) {
            PlayerData raidPlayerData[] = mConstraints.getNonRemovedPlayerData();
            for (int i=0; i < raidPlayerData.length; i++) {
                PlayerData iterData = raidPlayerData[i];
                if (iterData.isNonRaidMember()) {
                    // No zero sum points for non-raid members.
                    continue;
                }
                String iterName = iterData.getPlayerName();
                if (!mConstraints.isZeroSumExcludeLooter() || !iterName.equals(playerName)) {
                    IPointSystemPlayerData constraintData = 
                        (IPointSystemPlayerData) raidConstraintsData.get(iterName);
                    List exceptions = constraintData.getDistributionExceptions();
                    PlayTime playTime = iterData.getPlayTime();
                    boolean isIncluded = playTime.includes(timeLooted);
                    if (exceptions != null && exceptions.contains(distributionID)) {
                        isIncluded = !isIncluded;
                    }
                    if (isIncluded) {
                        targetPlayers.add(iterData);
                    }
                    
                    // Update the point detail, for now enter in zero points because
                    // we do not have point information at this time.
                    Map distributionDetailMap = constraintData.getFormula().getDistributionDetailMap();
                    if (distributionDetailMap == null) {
                        distributionDetailMap = new HashMap();
                    }
                    distributionDetailMap.put(distributionID, 
                            new DistributionPointDetail(
                                    isIncluded, 
                                    0, 
                                    timeLooted, 
                                    costItem,
                                    playerName));
                }
            }
        }
        return (PlayerData[]) targetPlayers.toArray(new PlayerData[0]);
    }
    
    public static String zeroSum_getDistributionID(CostItem costItem, 
                                                   String player,
                                                   PlayerData playerData)
    {
        String id = null;
        if (costItem.isLoot()) {
            Loot loot = costItem.getLoot();
            Date lootTime = playerData.getLootTimeForLoot(loot);
            id = loot.getID() + ":" + player + ":" + lootTime.getTime();
        } else {
            id = zeroSum_getDistributionID(costItem.getFormulaItem(), player);
        }
        return id;
    }
    
    public static String zeroSum_getDistributionID(CustomFormulaItem item, String player)
    {
        return item.getItemText() + ":" + player + ":" + item.getIndex();
    }
}
