package org.wow.autodkp.model.points.dkp;

import java.util.Date;
import java.util.Iterator;
import java.util.Map;

import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.RollCall;
import org.wow.autodkp.model.points.TimeThreshold;


public class TimeDKPCalculation {
    
    private boolean mIsInitialized;
    private IPointSystem mPointSystem;
    private Date mStartDate;
    private Date mEndDate;
    private int mRaidMinutes;
    private float mMaximumStandardDurationPoints;
    private float mMaximumThresholdDurationPoints;
    private float mMaximumRollCallPoints;
    private float mMaximumPossiblePoints;
    
    private int mCutoffMinutes;
    private boolean mHasThreshold;
    
    
    public TimeDKPCalculation() {
    }
    
    
    private void setupThreshold() {
        TimeThreshold threshold = mPointSystem.getTimeThreshold();
        if (threshold == null || threshold.getValue() == 0) {
            mHasThreshold = false;
        } else {
            mHasThreshold = true;
            if (threshold.isPercent()) {
                mCutoffMinutes = Math.round(
                        (float) mRaidMinutes * ((float) threshold.getValue() / 100.0f));
            } else {
                mCutoffMinutes = threshold.getValue();
            }
        }
    }
    
    private int getFactor(float dividend, float divisor) {
        if (divisor == 0) {
            return 0;
        }
        // typecast to integer to chop off decimal portion
        return (int) (dividend / divisor);
    }
    
    public boolean isInitialized() {
        return mIsInitialized;
    }
    
    public void initialize(IPointSystem pointSystem, Date startDate, Date endDate) {
        mIsInitialized = true;
        mPointSystem = pointSystem;
        mStartDate = startDate;
        mEndDate = endDate;
        mRaidMinutes = (int) ((mEndDate.getTime() - mStartDate.getTime()) / 60000);
        setupThreshold();
    }
    
    public int getRaidMinutes() {
        return mRaidMinutes;
    }
    
    public float getMaximumPossiblePoints() {
        return mMaximumPossiblePoints;
    }
    
    public float getMaximumRollCallPoints() {
        return mMaximumRollCallPoints;
    }
    
    public float getMaximumStandardDurationPoints() {
        return mMaximumStandardDurationPoints;
    }
    
    public float getMaximumThresholdDurationPoints() {
        return mMaximumThresholdDurationPoints;
    }
    
    public boolean hasThreshold() {
        return mHasThreshold;
    }
    
    public void calculateMaximumPointsAvailable() {
        mMaximumStandardDurationPoints  = mHasThreshold ? 
                calculateDurationPoints(mCutoffMinutes - 1) :
                calculateDurationPoints(mRaidMinutes);
        mMaximumThresholdDurationPoints = mHasThreshold ?
                calculateDurationPoints(mRaidMinutes) :
                0;
        mMaximumRollCallPoints = calculateAllRollCalls();
        
        mMaximumPossiblePoints = 
            Math.max(mMaximumStandardDurationPoints, mMaximumThresholdDurationPoints) +
            mMaximumRollCallPoints;
    }
    
    public float calculateDurationPoints(int minutesInRaid) {
        if (mHasThreshold) {
            if (minutesInRaid < mCutoffMinutes) {
                int factor = getFactor(minutesInRaid, mPointSystem.getDurationMinutes());
                return factor * mPointSystem.getDurationPoints();
            } else {
                int factor = getFactor(minutesInRaid, mPointSystem.getTimeThreshold().getMinutes());
                return factor * mPointSystem.getTimeThreshold().getPoints();
            }
        } else {
            int factor = getFactor(minutesInRaid, mPointSystem.getDurationMinutes());
            return factor * mPointSystem.getDurationPoints();
        }
    }
    
    public float calculateAllRollCalls() {
        Map rollCallDateMap = 
            mPointSystem.getCurrentRaidData().getRollCallDateMap();
        float totalRollCallPoints = 0.0f;
        for (Iterator iter=mPointSystem.getRollCalls().iterator(); iter.hasNext();) {
            RollCall rollCall = (RollCall) iter.next();
            if (rollCall.getRecurrence() > 0) {
                int factor = getFactor(mRaidMinutes, rollCall.getRecurrence());
                totalRollCallPoints += factor * rollCall.getPoints();
            } else {
                Date date = (Date) rollCallDateMap.get(rollCall.getName());
                if (date == null) {
                    continue;
                }
                totalRollCallPoints += rollCall.getPoints();
            }
        }
        return totalRollCallPoints;
    }
}
