package org.wow.autodkp.persistence;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import org.wow.autodkp.model.AlternatesGroup;
import org.wow.autodkp.model.PlayerData;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.persistence.xml.XMLAlternatesReader;
import org.wow.autodkp.persistence.xml.XMLAlternatesWriter;


/**
 * Handles all the player mains-to-alternates character mappings.
 * 
 * @author jsandusky
 */
public class AlternatesManager {
    
    private static final File SAVE_FILE = new File(
            Persistence.DEFINITIONS_DIR + 
            File.separator + 
            Persistence.ALTERNATES_FILE);
    
    private Map mAllAlternatesGroupings = new TreeMap();
    private List mAlternatesCache = new ArrayList();
    private Map mAlternateToPrimaryCache = new HashMap();
    
    private static AlternatesManager mInstance = new AlternatesManager();
    
    private AlternatesManager() {
        // Load from bosses file
        if (SAVE_FILE.exists()) {
            mAllAlternatesGroupings = new XMLAlternatesReader().readXML(SAVE_FILE);
            for (Iterator groupIter=getAllAlternatesGroupings().iterator(); groupIter.hasNext();) {
                AlternatesGroup group = (AlternatesGroup) groupIter.next();
                String primary = group.getPrimaryPlayerName();
                List alternates = group.getAlternates();
                if (alternates != null) {
                    for (Iterator altIter=alternates.iterator(); altIter.hasNext();) {
                        String alternate = (String) altIter.next();
                        mAlternateToPrimaryCache.put(alternate, primary);
                        mAlternatesCache.add(alternate);
                    }
                }
            }
        }
    }
    
    public void save() {
        new XMLAlternatesWriter().writeXML(SAVE_FILE);
    }
    
    public static AlternatesManager getDefault() {
        return mInstance;
    }
    
    public Collection getAllAlternatesGroupings() {
        return mAllAlternatesGroupings.values();
    }
    
    public Set getAllPrimaryPlayers() {
        return mAllAlternatesGroupings.keySet();
    }
    
    public void addPrimaryPlayer(String primaryPlayerName, String primaryPlayerClass) {
        AlternatesGroup group = new AlternatesGroup();
        group.setPrimaryPlayerName(primaryPlayerName);
        group.setPrimaryPlayerClass(primaryPlayerClass);
        mAllAlternatesGroupings.put(primaryPlayerName, group);
    }
    
    public void removePrimaryPlayer(String primaryPlayerName) {
        AlternatesGroup group = (AlternatesGroup) mAllAlternatesGroupings.get(primaryPlayerName);
        if (group != null) {
            mAllAlternatesGroupings.remove(primaryPlayerName);
            List alternates = group.getAlternates();
            if (alternates != null) {
                mAlternatesCache.removeAll(alternates);
                for (Iterator altIter=alternates.iterator(); altIter.hasNext();) {
                    String alternate = (String) altIter.next();
                    mAlternateToPrimaryCache.remove(alternate);
                }
            }
        }
    }
    
    public void addAlternateForPlayer(String primaryPlayerName, String alternateName) {
        AlternatesGroup group = (AlternatesGroup) mAllAlternatesGroupings.get(primaryPlayerName);
        if (group != null) {
            group.addAlternate(alternateName);
            mAlternatesCache.add(alternateName);
            mAlternateToPrimaryCache.put(alternateName, primaryPlayerName);
        }
    }
    
    public void removeAlternateForPlayer(String primaryPlayerName, String alternateName) {
        AlternatesGroup group = (AlternatesGroup) mAllAlternatesGroupings.get(primaryPlayerName);
        if (group != null) {
            group.removeAlternate(alternateName);
            mAlternatesCache.remove(alternateName);
            mAlternateToPrimaryCache.remove(alternateName);
        }
    }
    
    public List getAlternatesForPlayer(String primaryPlayerName) {
        AlternatesGroup group = (AlternatesGroup) mAllAlternatesGroupings.get(primaryPlayerName);
        if (group != null) {
            return group.getAlternates();
        }
        return null;
    }
    
    public AlternatesGroup getAlternatesGroupForPlayer(String primaryPlayerName) {
        return (AlternatesGroup) mAllAlternatesGroupings.get(primaryPlayerName);
    }
    
    public String getPrimaryForAlternate(String alternateName) {
        return (String) mAlternateToPrimaryCache.get(alternateName);
    }
    
    public boolean isAlternate(String playerName) {
        return mAlternatesCache.contains(playerName);
    }
    
    
    public static void mergeAlternates(Raid raid) {
        Map mergedPlayerData = new HashMap();
        PlayerData[] playerData = raid.getPlayerData();
        for (int i=0; i < playerData.length; i++) {
            PlayerData iterData = playerData[i];
            String iterName = iterData.getPlayerName();
            if (AlternatesManager.getDefault().isAlternate(iterName)) {
                // alternate
                String primaryName = AlternatesManager.getDefault().getPrimaryForAlternate(iterName);
                AlternatesGroup group = AlternatesManager.getDefault().getAlternatesGroupForPlayer(primaryName);
                String primaryClass = group.getPrimaryPlayerClass();
                PlayerData primaryData = (PlayerData) mergedPlayerData.get(primaryName);
                if (primaryData != null) {
                    // primary data exists, merge into it
                    primaryData.merge(iterData, primaryName, primaryClass);
                } else {
                    // no primary data, add our data in the name of the primary
                    iterData.setPlayerName(primaryName);
                    iterData.setPlayerClass(primaryClass);
                    mergedPlayerData.put(primaryName, iterData);
                }
            } else {
                // primary
                String primaryName  = iterData.getPlayerName();
                String primaryClass = iterData.getPlayerClass();
                PlayerData primaryData = (PlayerData) mergedPlayerData.get(primaryName);
                if (primaryData != null) {
                    // primary data exists, merge into it
                    primaryData.merge(iterData, primaryName, primaryClass);
                } else {
                    // just add ourselves
                    mergedPlayerData.put(primaryName, iterData);
                }
            }
        }
        
        raid.setPlayerData((PlayerData[]) mergedPlayerData.values().toArray(new PlayerData[0]));
    }
}
