package org.wow.autodkp.persistence.xml;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.wow.autodkp.model.AutoDKPModel;
import org.wow.autodkp.model.Cost;
import org.wow.autodkp.model.Loot;
import org.wow.autodkp.model.LootConstants;
import org.wow.autodkp.model.PlayerData;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.persistence.ItemMasterDB;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;


/**
 * 
 * 
 * @author jsandusky
 */
public class XMLRaidReader {
    
    
    public Raid readXML(String id, File xmlFile) {
        
        RaidContentHandler handler = new RaidContentHandler();
        XMLUtilities.parse(xmlFile, handler);
        
        Raid raid = new Raid(
                id, 
                xmlFile, 
                handler.getStartDate(), 
                handler.getEndDate(), 
                handler.getBossKeyToKillDateMap(),
                handler.getInstanceNames(),
                handler.getPlayerData());
        
        return raid;
    }
    
    
    private class RaidContentHandler extends DefaultHandler {
        private StringBuffer mBuffer = new StringBuffer();
        private Date mStartDate;
        private Date mEndDate;
        private Map mBossKeyToKillDateMap = new HashMap();
        private List mPlayerData = new ArrayList();
        private List mInstanceKeys = new ArrayList();
        
        private String mCurrentBossKey;
        private Date mCurrentBossKillDate;
        private PlayerData mCurrentPlayerData;
        
        private String mCurrentLootID;
        private Date mCurrentLootDate;
        private Cost mCurrentAuctionCost;
        
        private String mCurrentLootName;
        private String mCurrentLootQuality;
        private String mCurrentLootInstance;
        private String mCurrentLootBoss;
        
        
        public void characters(char[] ch, int start, int length) throws SAXException {
            super.characters(ch, start, length);
            if (mBuffer == null) {
                mBuffer = new StringBuffer();
            }
            mBuffer.append(ch, start, length);
        }

        public void endElement(String uri, String localName, String qName) throws SAXException {
            super.endElement(uri, localName, qName);
            String str = null;
            if (mBuffer != null) {
                str = mBuffer.toString();
            }
            if (localName.equals(XMLRaidWriter.XML_RAID_DATE_START)) {
                mStartDate = XMLUtilities.parseDate(str);
            } else if (localName.equals(XMLRaidWriter.XML_RAID_DATE_END)) {
                mEndDate = XMLUtilities.parseDate(str);
            } else if (localName.equals(XMLUtilities.XML_RAID_DATA_PLAYER)) {
                if (
                        !mCurrentPlayerData.getPlayerName().equals(PlayerData.UNKNOWN_ENTITY) &&
                        !mCurrentPlayerData.getPlayerName().equals(PlayerData.UNKNOWN)) {
                    mPlayerData.add(mCurrentPlayerData);
                }
                mCurrentPlayerData = null;
            } else if (localName.equals(XMLUtilities.XML_PLAYER_NAME)) {
                mCurrentPlayerData.setPlayerName(str);
            } else if (localName.equals(XMLUtilities.XML_PLAYER_CLASS)) {
                mCurrentPlayerData.setPlayerClass(str);
            } else if (localName.equals(XMLRaidWriter.XML_PLAYER_JOIN_DATE)) {
                mCurrentPlayerData.addTimeJoin(XMLUtilities.parseDate(str));
            } else if (localName.equals(XMLRaidWriter.XML_PLAYER_LEAVE_DATE)) {
                mCurrentPlayerData.addTimeLeave(XMLUtilities.parseDate(str));
            } else if (localName.equals(XMLRaidWriter.XML_LOOT)) {
                if (mCurrentLootID != null && mCurrentLootDate != null) {
                    if (ItemMasterDB.getDefault().hasItem(mCurrentLootID)) {
                        Loot loot = AutoDKPModel.getInstance().createLoot(mCurrentLootID);
                        // If we had no boss or instance data, but we do now, try to add it in.
                        updateLoot(loot);
                        mCurrentPlayerData.addLoot(loot, mCurrentLootDate, mCurrentAuctionCost);
                    } else if (mCurrentLootName != null && mCurrentLootQuality != null) {
                        // Item not found in our database. Because raid files may store
                        // the full loot information, we can add the loot to our database.
                        int quality = LootConstants.getQualityValueFromKey(mCurrentLootQuality);
                        Loot loot = AutoDKPModel.getInstance().createLoot(
                                mCurrentLootID, 
                                mCurrentLootName, 
                                Loot.getColorStringFromQuality(quality),
                                mCurrentLootBoss,
                                mCurrentLootInstance);
                        mCurrentPlayerData.addLoot(loot, mCurrentLootDate, mCurrentAuctionCost);
                    }
                }
                mCurrentLootID = null;
                mCurrentLootDate = null;
                mCurrentAuctionCost = null;
                mCurrentLootName = null;
                mCurrentLootQuality = null;
                mCurrentLootBoss = null;
                mCurrentLootInstance = null;
            } else if (localName.equals(XMLRaidWriter.XML_LOOT_ID)) {
                mCurrentLootID = str;
            } else if (localName.equals(XMLRaidWriter.XML_LOOT_DATE)) {
                mCurrentLootDate = XMLUtilities.parseDate(str);
            } else if (localName.equals(XMLRaidWriter.XML_BOSS_NAME)) {
                mCurrentBossKey = str;
            } else if (localName.equals(XMLUtilities.XML_INSTANCES_NAME)) {
                mInstanceKeys.add(str);
            }  else if (localName.equals(XMLRaidWriter.XML_BOSS_KILL_DATE)) {
                mCurrentBossKillDate = XMLUtilities.parseDate(str);
            } else if (localName.equals(XMLRaidWriter.XML_RAID_DATA_BOSS)) {
                if (mCurrentBossKey != null && mCurrentBossKillDate != null) {
                    mBossKeyToKillDateMap.put(mCurrentBossKey, mCurrentBossKillDate);
                }
                mCurrentBossKey = null;
                mCurrentBossKillDate = null;
            } else if (localName.equals(XMLRaidWriter.XML_LOOT_AUCTION_VALUE)) {
                mCurrentAuctionCost.setCost(XMLUtilities.parseFloat(str));
            } else if (localName.equals(XMLRaidWriter.XML_LOOT_AUCTION_ISPERCENT)) {
                if (str.equals(XMLUtilities.IS_YES)) {
                    mCurrentAuctionCost.setIsPercent(true);
                } else if (str.equals(XMLUtilities.IS_NO)) {
                    mCurrentAuctionCost.setIsPercent(false);
                }
            } else if (localName.equals(XMLRaidWriter.XML_LOOT_NAME)) {
                mCurrentLootName = str;
            } else if (localName.equals(XMLRaidWriter.XML_LOOT_QUALITY)) {
                mCurrentLootQuality = str;
            } else if (localName.equals(XMLRaidWriter.XML_LOOT_INSTANCE)) {
                mCurrentLootInstance = str;
            } else if (localName.equals(XMLRaidWriter.XML_LOOT_BOSS)) {
                mCurrentLootBoss = str;
            }
        }

        public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
            super.startElement(uri, localName, qName, attributes);
            mBuffer = null;
            if (localName.equals(XMLUtilities.XML_RAID_DATA_PLAYER)) {
                mCurrentPlayerData = new PlayerData();
            } else if (localName.equals(XMLRaidWriter.XML_LOOT_AUCTION)) {
                mCurrentAuctionCost = new Cost();
            }
        }

        public Date getStartDate() {
            return mStartDate;
        }
        
        public Date getEndDate() {
            return mEndDate;
        }
        
        public Map getBossKeyToKillDateMap() {
            return mBossKeyToKillDateMap;
        }
        
        public List getInstanceNames() {
            return mInstanceKeys;
        }
        
        public PlayerData[] getPlayerData() {
            return (PlayerData[]) mPlayerData.toArray(new PlayerData[0]);
        }
        
        private void updateLoot(Loot loot) {
            String dbBoss = loot.getBossName();
            if (
                    (dbBoss == null || dbBoss.length() == 0) &&
                    (mCurrentLootBoss != null && mCurrentLootBoss.length() > 0)) {
                loot.setBossName(mCurrentLootBoss);
                ItemMasterDB.getDefault().setUpdateFlag(true);
                ItemMasterDB.getDefault().addLoot(loot);
            }
            String dbInstance = loot.getInstanceName();
            if (
                    (dbInstance == null || dbInstance.length() == 0) &&
                    (mCurrentLootInstance != null && mCurrentLootInstance.length() > 0)) {
                loot.setInstanceName(mCurrentLootInstance);
                ItemMasterDB.getDefault().setUpdateFlag(true);
                ItemMasterDB.getDefault().addLoot(loot);
            }
        }
    }
}
