package org.wow.autodkp.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Image;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JSplitPane;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;

import org.wow.autodkp.model.AutoDKPModel;
import org.wow.autodkp.model.FormatConstants;
import org.wow.autodkp.model.ModelLoadStatus;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.persistence.ItemMasterDB;
import org.wow.autodkp.view.actions.AboutAction;
import org.wow.autodkp.view.actions.AddNonRaidMemberAction;
import org.wow.autodkp.view.actions.DeleteSelectedRaid;
import org.wow.autodkp.view.actions.ExitAction;
import org.wow.autodkp.view.actions.ExportAction;
import org.wow.autodkp.view.actions.GenerateHTMLAction;
import org.wow.autodkp.view.actions.ImportAction;
import org.wow.autodkp.view.actions.ItemManagerAction;
import org.wow.autodkp.view.actions.MergeRaidsAction;
import org.wow.autodkp.view.actions.PreferencesAction;
import org.wow.autodkp.view.actions.PromoteAction;
import org.wow.autodkp.view.actions.RTFileAction;
import org.wow.autodkp.view.actions.ReloadAction;
import org.wow.autodkp.view.actions.RemovePlayerPointsAction;
import org.wow.autodkp.view.actions.StyleHTMLAction;
import org.wow.autodkp.view.items.ItemManagerWindow;
import org.wow.autodkp.view.skin.AutoDKPLookAndFeelSupport;
import org.wow.autodkp.view.summary.SummaryPane;
import org.wow.autodkp.view.table.players.PlayerTable;


/**
 * The main AutoDKP window.
 * 
 * @author jsandusky
 */
public class AutoDKPView {
    
    private static AutoDKPView mInstance;
    
    private JFrame mFrame;
    private PlayerTable mPlayerTable;
    private MainSelector mMainSelector;
    private GenerateHTMLAction mGenerateHTMLAction;
    private StyleHTMLAction mStyleHTMLAction;
    private ExitAction mExitAction;
    private ItemManagerAction mItemManagerAction;
    private AboutAction mAboutAction;
    private PromoteAction mPromoteRaidAction;
    private PreferencesAction mPreferencesAction;
    private RTFileAction mRaidTrackerAction;
    private RemovePlayerPointsAction mRemovePlayerPointsAction;
    private AddNonRaidMemberAction mAddNonRaidMemberAction;
    private DeleteSelectedRaid mDeleteRaidAction;
    private MergeRaidsAction mMergeRaidsAction;
    private ReloadAction mReloadAction;
    private ExportAction mExportAction;
    private ImportAction mImportAction;
    
    private JPanel mTopPane;
    private JPanel mQuickButtonPanel;
    
    
    public static AutoDKPView getInstance() {
        if (mInstance == null) {
            mInstance = new AutoDKPView();
        }
        return mInstance;
    }

    
    private AutoDKPView() {
        mFrame = new JFrame(AutoDKP.PROGRAM_NAME + " " + AutoDKP.PROGRAM_VERSION);
        
        Image windowIcon = ImageUtilities.loadJarImage("castle.gif");
        mFrame.setIconImage(windowIcon);
        
        mItemManagerAction = new ItemManagerAction(AutoDKPModel.getInstance(), new IChangeCommitListener() {
            public void changesCancelled() {
            }
            public void changesCommitted() {
                mPlayerTable.fireTableDataChanged();
            }
        });
        mExitAction = new ExitAction();
        mGenerateHTMLAction = new GenerateHTMLAction();
        mStyleHTMLAction = new StyleHTMLAction();
        mAboutAction = new AboutAction();
        mPromoteRaidAction = new PromoteAction();
        mPreferencesAction = new PreferencesAction();
        mRaidTrackerAction = new RTFileAction();
        mRemovePlayerPointsAction = new RemovePlayerPointsAction();
        mAddNonRaidMemberAction = new AddNonRaidMemberAction();
        mDeleteRaidAction = new DeleteSelectedRaid();
        mMergeRaidsAction = new MergeRaidsAction();
        mReloadAction = new ReloadAction();
        mExportAction = new ExportAction();
        mImportAction = new ImportAction();
        
        JMenu fileMenu = new JMenu("File");
        fileMenu.setMnemonic(KeyEvent.VK_F);
        addMenuItem(fileMenu, mItemManagerAction,  KeyEvent.VK_I);
        addMenuItem(fileMenu, mGenerateHTMLAction, KeyEvent.VK_H);
        fileMenu.addSeparator();
        fileMenu.add(mExitAction);
        
        JMenu editMenu = new JMenu("Edit");
        editMenu.setMnemonic(KeyEvent.VK_E);
        addMenuItem(editMenu, mImportAction,      KeyEvent.VK_I);
        addMenuItem(editMenu, mExportAction,      KeyEvent.VK_E);
        editMenu.addSeparator();
        addMenuItem(editMenu, mStyleHTMLAction,   KeyEvent.VK_S);
        addMenuItem(editMenu, mRaidTrackerAction, KeyEvent.VK_T);
        addMenuItem(editMenu, mPreferencesAction, KeyEvent.VK_P);
        
        JMenu raidMenu = new JMenu("Raid");
        raidMenu.setMnemonic(KeyEvent.VK_R);
        addMenuItem(raidMenu, mReloadAction,      KeyEvent.VK_L);
        raidMenu.addSeparator();
        addMenuItem(raidMenu, mPromoteRaidAction, KeyEvent.VK_P);
        addMenuItem(raidMenu, mDeleteRaidAction,  KeyEvent.VK_D);
        addMenuItem(raidMenu, mMergeRaidsAction,  KeyEvent.VK_M);
        raidMenu.addSeparator();
        addMenuItem(raidMenu, mAddNonRaidMemberAction,   KeyEvent.VK_A);
        addMenuItem(raidMenu, mRemovePlayerPointsAction, KeyEvent.VK_R);
        
        JMenu helpMenu = new JMenu("Help");
        helpMenu.setMnemonic(KeyEvent.VK_H);
        addMenuItem(helpMenu, mAboutAction, KeyEvent.VK_A);
        
        JMenuBar menuBar = new JMenuBar();
        menuBar.add(fileMenu);
        menuBar.add(editMenu);
        menuBar.add(raidMenu);
        menuBar.add(helpMenu);
        mFrame.setJMenuBar(menuBar);
        
        mFrame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        mFrame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                super.windowClosing(e);
                Set saveOnExitProperties = GeneralProperties.getDefault().getSaveOnExitProperties();
                for (Iterator iter=saveOnExitProperties.iterator(); iter.hasNext();) {
                    SaveOnExitProperty property = (SaveOnExitProperty) iter.next();
                    property.setProperty();
                }
                GeneralProperties.getDefault().save();
                mExitAction.exit();
            }
        });
        
        initialize();
        
        GeneralProperties props = GeneralProperties.getDefault();
        props.loadWindowRect(
                mFrame, 
                props.MAIN_WINDOW_X,
                props.MAIN_WINDOW_Y,
                props.MAIN_WINDOW_WIDTH,
                props.MAIN_WINDOW_HEIGHT,
                props.DEFAULT_MAIN_WINDOW_WIDTH,
                props.DEFAULT_MAIN_WINDOW_HEIGHT);
        
    }
    
    private void addMenuItem(JMenu menu, Action action, int mnemonic) {
        JMenuItem menuItem = menu.add(action);
        menuItem.setMnemonic(mnemonic);
    }
    
    
    public JFrame getFrame() {
        return mFrame;
    }
    
    public boolean removeSelectedPlayers() {
        return mMainSelector.removeSelectedPlayers();
    }
    
    public boolean addNonRaidMember() {
        return mMainSelector.addNonRaidMember();
    }
    
    public void reloadRaids() {
        mMainSelector.reloadRaids();
    }
    
    public String generateHTMLReport() {
        return mMainSelector.generateHTMLReport();
    }
    
    public Color getStylePanelColor() {
        return mQuickButtonPanel.getBackground();
    }
    
    public void setStylePanelColor(Color color) {
        mQuickButtonPanel.setBackground(color);
        mTopPane.setBackground(color);
    }
    
    private void initialize() {
        mPlayerTable = new PlayerTable();
        
        // Styled panes, these panels are colored.
        mQuickButtonPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 10, 0));
        mTopPane = new JPanel(new BorderLayout());

        JButton preferencesButton = new JButton(mPreferencesAction);
        preferencesButton.setBackground(AutoDKPLookAndFeelSupport.COLOR_TOOLBAR);
        preferencesButton.setText("");
        preferencesButton.setToolTipText("Open Preferences");
        
        JButton itemManagerButton = new JButton(mItemManagerAction);
        itemManagerButton.setBackground(AutoDKPLookAndFeelSupport.COLOR_TOOLBAR);
        itemManagerButton.setText("");
        itemManagerButton.setToolTipText("Open Item Manager");
        itemManagerButton.setPreferredSize(preferencesButton.getPreferredSize());

        JButton generateHTMLButton = new JButton(mGenerateHTMLAction);
        generateHTMLButton.setBackground(AutoDKPLookAndFeelSupport.COLOR_TOOLBAR);
        generateHTMLButton.setText("");
        generateHTMLButton.setToolTipText("Generate HTML for Selected Raid");
        generateHTMLButton.setPreferredSize(preferencesButton.getPreferredSize());
        
        JButton exportButton = new JButton(mExportAction);
        exportButton.setBackground(AutoDKPLookAndFeelSupport.COLOR_TOOLBAR);
        exportButton.setText("");
        exportButton.setToolTipText("Export Raids/Points");
        exportButton.setPreferredSize(preferencesButton.getPreferredSize());
        
        JButton importButton = new JButton(mImportAction);
        importButton.setBackground(AutoDKPLookAndFeelSupport.COLOR_TOOLBAR);
        importButton.setText("");
        importButton.setToolTipText("Import Raids/Points");
        importButton.setPreferredSize(preferencesButton.getPreferredSize());
        
        mQuickButtonPanel.add(itemManagerButton);
        mQuickButtonPanel.add(generateHTMLButton);
        mQuickButtonPanel.add(new JSeparator(JSeparator.HORIZONTAL));
        mQuickButtonPanel.add(preferencesButton);
        mQuickButtonPanel.add(new JSeparator(JSeparator.HORIZONTAL));
        mQuickButtonPanel.add(importButton);
        mQuickButtonPanel.add(exportButton);
        mQuickButtonPanel.setBorder(BorderFactory.createEmptyBorder(5, 0, 5, 0));
        
        GeneralProperties props = GeneralProperties.getDefault();
        Integer vertSplitProp = props.getIntegerProperty(props.MAIN_SPLIT);
        int vertSplit = props.DEFAULT_MAIN_SPLIT;
        if (vertSplitProp != null) {
            vertSplit = vertSplitProp.intValue();
        }
        
        SummaryPane constraintsPane = new SummaryPane(this, mPlayerTable);
        mMainSelector = new MainSelector(mPlayerTable, constraintsPane);

        JPanel controlsPane = new JPanel();
        controlsPane.setLayout(new BorderLayout());
        controlsPane.add(mQuickButtonPanel, BorderLayout.NORTH);
        controlsPane.add(mMainSelector.getComponent(),    BorderLayout.CENTER);
        controlsPane.add(mMainSelector.getTableControl(), BorderLayout.SOUTH);
        
        mTopPane.add(constraintsPane, BorderLayout.EAST);
        mTopPane.add(controlsPane, BorderLayout.CENTER);
        
        final JSplitPane mainSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        mainSplit.setDividerLocation(vertSplit);
        mainSplit.setTopComponent(mTopPane);
        mainSplit.setBottomComponent(mPlayerTable.getComponent());
        
        mFrame.getContentPane().add(mainSplit, BorderLayout.CENTER);
        
        props.addSaveOnExitProperty(new SaveOnExitProperty(props.MAIN_SPLIT) {
            public Object getValue() { return new Integer(mainSplit.getDividerLocation()); }
        });
    }
    
    public static boolean unloadReload() {
        if (mInstance != null) {
            if (mInstance.getFrame() != null) {
                mInstance.getFrame().setVisible(false);
                mInstance.getFrame().dispose();
            }
            mInstance = null;
        }
        
        GeneralProperties props = GeneralProperties.getDefault();
        String raidFile = props.getStringProperty(props.RAID_TRACKER_FILE);
        ModelLoadStatus loadStatus = AutoDKPModel.getInstance().load(raidFile);
        if (!loadStatus.isRaidsLoaded()) {
            Throwable throwable = loadStatus.getThrowable();
            if (
                    throwable != null && 
                    throwable.getMessage() != null)
            {
                JOptionPane.showMessageDialog(
                        null, 
                        "Error occurred while loading " + raidFile + "\n" +
                        throwable.toString(),
                        "Severe Error",
                        JOptionPane.ERROR_MESSAGE);
            } else {
                JOptionPane.showMessageDialog(
                        null, 
                        raidFile + " appears to be corrupted.",
                        "Severe Error",
                        JOptionPane.ERROR_MESSAGE);
            }
            RTFileAction.showDialog();
            return false;
        }
        
        List invalidRaids = loadStatus.getInvalidRaids();
        if (invalidRaids != null && invalidRaids.size() > 0) {
            StringBuffer buf = new StringBuffer();
            for (Iterator iter=invalidRaids.iterator(); iter.hasNext();) {
                String raidID = (String) iter.next();
                try {
                    Date date = FormatConstants.FORMAT_ID.parse(raidID);
                    String dateText = 
                        props.getDateFormatProperty(props.DATE_FORMAT_LONG).format(date);
                    buf.append(dateText + "\n");
                } catch (Exception e) {
                    buf.append(raidID + "\n");
                }
            }
            JOptionPane.showMessageDialog(
                    null, 
                    "The following raids appear to be corrupt or invalid:\n" +
                    buf.toString(),
                    "Severe Error",
                    JOptionPane.ERROR_MESSAGE);
        }
        
        // During reload, we could have encountered new loot information.
        if (ItemMasterDB.getDefault().isUpdateFlagSet()) {
            ItemMasterDB.getDefault().setUpdateFlag(false);
            ItemMasterDB.getDefault().save();
        }
        
        List missingLoot = loadStatus.getMissingLoot();
        IPointSystem[] pointSystems = AutoDKPModel.getInstance().getConstraints();
        if (missingLoot.size() > 0) {
            
            // We MUST save any new loot that is discovered.
            // Here we only save base item info.
            ItemMasterDB.getDefault().saveLoots(missingLoot, pointSystems);
            
            final ItemManagerWindow missingLootsWindow = new ItemManagerWindow(
                    pointSystems, 
                    missingLoot,
                    null,
                    "The following new items have been discovered. Please assign any costs if needed.",
                    true);
            
            getInstance().getFrame().setVisible(true);
            
            missingLootsWindow.setVisible(true);
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    missingLootsWindow.toFront();
                }
            });
        } else {
            getInstance().getFrame().setVisible(true);
        }
        
        return true;
    }
}
