package org.wow.autodkp.view.html;

import java.io.File;
import java.text.DateFormat;
import java.util.Date;
import java.util.Map;

import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import org.wow.autodkp.model.FormatConstants;
import org.wow.autodkp.model.PlayerData;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.RollCall;
import org.wow.autodkp.persistence.FileIOUtilities;
import org.wow.autodkp.persistence.Persistence;
import org.wow.autodkp.view.GeneralProperties;
import org.wow.autodkp.view.table.DateListRenderer;
import org.wow.autodkp.view.table.LootRenderer;
import org.wow.autodkp.view.table.TableSorter;
import org.wow.autodkp.view.table.players.FormulaEditor;
import org.wow.autodkp.view.table.players.FormulaPrintObject;
import org.wow.autodkp.view.table.players.FormulaUtilities;
import org.wow.autodkp.view.table.players.PlayerTableModel;


/**
 * Generates HTML from raid/point data.
 * 
 * @author jsandusky
 */
public class HTMLGenerator {
    
    public static final String NEWLINE = System.getProperty("line.separator");
    
    
    public static String generateHTMLReport(Raid raid,
                                            IPointSystem pointSystem,
                                            PlayerData[] raidPlayerData,
                                            TableColumnModel columnModel, 
                                            TableSorter sorter,
                                            LootRenderer lootRenderer)
    {
        int rows = sorter.getRowCount();
        int columns = sorter.getColumnCount();
        StringBuffer buf = new StringBuffer();
        buf.append("<HTML>" + NEWLINE);
        
        // insert stylesheet text
        if (StyleUtilities.INTERNAL_STYLE_SHEET != null) {
            buf.append("<HEAD><STYLE TYPE=\"text/css\">" + NEWLINE);
            buf.append(StyleUtilities.INTERNAL_STYLE_SHEET);
            buf.append("</STYLE></HEAD>" + NEWLINE);
            buf.append(NEWLINE + NEWLINE);
        }
        
        buf.append("<BODY>" + NEWLINE + NEWLINE);
        
        includeRaidDate(raid, pointSystem, buf);
        
        buf.append("<DIV CLASS=" + StyleUtilities.STYLE_GENERAL_TABLE + ">" + NEWLINE);
        
        buf.append("<P>" + NEWLINE);
        buf.append("<TABLE CLASS=" + StyleUtilities.STYLE_TABLE_MAIN + ">" + NEWLINE);
        
        buf.append(generateTableHeader(columns, columnModel));
        
        buf.append(NEWLINE + NEWLINE + NEWLINE);
        buf.append("<!-- PLAYER ROWS SECTION -->" + NEWLINE + NEWLINE);
        
        for (int row=0; row < rows; row++) {
            int modelIndex = sorter.modelIndex(row);
            PlayerData playerData = raidPlayerData[modelIndex];
            
            String playerName = playerData.getPlayerName();
            if (
                    playerName.equals(PlayerData.UNKNOWN_ENTITY) ||
                    playerName.equals(PlayerData.UNKNOWN)) {
                continue;
            }
            
            String playerClass = playerData.getPlayerClass().trim().toLowerCase().trim();
            buf.append("<TR CLASS=\"main-" + playerClass + "_row\">" + NEWLINE);
            for (int column=0; column < columns; column++) {
                if (column == PlayerTableModel.COLUMN_SEPARATOR) {
                    continue;
                }
                
                boolean isFormulaColumn = false;
                TableColumn tableColumn = columnModel.getColumn(column);
                if (tableColumn.getCellEditor() instanceof FormulaEditor) {
                    isFormulaColumn = true;
                }
                
                buf.append("<TD CLASS=");
                switch (column) {
                case PlayerTableModel.COLUMN_PLAYER:     buf.append(StyleUtilities.STYLE_TD_MAIN_PLAYER_NAME); break;
                case PlayerTableModel.COLUMN_CLASS:      buf.append(StyleUtilities.STYLE_TD_MAIN_PLAYER_CLASS); break;
                case PlayerTableModel.COLUMN_TIME_JOIN:  buf.append(StyleUtilities.STYLE_TD_MAIN_JOINS); break;
                case PlayerTableModel.COLUMN_TIME_LEAVE: buf.append(StyleUtilities.STYLE_TD_MAIN_LEAVES); break;
                case PlayerTableModel.COLUMN_LOOT:       buf.append(StyleUtilities.STYLE_TD_MAIN_LOOT); break;
                default:
                    if (column + 1 == columns) {
                        buf.append(StyleUtilities.STYLE_TD_MAIN_POINTS_TOTAL);
                    } else {
                        if (isFormulaColumn) {
                            buf.append(StyleUtilities.STYLE_TD_MAIN_FORMULA);
                        } else {
                            buf.append(StyleUtilities.STYLE_TD_MAIN_POINTS);
                        }
                    }
                    break;
                }
                buf.append(">");
                
                Object ob = sorter.getValueAt(row, column);
                
                String value = null;
                if        (column == PlayerTableModel.COLUMN_LOOT) {
                    value = lootRenderer.renderToHTML(ob, playerData);
                } else if (column == PlayerTableModel.COLUMN_TIME_JOIN) {
                    value = DateListRenderer.renderToHTML(ob);
                } else if (column == PlayerTableModel.COLUMN_TIME_LEAVE) {
                    value = DateListRenderer.renderToHTML(ob);
                } else if (ob instanceof Float) {
                    Float floatValue = (Float) ob;
                    value = FormatConstants.NUMBER_FORMAT.format(floatValue.doubleValue());
                } else if (isFormulaColumn) {
                    FormulaPrintObject printObject = (FormulaPrintObject) ob;
                    value = FormulaUtilities.generateHTMLString(printObject);
                } else {
                    if (ob == null) {
                        value = "";
                    } else {
                        value = ob.toString();
                    }
                }
                if (value == null || value.length() == 0) {
                    value = "&nbsp;";
                } else {
                    value = stripTags(value);
                    if (isFormulaColumn) {
                        // Formula strings need to have their styles stripped.
                        int endHeader = value.indexOf("</HEAD>");
                        if (endHeader >= 0) {
                            value = value.substring(endHeader + 7);
                        }
                    }
                }
                buf.append(value);
                buf.append("</TD>" + NEWLINE);
            }
            buf.append("</TR>" + NEWLINE + NEWLINE + NEWLINE);
        }
        buf.append(NEWLINE  + "</TABLE>" + NEWLINE);
        buf.append("</P>"   + NEWLINE);
        buf.append("</DIV>" + NEWLINE);
        
        
        includePointSystemNotes(pointSystem, buf);
        
        includeGeneralNotes(buf);
        
        
        buf.append(NEWLINE + "</BODY>"  + NEWLINE);
        buf.append(NEWLINE + "</HTML>"  + NEWLINE);
        return buf.toString();
    }
    
    
    private static String generateTableHeader(int columns, TableColumnModel columnModel) {
        StringBuffer buf = new StringBuffer();
        buf.append(NEWLINE + NEWLINE + NEWLINE);
        buf.append("<!-- COLUMN HEADER SECTION -->" + NEWLINE);
        buf.append("<TR>" + NEWLINE);
        for (int column=0; column < columns; column++) {
            if (column == PlayerTableModel.COLUMN_SEPARATOR) {
                continue;
            }
            TableColumn tableColumn = columnModel.getColumn(column);
            String value = (String) tableColumn.getHeaderValue();
            
            buf.append("<TH CLASS=");
            switch (column) {
            case 0: buf.append(StyleUtilities.STYLE_TH_MAIN_PLAYER_NAME); break;
            case 1: buf.append(StyleUtilities.STYLE_TH_MAIN_PLAYER_CLASS); break;
            case 2: buf.append(StyleUtilities.STYLE_TH_MAIN_JOINS); break;
            case 3: buf.append(StyleUtilities.STYLE_TH_MAIN_LEAVES); break;
            case 4: buf.append(StyleUtilities.STYLE_TH_MAIN_LOOT); break;
            default:
                if (tableColumn.getCellEditor() instanceof FormulaEditor) {
                    buf.append(StyleUtilities.STYLE_TH_MAIN_FORMULA);
                } else {
                    buf.append(StyleUtilities.STYLE_TH_MAIN_POINTS);
                }
                break;
            }
            buf.append(">");
            
            if (value == null || value.length() == 0) {
                value = "";
            } else {
                value = stripTags(value);
            }
            buf.append(value);
            buf.append("</TH>" + NEWLINE);
        }
        buf.append("</TR>" + NEWLINE);
        
        return buf.toString();
    }
    
    private static String stripTags(String str) {
        str = str.replaceAll("<HTML>", "");
        str = str.replaceAll("</HTML>", "");
        return str;
    }
    
    private static void includeRaidDate(Raid raid, IPointSystem system, StringBuffer buf) {
        GeneralProperties props = GeneralProperties.getDefault();
        boolean includeDate = props.getBooleanProperty(props.HTML_INCLUDE_RAID_DATE);
        if (includeDate) {
            DateFormat dateFormat = props.getDateFormatProperty(props.DATE_FORMAT_LONG);
            String dateStr = "Raid Created " + dateFormat.format(raid.getStartTime());
            if (system.isTimeBased()) {
                Map rollCallDateMap = system.getCurrentRaidData().getRollCallDateMap();
                if (rollCallDateMap != null) {
                    Date startTime = (Date) rollCallDateMap.get(RollCall.RAID_START_NAME);
                    Date endTime   = (Date) rollCallDateMap.get(RollCall.RAID_END_NAME);
                    DateFormat timeFormat = props.getDateFormatProperty(props.DATE_FORMAT_SHORT);
                    if (startTime != null && endTime != null) {
                        dateStr += 
                            " (" + 
                            timeFormat.format(startTime) + " to " + 
                            timeFormat.format(endTime) +
                            ")";
                    }
                }
            }
            buf.append(NEWLINE + NEWLINE + NEWLINE);
            buf.append("<!-- RAID CREATION DATE -->" + NEWLINE);
            buf.append("<DIV CLASS=" + StyleUtilities.STYLE_GENERAL_DATE + ">" + NEWLINE);
            buf.append("<P>" + dateStr + "</P>");
            buf.append(NEWLINE + "</DIV>" + NEWLINE);
            buf.append(NEWLINE);
            buf.append(NEWLINE);
        }
    }
    
    private static void includePointSystemNotes(IPointSystem system, StringBuffer buf) {
        String notesStr = null;
        File notesFile = new File(
                Persistence.DEFINITIONS_DIR + 
                File.separator + 
                Persistence.NOTES_DIR + 
                File.separator + 
                system.getID() + "_notes.txt");
        if (notesFile.exists()) {
            notesStr = FileIOUtilities.readFileToString(notesFile);
        }
        if (notesStr != null && notesStr.length() > 0) {
            buf.append(NEWLINE + NEWLINE + NEWLINE);
            buf.append("<!-- POINT SYSTEMS NOTES SECTION -->" + NEWLINE);
            buf.append("<DIV CLASS=" + StyleUtilities.STYLE_POINT_SYSTEM_NOTES + ">" + NEWLINE);
            buf.append(notesStr);
            buf.append(NEWLINE + "</DIV>" + NEWLINE);
            buf.append(NEWLINE);
            buf.append(NEWLINE);
        }
    }
    
    private static void includeGeneralNotes(StringBuffer buf) {
        String notesStr = null;
        File notesFile = new File(
                Persistence.DEFINITIONS_DIR + 
                File.separator + 
                Persistence.NOTES_DIR + 
                File.separator + 
                Persistence.HTML_NOTES_FILE);
        if (notesFile.exists()) {
            notesStr = FileIOUtilities.readFileToString(notesFile);
        }
        if (notesStr != null && notesStr.length() > 0) {
            buf.append(NEWLINE + NEWLINE + NEWLINE);
            buf.append("<!-- GENERAL NOTES SECTION -->" + NEWLINE);
            buf.append("<DIV CLASS=" + StyleUtilities.STYLE_GENERAL_NOTES + ">" + NEWLINE);
            buf.append(notesStr);
            buf.append(NEWLINE + "</DIV>" + NEWLINE);
            buf.append(NEWLINE);
            buf.append(NEWLINE);
        }
    }
}
