package org.wow.autodkp.view.items;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;

import org.wow.autodkp.model.Loot;
import org.wow.autodkp.model.LootConstants;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.persistence.BossManager;
import org.wow.autodkp.persistence.InstanceManager;
import org.wow.autodkp.persistence.ItemMasterDB;
import org.wow.autodkp.view.BannerWindow;
import org.wow.autodkp.view.GeneralProperties;
import org.wow.autodkp.view.IChangeCommitListener;
import org.wow.autodkp.view.SaveOnExitProperty;
import org.wow.autodkp.view.filters.BossItemFilter;
import org.wow.autodkp.view.filters.InstanceItemFilter;
import org.wow.autodkp.view.filters.ItemFilter;
import org.wow.autodkp.view.filters.NoFilter;
import org.wow.autodkp.view.filters.QualityItemFilter;
import org.wow.autodkp.view.table.ITableColumn;
import org.wow.autodkp.view.table.LootRenderer;


/**
 * The AutoDKP Item Manager.
 * All item values are entered either directly or in groups via
 * the Item Manager.
 * 
 * @author jsandusky
 */
public class ItemManagerWindow extends BannerWindow {
    
    private IPointSystem[] mFilteredConstraints;
    private String mStatusText;
    private List mLoots;
    private LootTableModel mTableModel;
    private JComboBox mItemFilter1;
    private JComboBox mItemFilter2;
    protected JTable mTable;
    private Map mPointSystemIDToLootColumnMap;
    
    private static final ItemFilter NO_FILTER = new NoFilter();
    private static ItemFilter[] FILTERS1;
    private static ItemFilter[] FILTERS2;
    
    static {
        List filters = new ArrayList();
        filters.add(NO_FILTER);
        for (int i=1; i < LootConstants.QUALITY_KEYS.length; i++) {
            int quality = i;
            filters.add(new QualityItemFilter(quality));
        }
        FILTERS1 = (ItemFilter[]) filters.toArray(new ItemFilter[0]);
        
        filters.clear();
        filters.add(NO_FILTER);
        for (int i=1; i < InstanceManager.getDefault().getNames().size(); i++) {
            String key = InstanceManager.getDefault().getName(i);
            filters.add(new InstanceItemFilter(key));
        }
        for (int i=1; i < BossManager.getDefault().getNames().size(); i++) {
            String name = BossManager.getDefault().getName(i);
            filters.add(new BossItemFilter(name));
        }
        FILTERS2 = (ItemFilter[]) filters.toArray(new ItemFilter[0]);
    }
    
    
    public ItemManagerWindow(IPointSystem[] constraints, 
                             List loots,
                             final IChangeCommitListener listener,
                             String statusText,
                             boolean isIgnoringFilterSettings) {
        super("Item Manager");
        mListener = listener;
        mStatusText = statusText;
        mLoots = loots;
        
        buildLootValueColumns(constraints);

        mTableModel = new LootTableModel(constraints);
        mTable.setModel(mTableModel);
        
        TableColumn nameColumn = mTable.getColumnModel().getColumn(LootTableModel.COLUMN_NAME);
        nameColumn.setHeaderValue("Item Name");
        nameColumn.setPreferredWidth(200);
        
        JLabel label = (JLabel) mTable.getDefaultRenderer(String.class);
        label.setHorizontalAlignment(SwingConstants.CENTER);
        
        nameColumn.setCellRenderer(new LootRenderer());
        
        if (constraints != null) {
            for (int i=0; i < mFilteredConstraints.length; i++) {
                ITableColumn columnDef = 
                    (ITableColumn) mPointSystemIDToLootColumnMap.get(mFilteredConstraints[i].getID());
                if (columnDef != null) {
                    TableColumn columnView = 
                        mTable.getColumnModel().getColumn(LootTableModel.COLUMN_CONSTRAINTS + i);
                    columnView.setHeaderValue(columnDef.getColumnName());
                    columnView.setPreferredWidth(columnDef.getPreferredWidth());
                    if (columnDef.getCellRenderer() != null) {
                        columnView.setCellRenderer(columnDef.getCellRenderer());
                    }
                    if (columnDef.getCellEditor() != null) {
                        columnView.setCellEditor(columnDef.getCellEditor());
                    }
                }
            }
        }
        
        restoreColumnWidths();
        
        GeneralProperties props = GeneralProperties.getDefault();
        Integer filter1Prop = props.getIntegerProperty(props.ITEM_FILTER1);
        Integer filter2Prop = props.getIntegerProperty(props.ITEM_FILTER2);
        if (!isIgnoringFilterSettings) {
            if (filter1Prop != null) {
                int index = filter1Prop.intValue();
                if (index < mItemFilter1.getItemCount()) {
                    mItemFilter1.setSelectedIndex(index);
                }
            }
            if (filter2Prop != null) {
                int index = filter2Prop.intValue();
                if (index < mItemFilter2.getItemCount()) {
                    mItemFilter2.setSelectedIndex(index);
                }
            }
        }
        props.addSaveOnExitProperty(new SaveOnExitProperty(props.ITEM_FILTER1) {
            public Object getValue() { return new Integer(mItemFilter1.getSelectedIndex()); }
        });
        props.addSaveOnExitProperty(new SaveOnExitProperty(props.ITEM_FILTER2) {
            public Object getValue() { return new Integer(mItemFilter2.getSelectedIndex()); }
        });
        
        props.loadWindowRect(
                mWindow, 
                props.ITEM_WINDOW_X,
                props.ITEM_WINDOW_Y,
                props.ITEM_WINDOW_WIDTH,
                props.ITEM_WINDOW_HEIGHT,
                props.DEFAULT_ITEM_WINDOW_WIDTH,
                props.DEFAULT_ITEM_WINDOW_HEIGHT);
    }

    
    private void buildLootValueColumns(IPointSystem[] pointSystems) {
        mPointSystemIDToLootColumnMap = new HashMap();
        List filteredPointSystems = new ArrayList();
        for (int i=0; i < pointSystems.length; i++) {
            IPointSystem pointSystem = pointSystems[i];
            if (pointSystem.isAuctionedLootValues()) {
                // Loot values from auctions are determined at loot-time
                // and are not predetermined.
                continue;
            }
            mPointSystemIDToLootColumnMap.put(
                    pointSystem.getID(), 
                    new ColumnLoot(pointSystem));
            filteredPointSystems.add(pointSystem);
        }
        mFilteredConstraints = (IPointSystem[]) filteredPointSystems.toArray(
                new IPointSystem[filteredPointSystems.size()]);
    }
    
    protected JComponent getBannerCustomizer() {
        JLabel status = new JLabel(mStatusText);
        status.setBorder(BorderFactory.createEmptyBorder(0, 0, 10, 5));
        status.setFont(new Font("Arial", Font.ITALIC, 12));
        
        JLabel filterLabel = new JLabel("Filters:");
        mItemFilter1 = new JComboBox(FILTERS1);
        mItemFilter2 = new JComboBox(FILTERS2);
        ActionListener filterListener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mTableModel.filterData();
                mTable.tableChanged(new TableModelEvent(mTable.getModel()));
            }
        };
        mItemFilter1.addActionListener(filterListener);
        mItemFilter2.addActionListener(filterListener);
        Box filterPane = Box.createHorizontalBox();
        filterPane.add(filterLabel);
        filterPane.add(Box.createHorizontalStrut(5));
        filterPane.add(mItemFilter1);
        filterPane.add(Box.createHorizontalStrut(5));
        filterPane.add(mItemFilter2);
        
        JPanel statusFilter = new JPanel();
        statusFilter.setLayout(new BorderLayout());
        statusFilter.add(status,      BorderLayout.NORTH);
        statusFilter.add(filterPane,  BorderLayout.EAST);
        statusFilter.setBorder(BorderFactory.createEmptyBorder(5, 5, 0, 5));
                
        return statusFilter;
    }
    
    protected JComponent getComponent() {
        mTable = new JTable();
        mTable.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        JScrollPane tableScroller = new JScrollPane(mTable);
        tableScroller.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createEmptyBorder(5, 5, 10, 5),
                BorderFactory.createRaisedBevelBorder()));
        return tableScroller;
    }
    
    private void restoreColumnWidths() {
        GeneralProperties props = GeneralProperties.getDefault();
        for (int i=0; i < mTable.getColumnModel().getColumnCount(); i++) {
            TableColumn iterColumn = mTable.getColumnModel().getColumn(i);
            Integer widthProp = props.getIntegerProperty(props.ITEM_COLUMN_WIDTH + i);
            if (widthProp != null) {
                iterColumn.setPreferredWidth(widthProp.intValue());
            }
            final TableColumn finalColumn = iterColumn;
            props.addSaveOnExitProperty(new SaveOnExitProperty(props.ITEM_COLUMN_WIDTH + i) {
                public Object getValue() { return new Integer(finalColumn.getWidth()); }
            });
        }
    }
    
    private void storeColumnWidths() {
        GeneralProperties props = GeneralProperties.getDefault();
        for (int i=0; i < mTable.getColumnModel().getColumnCount(); i++) {
            TableColumn iterColumn = mTable.getColumnModel().getColumn(i);
            props.setIntegerProperty(props.ITEM_COLUMN_WIDTH + i, iterColumn.getWidth());
        }
    }
    
    protected void closing() {
        storeColumnWidths();
        GeneralProperties props = GeneralProperties.getDefault();
        props.setIntegerProperty(props.ITEM_FILTER1, mItemFilter1.getSelectedIndex());
        props.setIntegerProperty(props.ITEM_FILTER2, mItemFilter2.getSelectedIndex());
        props.setIntegerProperty(props.ITEM_WINDOW_WIDTH,  getWidth());
        props.setIntegerProperty(props.ITEM_WINDOW_HEIGHT, getHeight());
        props.setIntegerProperty(props.ITEM_WINDOW_X,      getX());
        props.setIntegerProperty(props.ITEM_WINDOW_Y,      getY());
        
    }
    
    protected boolean applyChanges() {
        // Apply the changes from the temporary copy of the data to the actual
        // loot data. This applies the table changes to the model.
        int i = 0;
        for (Iterator iter=mLoots.iterator(); iter.hasNext(); i++) {
            Loot loot = (Loot) iter.next();
            LootColumnData columnData = (LootColumnData) mTableModel.mTableData.get(i);
            for (int j=0; j < mFilteredConstraints.length; j++) {
                String value = columnData.getColumn(LootTableModel.COLUMN_CONSTRAINTS + j);
                if (value != null) {
                    int x = 3;
                    x++;
                }
                loot.putValueForConstraint(mFilteredConstraints[j], value);
            }
        }

        ItemMasterDB.getDefault().saveLoots(mLoots, mFilteredConstraints);
        
        return true;
    }

    
    private class LootTableModel extends AbstractTableModel {

        private static final int COLUMN_NAME        = 0;
        private static final int COLUMN_CONSTRAINTS = 1;
        private static final int SIZE_BASE = 1;
        
        private List mTableData;
        private List mFilteredData;
        
        public LootTableModel(IPointSystem[] constraints) {
            // Work with a temporary copy of the data. This allows the user
            // to back out all changes by cancelling.
            mTableData = new ArrayList();
            for (Iterator iter=mLoots.iterator(); iter.hasNext();) {
                Loot loot = (Loot) iter.next();
                LootColumnData columnData = new LootColumnData(loot);
                mTableData.add(columnData);
                for (int j=0; j < mFilteredConstraints.length; j++) {
                    columnData.addColumn(loot.getValueForConstraint(mFilteredConstraints[j]));
                }
            }
            mFilteredData = mTableData;
        }
        
        public void filterData() {
            ItemFilter filter1 = (ItemFilter) mItemFilter1.getSelectedItem();
            ItemFilter filter2 = (ItemFilter) mItemFilter2.getSelectedItem();
            if (filter1 == NO_FILTER && filter2 == NO_FILTER) {
                mFilteredData = mTableData;
            } else {
                mFilteredData = new ArrayList();
                int i = 0;
                for (Iterator iter=mLoots.iterator(); iter.hasNext(); i++) {
                    Loot loot = (Loot) iter.next();
                    if (!filter1.isFiltered(loot) && !filter2.isFiltered(loot)) {
                        mFilteredData.add(mTableData.get(i));
                    }
                }
            }
        }
        
        public int getColumnCount() {
            if (mFilteredConstraints != null) {
                return SIZE_BASE + mFilteredConstraints.length;
            }
            return SIZE_BASE;
        }

        public int getRowCount() {
            return mFilteredData.size();
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            LootColumnData columnData = (LootColumnData) mFilteredData.get(rowIndex);
            
            if (columnIndex == COLUMN_NAME) {
                return columnData.getLoot();
            } else if (columnIndex >= COLUMN_CONSTRAINTS) {
                int constraintsIndex = columnIndex - SIZE_BASE;
                String lootValue = columnData.getColumn(columnIndex);
                if (lootValue != null) {
                    if (LootConstants.isLootCategoryID(lootValue)) {
                        IPointSystem constraint = mFilteredConstraints[constraintsIndex];
                        int index = constraint.getLootConstraintIndex(lootValue);
                        return constraint.getLootConstraintNames().get(index);
                    } else {
                        return lootValue;
                    }
                }
            }
            
            return null;
        }
        
        public boolean isCellEditable(int row, int col) {
            if (col == COLUMN_NAME) { 
                return false;
            } else {
                return true;
            }
        }
        
        public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
            LootColumnData columnData = (LootColumnData) mFilteredData.get(rowIndex);
            
            if (columnIndex >= COLUMN_CONSTRAINTS && aValue != null) {
                int constraintsIndex = columnIndex - SIZE_BASE;
                IPointSystem constraint = mFilteredConstraints[constraintsIndex];
                
                String lootValueName = ((String) aValue).trim();
                if (lootValueName != null && lootValueName.length() > 0) {
                    List lootValueNames = constraint.getLootConstraintNames();
                    int index = lootValueNames.indexOf(lootValueName);
                    if (index >= 0) {
                        // value is a loot category ID
                        String lootValue = constraint.getLootConstraintKey(index);
                        columnData.setColumn(columnIndex, lootValue);
                    } else {
                        // value could be a direct hard value
                        try {
                            Float.parseFloat(lootValueName);
                            columnData.setColumn(columnIndex, lootValueName);
                        } catch (NumberFormatException nfe) {
                            // ignore, user could have mistyped a number
                        }
                    }
                } else {
                    columnData.setColumn(columnIndex, LootConstants.LOOT_CONSTRAINT_KEY_NONE);
                }
            }
        }
    }
    
    
    private static class LootColumnData {
        private Loot mLoot;
        private List mData;
        public LootColumnData(Loot loot) {
            mLoot = loot;
            mData = new ArrayList();
            mData.add(loot.getName());
        }
        public Loot getLoot() {
            return mLoot;
        }
        public void addColumn(String value) {
            mData.add(value);
        }
        public String getColumn(int index) {
            return (String) mData.get(index);
        }
        public void setColumn(int index, String value) {
            mData.set(index, value);
        }
    }
}
