package org.wow.autodkp.view.preferences;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;

import org.wow.autodkp.model.FormatConstants;
import org.wow.autodkp.persistence.Persistence;
import org.wow.autodkp.view.GeneralProperties;
import org.wow.autodkp.view.HelpDialog;


/**
 * 
 * 
 * @author jsandusky
 */
public class FormatTab extends JPanel implements IPreferencesTab {

    private static final Font  TEST_FONT  = new Font("Monospaced", Font.BOLD, 14);
    private static final Color TEST_COLOR = Color.BLUE;
    
    private static final String GUIDE_SHOW_TEXT = "Show Date Formatting Guide";
    private static final String GUIDE_HIDE_TEXT = "Hide Date Formatting Guide";
    
    private JDialog mOwner;
    private PropertyTableModel mTableModel;
    private JTable mTable;
    private JLabel mLongTestValue;
    private JLabel mShortTestValue;
    private boolean mIsModified;
    
    private JButton mStyleButton;
    private JComboBox mTimeZoneButton;
    private HelpDialog mGuide;

    
    public FormatTab(JDialog owner) {
        mOwner = owner;
        
        setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        
        initTablePane();
        JComponent testPane     = getTestPane();
        JComponent stylePane    = getStylePane();
        JComponent timeZonePane = getTimeZonePane();
        
        Box topBox = Box.createVerticalBox();
        topBox.add(stylePane);
        topBox.add(Box.createVerticalStrut(10));
        topBox.add(timeZonePane);
        
        JPanel tablePanel = new JPanel(new BorderLayout());
        tablePanel.add(new JScrollPane(mTable), BorderLayout.CENTER);
        tablePanel.add(testPane,                BorderLayout.SOUTH);
        
        setLayout(new BorderLayout(0, 10));
        add(topBox,     BorderLayout.NORTH);
        add(tablePanel, BorderLayout.CENTER);
        
        updateTestDates();
    }
    
    
    private void initTablePane() {
        final GeneralProperties props = GeneralProperties.getDefault();
        List propertyItems = new ArrayList();
        propertyItems.add(new PropertyItem(props.DATE_FORMAT_LONG,  "Long Form"));
        propertyItems.add(new PropertyItem(props.DATE_FORMAT_SHORT, "Short Form"));
        mTableModel = new PropertyTableModel(propertyItems, "Type", "Format") {
            protected String getProperty(PropertyItem propertyItem) {
                String value = propertyItem.getValue();
                if (value == null || value.length() == 0) {
                    SimpleDateFormat format = props.getDateFormatProperty(propertyItem.getKey());
                    value = format.toPattern();
                }
                return value;
            }
            protected void setProperty(PropertyItem propertyItem, String newValue) {
                String errorMessage = null;
                if (newValue == null || newValue.trim().length() == 0) {
                    errorMessage = "The format you entered is empty.";
                } else {
                    try {
                        // Verify that the pattern the user set is valid.
                        new SimpleDateFormat(newValue);
                        propertyItem.setDateFormatProperty(newValue.trim());
                        updateTestDates();
                        return;
                    } catch (IllegalArgumentException e) {
                        // User entered in invalid pattern.
                        errorMessage = e.getMessage();
                    }
                }
                JOptionPane.showMessageDialog(
                        FormatTab.this,
                        errorMessage,
                        "Invalid Date Format",
                        JOptionPane.ERROR_MESSAGE);
            }
        };
        
        mTable = new JTable(mTableModel);
        
        mTable.setRowHeight(40);
        
        JLabel renderer = (JLabel) mTable.getDefaultRenderer(String.class);
        renderer.setHorizontalAlignment(SwingConstants.CENTER);
        
        mTable.getColumnModel().getColumn(0).setPreferredWidth(100);
        mTable.getColumnModel().getColumn(1).setPreferredWidth(200);
    }
    
    private JComponent getStylePane() {
        mStyleButton = new JButton(GUIDE_SHOW_TEXT);
        mStyleButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                showGuide();
            }
        });
        
        Box styleBox = Box.createHorizontalBox();
        styleBox.add(new JLabel("Date Formats are fully customizable: "));
        styleBox.add(Box.createHorizontalStrut(10));
        styleBox.add(mStyleButton);
        styleBox.add(Box.createHorizontalGlue());
        
        JPanel stylePanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        stylePanel.add(styleBox);

        return stylePanel;
    }
    
    private JComponent getTimeZonePane() {
        final GeneralProperties props = GeneralProperties.getDefault();
        
        String ids[] = TimeZone.getAvailableIDs();
        Arrays.sort(ids);
        mTimeZoneButton = new JComboBox(ids);
        mTimeZoneButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String id = (String) mTimeZoneButton.getSelectedItem();
                if (id != null) {
                    mIsModified = true;
                    props.setStringProperty(props.DATE_TIME_ZONE, id);
                    props.resetDateFormats();
                    props.save();
                    updateTestDates();
                }
            }
        });
        String timeZoneID = props.getStringProperty(props.DATE_TIME_ZONE);
        if (timeZoneID != null) {
            mTimeZoneButton.setSelectedItem(timeZoneID);
        } else {
            // Default to server time.
            mTimeZoneButton.setSelectedItem(FormatConstants.TIME_ZONE_SERVER);
        }
        
        Box timeZoneBox = Box.createHorizontalBox();
        timeZoneBox.add(new JLabel("Time Zone: "));
        timeZoneBox.add(Box.createHorizontalStrut(10));
        timeZoneBox.add(mTimeZoneButton);
        timeZoneBox.add(Box.createHorizontalGlue());

        JPanel timeZonePanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        timeZonePanel.add(timeZoneBox);
        
        return timeZonePanel;
    }
    
    private JComponent getTestPane() {
        JLabel shortTestLabel = new JLabel("Short Form Example: ");
        JLabel longTestLabel  = new JLabel("Long Form Example: ");
        longTestLabel.setPreferredSize(shortTestLabel.getPreferredSize());
        
        mLongTestValue  = new JLabel();
        mShortTestValue = new JLabel();
        mLongTestValue.setFont(TEST_FONT);
        mShortTestValue.setFont(TEST_FONT);
        mLongTestValue.setForeground(TEST_COLOR);
        mShortTestValue.setForeground(TEST_COLOR);

        Box longBox = Box.createHorizontalBox();
        longBox.add(Box.createHorizontalStrut(10));
        longBox.add(longTestLabel);
        longBox.add(Box.createHorizontalStrut(10));
        longBox.add(mLongTestValue);
        longBox.add(Box.createHorizontalGlue());
        JPanel longTestPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        longTestPanel.add(longBox);
        longTestPanel.setBackground(Color.WHITE);

        Box shortBox = Box.createHorizontalBox();
        shortBox.add(Box.createHorizontalStrut(10));
        shortBox.add(shortTestLabel);
        shortBox.add(Box.createHorizontalStrut(10));
        shortBox.add(mShortTestValue);
        shortBox.add(Box.createHorizontalGlue());
        JPanel shortTestPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        shortTestPanel.add(shortBox);
        shortTestPanel.setBackground(Color.WHITE);
        
        Box testBox = Box.createVerticalBox();
        testBox.add(longTestPanel);
        testBox.add(Box.createVerticalStrut(10));
        testBox.add(shortTestPanel);
        testBox.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("Formatted Date Examples (Using Today's Date)"),
                BorderFactory.createEmptyBorder(10, 10, 20, 10)));
        
        return testBox;
    }
    
    private void updateTestDates() {
        GeneralProperties props = GeneralProperties.getDefault();
        SimpleDateFormat longFormat  = props.getDateFormatProperty(props.DATE_FORMAT_LONG);
        SimpleDateFormat shortFormat = props.getDateFormatProperty(props.DATE_FORMAT_SHORT);
        Date currentDate = new Date();
        String timeZoneID = props.getStringProperty(props.DATE_TIME_ZONE);
        if (timeZoneID != null) {
            TimeZone timeZone = TimeZone.getTimeZone(timeZoneID);
            longFormat  = new SimpleDateFormat(longFormat.toPattern());
            shortFormat = new SimpleDateFormat(shortFormat.toPattern());
            longFormat.setTimeZone(timeZone);
            shortFormat.setTimeZone(timeZone);
        }
        mLongTestValue.setText(longFormat.format(currentDate));
        mShortTestValue.setText(shortFormat.format(currentDate));
    }
    
    public void closing() {
    }

    public String getName() {
        return "Format";
    }
    
    public JComponent getComponent() {
        return this;
    }

    public boolean shouldReload() {
        return mIsModified || mTableModel.isModified();
    }
    
    private void showGuide() {
        if (mGuide == null || !mGuide.isVisible()) {
            mStyleButton.setText(GUIDE_HIDE_TEXT);
        } else if (mGuide != null && mGuide.isVisible()) {
            mStyleButton.setText(GUIDE_SHOW_TEXT);
            mGuide.setVisible(false);
            mGuide.dispose();
            mGuide = null;
            return;
        }
        GeneralProperties props = GeneralProperties.getDefault();
        mGuide =  
            new HelpDialog(
                    mOwner,
                    "Date Formatting Guide", 
                    Persistence.DATE_FORMAT_FILE, 
                    props.DATE_GUIDE_WINDOW_X,
                    props.DATE_GUIDE_WINDOW_Y,
                    props.DATE_GUIDE_WINDOW_WIDTH,
                    props.DATE_GUIDE_WINDOW_HEIGHT,
                    false);
        mGuide.addWindowListener(new WindowAdapter() {
            public void windowClosed(WindowEvent e) {
                super.windowClosed(e);
                mStyleButton.setText(GUIDE_SHOW_TEXT);
                mGuide = null;
            }
        });
        mGuide.setVisible(true);
    }
}
