package org.wow.autodkp.view.preferences;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.wow.autodkp.model.points.DefaultPointSystem;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.PointSystemFactory;
import org.wow.autodkp.view.GeneralProperties;
import org.wow.autodkp.view.SaveOnExitProperty;
import org.wow.autodkp.view.preferences.pointsystems.AbstractPointSystemsSection;
import org.wow.autodkp.view.preferences.pointsystems.AdjustmentsSection;
import org.wow.autodkp.view.preferences.pointsystems.BossesSection;
import org.wow.autodkp.view.preferences.pointsystems.GeneralSection;
import org.wow.autodkp.view.preferences.pointsystems.LootCategoriesSection;
import org.wow.autodkp.view.preferences.pointsystems.PointsHTMLTab;
import org.wow.autodkp.view.preferences.pointsystems.PointsTimeSection;

/**
 * 
 * 
 * @author jsandusky
 */
public class PointSystemsTab extends JPanel implements IPreferencesTab {
    
    public static final String TAB_TITLE_GENERAL         = "General";
    public static final String TAB_TITLE_ADJUSTMENTS     = "Adjustments";
    public static final String TAB_TITLE_LOOT_CATEGORIES = "Loot Categories";
    public static final String TAB_TITLE_BOSSES          = "Bosses";
    public static final String TAB_TITLE_TIME            = "Time";
    public static final String TAB_TITLE_POINTS_HTML     = "HTML";
    
    private IPointSystem mCurrentPointSystem;
    private Map mModifiedConstraintIDMap = new HashMap();
    private List mPointSystems = new ArrayList();
    private boolean mIsReloadNeeded;
    
    private JDialog mOwner;
    private JList mConstraintsList;
    private JSplitPane mMainPane;
    private JPanel mConstraintDetailPane;
    
    // Point System tab components
    private JTabbedPane mConstraintTabbedPane;
    private AbstractPointSystemsSection mGeneralPane;
    private AbstractPointSystemsSection mAdjustmentsPane;
    private AbstractPointSystemsSection mLootCategoriesPane;
    private AbstractPointSystemsSection mBossesPane;
    private AbstractPointSystemsSection mTimePane;
    private AbstractPointSystemsSection mPointsHTMLPane;

    
    public PointSystemsTab(JDialog owner, IPointSystem[] pointSystemArray) {
        mOwner = owner;
        
        for (int i=0; i < pointSystemArray.length; i++) {
            mPointSystems.add(pointSystemArray[i]);
        }

        mConstraintsList = new JList(pointSystemArray);
        JComponent constraintsPane = PreferencesWindow.generatePane(
                "Point Systems", 
                getAddConstraintListener(),
                null,
                getDeleteConstraintListener(),
                getConstraintSelectionListener(),
                mConstraintsList);
        
        mGeneralPane        = new GeneralSection(this);
        mAdjustmentsPane    = new AdjustmentsSection(this);
        mLootCategoriesPane = new LootCategoriesSection(this);
        mBossesPane         = new BossesSection(this);
        mTimePane           = new PointsTimeSection(this);
        mPointsHTMLPane     = new PointsHTMLTab(this);
        
        mConstraintTabbedPane = new JTabbedPane();
        mConstraintTabbedPane.addTab(TAB_TITLE_GENERAL,         mGeneralPane.getTabComponent());
        mConstraintTabbedPane.addTab(TAB_TITLE_ADJUSTMENTS,     mAdjustmentsPane.getTabComponent());
        mConstraintTabbedPane.addTab(TAB_TITLE_LOOT_CATEGORIES, mLootCategoriesPane.getTabComponent());
        mConstraintTabbedPane.addTab(TAB_TITLE_POINTS_HTML,     mPointsHTMLPane.getTabComponent());
        mConstraintTabbedPane.addTab(TAB_TITLE_BOSSES,          mBossesPane.getTabComponent());
        
        mConstraintDetailPane = new JPanel(new BorderLayout());
        mConstraintDetailPane.add(mConstraintTabbedPane, BorderLayout.CENTER);
        mConstraintDetailPane.setVisible(false);
        
        mMainPane = new JSplitPane();
        mMainPane.setLeftComponent(constraintsPane);
        mMainPane.setRightComponent(mConstraintDetailPane);
        mMainPane.setDividerLocation(200);
        mMainPane.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createEmptyBorder(5, 5, 5, 10),
                mMainPane.getBorder()));
        
        setLayout(new BorderLayout(0, 0));
        add(mMainPane, BorderLayout.CENTER);
        
        // Select last point system and tab, if any.
        GeneralProperties props = GeneralProperties.getDefault();
        String lastSelectedID = props.getStringProperty(props.PREFS_SELECTED_POINTS);
        if (lastSelectedID != null && lastSelectedID.length() > 0) {
            for (Iterator iter=mPointSystems.iterator(); iter.hasNext();) {
                IPointSystem iterSystem = (IPointSystem) iter.next();
                if (lastSelectedID.equals(iterSystem.getID())) {
                    mConstraintsList.setSelectedValue(iterSystem, true);
                    break;
                }
            }
        }
        
        Integer tabIndex = props.getIntegerProperty(props.PREFS_SELECTED_TAB_POINTS);
        if (tabIndex != null && tabIndex.intValue() < mConstraintTabbedPane.getTabCount()) {
            mConstraintTabbedPane.setSelectedIndex(tabIndex.intValue());
        }
        props.addSaveOnExitProperty(new SaveOnExitProperty(props.PREFS_SELECTED_TAB_POINTS) {
            public Object getValue() { return new Integer(mConstraintTabbedPane.getSelectedIndex()); }
        });

        // Reset modification bit.
        // It may have been set when we were initializing our tabs.
        mModifiedConstraintIDMap.clear();
    }
    
    
    public String getName() {
        return "Point Systems";
    }
    
    private ActionListener getAddConstraintListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                DialogPointSystem diag = new DialogPointSystem(
                        mOwner,
                        "Add New Point System",
                        null);
                diag.setVisible(true);
                ConstraintItem constraintItem = diag.getConstraintItem();
                if (constraintItem != null) {
                    DefaultPointSystem newPointSystem = new DefaultPointSystem(
                                constraintItem.getID(),
                                constraintItem.getName());
                    mPointSystems.add(newPointSystem);
                    PointSystemFactory.saveConstraint(newPointSystem);
                    mConstraintsList.setListData(mPointSystems.toArray());
                    mConstraintsList.setSelectedIndex(mConstraintsList.getModel().getSize() - 1);
                    mIsReloadNeeded = true;
                }
            }
        };
    }
    
    private ActionListener getDeleteConstraintListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (mCurrentPointSystem != null) {
                    int result = JOptionPane.showConfirmDialog(
                            mOwner, 
                            mCurrentPointSystem.getName() + "\n" +
                            "Are you sure you wish to delete this point system?\n" + 
                            "This cannot be undone.",
                            "Delete Point System",
                            JOptionPane.OK_CANCEL_OPTION,
                            JOptionPane.WARNING_MESSAGE);
                    if (result == JOptionPane.YES_OPTION) {
                        String id = mCurrentPointSystem.getID();
                        int deleteIndex = -1;
                        int i = 0;
                        for (Iterator iter=mPointSystems.iterator(); iter.hasNext(); i++) {
                            IPointSystem iterSystem = (IPointSystem) iter.next();
                            if (id.equals(iterSystem.getID())) {
                                deleteIndex = i;
                            }
                        }
                        if (deleteIndex >= 0) {
                            mModifiedConstraintIDMap.remove(id);
                            mPointSystems.remove(deleteIndex);
                            PointSystemFactory.deleteConstraint(mCurrentPointSystem);
                            mCurrentPointSystem = null;
                            mConstraintsList.setListData(mPointSystems.toArray());
                            mConstraintDetailPane.setVisible(false);
                            mIsReloadNeeded = true;
                        }
                    }
                }
            }
        };
    }
    
    private ListSelectionListener getConstraintSelectionListener() {
        return new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (!e.getValueIsAdjusting()) {
                    mCurrentPointSystem = (IPointSystem) mConstraintsList.getSelectedValue();
                    if (mCurrentPointSystem != null) {
                        mGeneralPane.pointSystemChanged();
                        mAdjustmentsPane.pointSystemChanged();
                        mLootCategoriesPane.pointSystemChanged();
                        mBossesPane.pointSystemChanged();
                        mTimePane.pointSystemChanged();
                        mPointsHTMLPane.pointSystemChanged();
                        mConstraintDetailPane.setVisible(true);
                        mMainPane.setDividerLocation(200);
                    }
                }
            }
        };
    }
    
    public boolean shouldReload() {
        return mIsReloadNeeded;
    }
    
    public void closing() {
        IPointSystem selectedSystem = (IPointSystem) mConstraintsList.getSelectedValue();
        String systemID = selectedSystem == null ? "" : selectedSystem.getID();
        GeneralProperties props = GeneralProperties.getDefault();
        props.setStringProperty(props.PREFS_SELECTED_POINTS, systemID);
        
        props.setIntegerProperty(props.PREFS_SELECTED_TAB_POINTS, 
                mConstraintTabbedPane.getSelectedIndex());

        // custom closing operations
        mPointsHTMLPane.closing();
        
        // save changes
        if (mModifiedConstraintIDMap.size() > 0) {
            for (Iterator iter=mModifiedConstraintIDMap.values().iterator(); iter.hasNext();) {
                IPointSystem pointSystem = (IPointSystem) iter.next();
                PointSystemFactory.saveConstraint(pointSystem);
            }
        }
    }
    
    public JComponent getComponent() {
        return this;
    }
    
    public JDialog getOwner() {
        return mOwner;
    }
    
    public IPointSystem getSelectedPointSystem() {
        return mCurrentPointSystem;
    }
    
    public void setCurrentPointSystemModified(boolean isModified) {
        mModifiedConstraintIDMap.put(
                mCurrentPointSystem.getID(), 
                mCurrentPointSystem);
    }
    
    public JTabbedPane getTabbedPane() {
        return mConstraintTabbedPane;
    }
    
    public AbstractPointSystemsSection getBossesSection() {
        return mBossesPane;
    }
    
    public AbstractPointSystemsSection getTimeSection() {
        return mTimePane;
    }
    
    public List getPointSystems() {
        return mPointSystems;
    }
}
