package org.wow.autodkp.view.preferences;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.WindowConstants;
import javax.swing.event.ListSelectionListener;

import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.view.AutoDKPView;
import org.wow.autodkp.view.BasicDialog;
import org.wow.autodkp.view.GeneralProperties;
import org.wow.autodkp.view.ImageUtilities;
import org.wow.autodkp.view.SaveOnExitProperty;
import org.wow.autodkp.view.skin.AutoDKPLookAndFeelSupport;

/**
 * 
 * 
 * @author jsandusky
 */
public class PreferencesWindow extends BasicDialog {

    private static final Dimension ICON_SIZE = new Dimension(36, 32);
    
    private static Image mPlusImage  = ImageUtilities.loadJarImage("plus.png");;
    private static Image mMinusImage = ImageUtilities.loadJarImage("minus.png");
    private static Image mEditImage  = ImageUtilities.loadJarImage("edit.png");
    
    private JTabbedPane mMainPane;
    
    private PointSystemsTab mPointSystemsTab;
    private AlternatesTab mAlternatesTab;
    private HTMLTab mHTMLPane;
    private FormatTab mFormatTab;
    private IPreferencesTab mSkinningTab;
    
    
    public PreferencesWindow(IPointSystem[] pointSystemArray) {
        super(AutoDKPView.getInstance().getFrame(), "Preferences", true);
        addBasicSupport();
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        
        JButton closeButton = new JButton("Close");
        closeButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                closing();
                dispose();
            }
        });
        
        JPanel buttonPane = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        buttonPane.add(closeButton);
        buttonPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        
        mPointSystemsTab = new PointSystemsTab(this, pointSystemArray);
        mAlternatesTab   = new AlternatesTab(this);
        mHTMLPane        = new HTMLTab(this);
        mFormatTab       = new FormatTab(this);
        mSkinningTab = AutoDKPLookAndFeelSupport.getSkinningControls();
        
        mMainPane = new JTabbedPane();
        mMainPane.addTab(mPointSystemsTab.getName(), mPointSystemsTab.getComponent());
        mMainPane.addTab(mAlternatesTab.getName(),   mAlternatesTab.getComponent());
        mMainPane.addTab(mHTMLPane.getName(),        mHTMLPane.getComponent());
        mMainPane.addTab(mFormatTab.getName(),       mFormatTab.getComponent());
        if (mSkinningTab != null) {
            mMainPane.addTab("Skin", mSkinningTab.getComponent());
        }
        
        setLayout(new BorderLayout());
        getContentPane().add(mMainPane,  BorderLayout.CENTER);
        getContentPane().add(buttonPane, BorderLayout.SOUTH);
        
        GeneralProperties props = GeneralProperties.getDefault();
        
        setSize(props.DEFAULT_PREFERENCES_WIDTH, props.DEFAULT_PREFERENCES_HEIGHT);
        
        Integer xLoc = props.getIntegerProperty(props.PREFS_WINDOW_X);
        Integer yLoc = props.getIntegerProperty(props.PREFS_WINDOW_Y);
        if (xLoc == null || yLoc == null) {
            Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
            Dimension windowSize = getSize();
            setLocation(
                    (screenSize.width  - windowSize.width)  / 2, 
                    (screenSize.height - windowSize.height) / 2);
        } else {
            setLocation(
                    xLoc.intValue() < 0 ? 0 : xLoc.intValue(), 
                    yLoc.intValue() < 0 ? 0 : yLoc.intValue());
        }
        
        // Prevent custom sizing because of resizing problems with the
        // Substance Look and Feel.
        setResizable(false);
        
        Integer mainTabIndex = props.getIntegerProperty(props.PREFS_SELECTED_TAB_MAIN);
        if (mainTabIndex != null && mainTabIndex.intValue() < mMainPane.getTabCount()) {
            mMainPane.setSelectedIndex(mainTabIndex.intValue());
        }
        props.addSaveOnExitProperty(new SaveOnExitProperty(props.PREFS_SELECTED_TAB_MAIN) {
            public Object getValue() { return new Integer(mMainPane.getSelectedIndex()); }
        });
        
        setVisible(true);
    }
    
    public static JComponent generatePane(
            String title,
            ActionListener addListener,
            ActionListener editListener,
            ActionListener deleteListener,
            ListSelectionListener selectionListener,
            JList list) {
        final List controls = new ArrayList();
        controls.add(list);
        Box listControl = Box.createHorizontalBox();
        if (addListener != null) {
            JButton addButton = new JButton(new ImageIcon(mPlusImage));
            addButton.setPreferredSize(ICON_SIZE);
            addButton.addActionListener(addListener);
            listControl.add(addButton);
            listControl.add(Box.createHorizontalStrut(5));
            controls.add(addButton);
        }
        if (editListener != null) {
            JButton editButton = new JButton(new ImageIcon(mEditImage));
            editButton.setPreferredSize(ICON_SIZE);
            editButton.addActionListener(editListener);
            listControl.add(editButton);
            listControl.add(Box.createHorizontalStrut(5));
            controls.add(editButton);
        }
        if (deleteListener != null) {
            JButton deleteButton = new JButton(new ImageIcon(mMinusImage));
            deleteButton.setPreferredSize(ICON_SIZE);
            deleteButton.addActionListener(deleteListener);
            listControl.add(deleteButton);
            controls.add(deleteButton);
        }
        listControl.add(Box.createHorizontalGlue());
        listControl.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        
        list.getSelectionModel().addListSelectionListener(selectionListener);
        JScrollPane listScroller = new JScrollPane(list);
        if (title != null) {
            listScroller.setBorder(BorderFactory.createTitledBorder(title));
        }
        list.setBackground(Color.WHITE);
        listScroller.setBackground(Color.WHITE);
        
        JPanel listPane = new JPanel() {
            public void setEnabled(boolean enabled) {
                super.setEnabled(enabled);
                for (Iterator iter=controls.iterator(); iter.hasNext();) {
                    JComponent component = (JComponent) iter.next();
                    component.setEnabled(enabled);
                    if (component instanceof JList) {
                        ((JList) component).clearSelection();
                        component.setForeground(enabled ? Color.BLACK : Color.GRAY);
                    }
                }
            }
        };
        listPane.setLayout(new BorderLayout());
        listPane.add(listControl,  BorderLayout.NORTH);
        listPane.add(listScroller, BorderLayout.CENTER);
        
        return listPane;
    }
    
    protected void closing() {
        GeneralProperties props = GeneralProperties.getDefault();
        props.setIntegerProperty(props.PREFS_WINDOW_X, getX());
        props.setIntegerProperty(props.PREFS_WINDOW_Y, getY());
        props.setIntegerProperty(props.PREFS_SELECTED_TAB_MAIN, 
                mMainPane.getSelectedIndex());
        
        mHTMLPane.closing();
        mAlternatesTab.closing();
        mPointSystemsTab.closing();
        mFormatTab.closing();
        if (mSkinningTab != null) {
            mSkinningTab.closing();
        }
        
        if (mPointSystemsTab.shouldReload() || mAlternatesTab.shouldReload()) {
            AutoDKPView.unloadReload();
        } else if (mHTMLPane.shouldReload() || mFormatTab.shouldReload()) {
            AutoDKPView.getInstance().reloadRaids();
        }
    }
}
