package org.wow.autodkp.view.preferences.pointsystems;

import java.awt.BorderLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JCheckBox;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.wow.autodkp.model.Cost;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.view.preferences.PointSystemsTab;
import org.wow.autodkp.view.preferences.PreferencesWindow;


public class LootCategoriesSection extends AbstractPointSystemsSection {

    private JList mLootCategoryList;
    private LootCategoryItem mCurrentLootCategoryItem;
    private ActionListener mEditLootCategoryListener;
    private JCheckBox mCanDirectEditLoots;
    private JCheckBox mAuctionedLootValues;

    
    public LootCategoriesSection(PointSystemsTab tab) {
        super(tab);
        
        mLootCategoryList = new JList();
        mLootCategoryList.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() > 1 && mLootCategoryList.isEnabled()) {
                    Point point = e.getPoint();
                    int index = mLootCategoryList.locationToIndex(point);
                    if (index >= 0) {
                        SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                                getEditLootCategoryListener().actionPerformed(null);
                            }
                        });
                    }
                }
            }
        });

        mTabComponent = PreferencesWindow.generatePane(
                "Loot Categories", 
                getAddLootCategoryListener(),
                getEditLootCategoryListener(),
                getDeleteLootCategoryListener(),
                getLootCategorySelectionListener(),
                mLootCategoryList);
        mCanDirectEditLoots = new JCheckBox("Allow Direct Editing of Loot Values?");
        mCanDirectEditLoots.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mPointSystemsTab.getSelectedPointSystem().setDirectLootValueEditing(
                        mCanDirectEditLoots.isSelected());
            }
        });
        mAuctionedLootValues = new JCheckBox("Auctioned Loot Values?");
        mAuctionedLootValues.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean isSelected = mAuctionedLootValues.isSelected();
                if (isSelected) {
                    JOptionPane.showMessageDialog(
                            mPointSystemsTab, 
                            "Loot values can now be modified in the formula editor.\n" +
                            "The Item Manager will no longer show pre-defined loot values for this point system.");
                } else {
                    JOptionPane.showMessageDialog(
                            mPointSystemsTab, 
                            "Pre-defined loot values for this point system may now be entered in the Item Manager.\n");
                }
                mPointSystemsTab.getSelectedPointSystem().setAuctionedLootValues(isSelected);
                mTabComponent.setEnabled(!isSelected);
            }
        });
        Box lootBox = Box.createVerticalBox();
        lootBox.add(mCanDirectEditLoots);
        lootBox.add(mAuctionedLootValues);
        lootBox.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        mTabComponent.add(lootBox, BorderLayout.SOUTH);
    }
    
    public void pointSystemChanged() {
        IPointSystem pointSystem = mPointSystemsTab.getSelectedPointSystem();
        mCanDirectEditLoots.setSelected(pointSystem.isDirectLootValueEditing());
        mAuctionedLootValues.setSelected(pointSystem.isAuctionedLootValues());
        updateLootCategories();
        mTabComponent.setEnabled(!mAuctionedLootValues.isSelected());
    }

    private ActionListener getAddLootCategoryListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                DialogLootCategory diag = new DialogLootCategory(
                        mPointSystemsTab.getOwner(),
                        "Add New Loot Category",
                        null);
                diag.setVisible(true);
                LootCategoryItem lootCategoryItem = diag.getLootCategoryItem();
                if (lootCategoryItem != null) {
                    mPointSystemsTab.getSelectedPointSystem().addLootCategory(
                            lootCategoryItem.getID(),
                            lootCategoryItem.getCategoryName(),
                            lootCategoryItem.getCost());
                    mPointSystemsTab.setCurrentPointSystemModified(true);
                    updateLootCategories();
                }
            }
        };
    }
    
    private ActionListener getEditLootCategoryListener() {
        if (mEditLootCategoryListener == null) {
            mEditLootCategoryListener = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (mCurrentLootCategoryItem != null) {
                        DialogLootCategory diag = new DialogLootCategory(
                                mPointSystemsTab.getOwner(),
                                "Edit Loot Category",
                                mCurrentLootCategoryItem);
                        diag.setVisible(true);
                        LootCategoryItem lootCategoryItem = diag.getLootCategoryItem();
                        if (lootCategoryItem != null) {
                            mPointSystemsTab.getSelectedPointSystem().setLootCategory(
                                    lootCategoryItem.getID(),
                                    lootCategoryItem.getCategoryName(),
                                    lootCategoryItem.getCost());
                            mPointSystemsTab.setCurrentPointSystemModified(true);
                            updateLootCategories();
                        }
                    }
                }
            };
        }
        return mEditLootCategoryListener;
    }
    
    private ActionListener getDeleteLootCategoryListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (mCurrentLootCategoryItem != null) {
                    mPointSystemsTab.getSelectedPointSystem().removeLootCategory(
                            mCurrentLootCategoryItem.getID());
                    mPointSystemsTab.setCurrentPointSystemModified(true);
                    updateLootCategories();
                }
            }
        };
    }
    
    private ListSelectionListener getLootCategorySelectionListener() {
        return new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (!e.getValueIsAdjusting()) {
                    mCurrentLootCategoryItem = 
                        (LootCategoryItem) mLootCategoryList.getSelectedValue();
                }
            }
        };
    }
    
    private void updateLootCategories() {
        IPointSystem pointSystem = mPointSystemsTab.getSelectedPointSystem();
        List keys = pointSystem.getLootConstraintKeys();
        List names = pointSystem.getLootConstraintNames();
        List costs = pointSystem.getLootConstraintCosts();
        int itemCount = keys.size();
        LootCategoryItem[] items = new LootCategoryItem[itemCount];
        for (int i=0; i < itemCount; i++) {
            items[i] = new LootCategoryItem(
                    (String) keys.get(i),
                    (String) names.get(i),
                    (Cost)   costs.get(i));
        }
        mLootCategoryList.setListData(items);
        mCurrentLootCategoryItem = null;
    }
}
