package org.wow.autodkp.view.summary;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.Rectangle2D;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import org.wow.autodkp.model.Raid;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.view.AutoDKPView;
import org.wow.autodkp.view.skin.AutoDKPLookAndFeelSupport;
import org.wow.autodkp.view.table.players.PlayerTable;


/**
 * Shows information specific to a raid and point system.
 * This is the main controls panel, other than the 
 * MainSelector panel.
 * 
 * @author jsandusky
 */
public class SummaryPane extends JTabbedPane {

    public static final String TAB_TITLE_BOSS_KILL = "Boss Kills";
    public static final String TAB_TITLE_RAID_TIME = "Time";
    
    private PointSummaryPane mConstraintPane;
    private PlayerTable mPlayerTable;
    private IntroPane mIntroPane;
    private AutoDKPView mRaidView;
    private JPanel mMainPanel;
    private Raid mCurrentRaid;
    private IPointSystem mCurrentPointSystem;
    private BossSummarySection mBossSection;
    private TimeSummarySection mTimeSection;
    
    
    public SummaryPane(AutoDKPView raidView, PlayerTable playerTable) {
        mRaidView = raidView;
        mPlayerTable = playerTable;
        
        setOpaque(false);
        
        mMainPanel = new JPanel();
        mMainPanel.setLayout(new BorderLayout());
        mMainPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
        
        mIntroPane = new IntroPane();
        mIntroPane.setText("");
        mIntroPane.setSubjectText("Your", "Raid");
        mRaidView.setStylePanelColor(PointSummaryPane.BG_COLOR);
        
        mMainPanel.add(mIntroPane, BorderLayout.CENTER);
        
        setPreferredSize(new Dimension(400, 300));
        
        addTab("Summary", mMainPanel);
    }

    
    public void loadConstraintsPane(Raid currentRaid, IPointSystem currentSystem, boolean isStarted) {
        mCurrentRaid = currentRaid;
        mCurrentPointSystem = currentSystem;
        
        if (currentSystem != null && currentRaid != null) {
            mMainPanel.removeAll();
            mRaidView.setStylePanelColor(PointSummaryPane.BG_COLOR);
            mConstraintPane = new PointSummaryPane(
                    this,
                    mPlayerTable, 
                    currentSystem, 
                    currentRaid, 
                    isStarted);
            mMainPanel.add(mConstraintPane, BorderLayout.CENTER);
            mMainPanel.revalidate();
        } else {
            // No point system specified.
            if (currentRaid != null) {
                // But a raid is specified.
                mIntroPane.setSubjectText("Point", "System");
                mRaidView.setStylePanelColor(PointSummaryPane.BG_COLOR);
                mIntroPane.trigger();
            } else {
                mIntroPane.trigger();
            }
        }
        
        handleSummaryTabs();
    }
    
    private void handleSummaryTabs() {
        int timeTabIndex = indexOfTab(TAB_TITLE_RAID_TIME);
        if (mCurrentRaid != null) {
            if (mBossSection == null) {
                // Boss tab initialization
                // Initialize only once, after we select an initial raid.
                mBossSection = new BossSummarySection(this);
                addTab(TAB_TITLE_BOSS_KILL, mBossSection.getTabComponent());
            }
            
            boolean showTimeTab = false;
            if (mCurrentPointSystem != null) {
                if (mCurrentPointSystem.isTimeBased()) {
                    if (mTimeSection == null) {
                        // Time tab initialization
                        // Initialize only once, after we select an initial raid and point system.
                        mTimeSection = new TimeSummarySection(this);
                    }
                    showTimeTab = mCurrentPointSystem.isPointsStarted(mCurrentRaid);
                }
            }
            
            if (showTimeTab) {
                if (timeTabIndex < 0) {
                    addTab(TAB_TITLE_RAID_TIME, mTimeSection.getTabComponent());
                    timeTabIndex = indexOfTab(TAB_TITLE_RAID_TIME);
                }
            } else if (mTimeSection != null && timeTabIndex >= 0) {
                removeTabAt(timeTabIndex);
                timeTabIndex = -1;
            }
        }
        
        
        if (mBossSection != null) {
            mBossSection.sectionChange();
        }
        
        if (mTimeSection != null && timeTabIndex >= 0) {
            mTimeSection.sectionChange();
        }
    }
    
    public Raid getCurrentRaid() {
        return mCurrentRaid;
    }
    
    public IPointSystem getCurrentPointSystem() {
        return mCurrentPointSystem;
    }
    
    public PlayerTable getPlayerTable() {
        return mPlayerTable;
    }
    
    
    private class IntroPane extends JLabel {
        private String mSubject1;
        private String mSubject2;
        private int mOpacity = 255;
        public void setSubjectText(String subject1, String subject2) {
            mSubject1 = subject1;
            mSubject2 = subject2;
        }
        public void trigger() {
            if (isVisible()) {
                // Add a nice text fade-in effect.
                Runnable runner = new Runnable() {
                    public void run() {
                        int increment = 5;
                        for (int i=0; i <= 255; i+=increment, increment+=5) {
                            try {
                                Thread.sleep(50);
                            } catch (Exception e) {
                                // who cares
                            }
                            mOpacity = i;
                            repaint();
                        }
                        mOpacity = 255;
                        repaint();
                    }
                };
                new Thread(runner).start();
            }
        }
        public void paint(Graphics g) {
            super.paint(g);
            int w = getWidth();
            int h = getHeight();
            Graphics2D g2 = (Graphics2D) g;
            GradientPaint grad = new GradientPaint(
                    20, 
                    20, 
                    mRaidView.getStylePanelColor(),
                    w, 
                    h, 
                    AutoDKPLookAndFeelSupport.getPrimaryThemeColor());
            g2.setPaint(grad);
            g2.fillRect(0, 0, w, h);
            
            g2.setFont(new Font("Times New Roman", Font.BOLD, 42));
            positionText(g2, "Select",   0.30, 0.30);
            positionText(g2, mSubject1,  0.50, 0.50);
            positionText(g2, mSubject2,  0.70, 0.70);
        }
        private void positionText(Graphics2D g, String text, double xRatio, double yRatio) {
            Rectangle2D rect = g.getFontMetrics().getStringBounds(text, g);
            int x = (int) ((getWidth() - rect.getWidth()) * xRatio);
            int y = (int) (getHeight() * yRatio);
            // Turn on anti-aliasing.
            g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            Color dark  = AutoDKPLookAndFeelSupport.getDarkColor();
            Color light = AutoDKPLookAndFeelSupport.getLightColor();
            // Create a drop-shadow.
            Color textBG = new Color(dark.getRed(), dark.getGreen(), dark.getBlue(), mOpacity);
            g.setColor(textBG);
            g.drawString(text, x+2, y+2);
            Color textFG = new Color(light.getRed(), light.getGreen(), light.getBlue(), mOpacity);
            g.setColor(textFG);
            g.drawString(text, x, y);
        }
    }
}
