package org.wow.autodkp.view.summary;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellRenderer;

import org.wow.autodkp.model.FormatConstants;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.RollCall;
import org.wow.autodkp.model.points.dkp.TimeDKPCalculation;
import org.wow.autodkp.view.AutoDKPView;


/**
 * Shows time info/controls for the selected raid & points.
 * 
 * @author jsandusky
 */
public class TimeSummarySection extends AbstractSummarySection {

    private static final String TEXT_RAID_LENGTH   = "Raid Length (minutes):";
    private static final String TEXT_DURATION_PRE  = "Duration Points, Pre-Threshold:";
    private static final String TEXT_DURATION_POST = "Duration Points, Post-Threshold:";
    private static final String TEXT_DURATION      = "Duration Points:";
    private static final String TEXT_ROLL_CALL     = "Roll Call Points:";
    private static final String TEXT_TOTAL         = "Maximimum Time Points Possible:";
    
    private JTable mTable;
    
    private JLabel mTimeInRaidLabel = new JLabel();
    private JLabel mStandardDurationPointsLabel = new JLabel();
    private JLabel mThresholdDurationPointsLabel = new JLabel();
    private JLabel mRollCallPointsLabel = new JLabel();
    private JLabel mTotalTimePointsLabel = new JLabel();
    
    private JLabel mStandardDurationNameLabel;
    private JComponent mPostThresholdContainer;
    

    public TimeSummarySection(SummaryPane tab) {
        super(tab);
        
        mTable = new JTable();
        mTable.getTableHeader().setDefaultRenderer(new TableCellRenderer() {
            private JLabel mHeaderLabel;
            public Component getTableCellRendererComponent(
                    JTable table, 
                    Object value, 
                    boolean isSelected, 
                    boolean hasFocus,
                    int row, 
                    int column) {
                if (mHeaderLabel == null) {
                    mHeaderLabel = new JLabel("Time Settings");
                    mHeaderLabel.setHorizontalAlignment(SwingConstants.CENTER);
                    mHeaderLabel.setBorder(BorderFactory.createEtchedBorder());
                }
                return mHeaderLabel;
            }
        });
        mTable.getTableHeader().setReorderingAllowed(false);
        RollCallDateCell cellRenderer = new RollCallDateCell(mSummaryPane);
        RollCallDateCell cellEditor = new RollCallDateCell(mSummaryPane);
        mTable.setDefaultRenderer(RollCallDateItem.class, cellRenderer);
        mTable.setDefaultEditor(RollCallDateItem.class, cellEditor);
        mTable.setRowHeight(cellRenderer.getPreferredHeight());
        
        Box statusBox = Box.createVerticalBox();
        JLabel minutesLabel        = new JLabel(TEXT_RAID_LENGTH);
        mStandardDurationNameLabel = new JLabel(TEXT_DURATION_PRE);
        JLabel thresholdLabel      = new JLabel(TEXT_DURATION_POST);
        JLabel rollCallLabel       = new JLabel(TEXT_ROLL_CALL);
        JLabel totalLabel          = new JLabel(TEXT_TOTAL);
        Dimension labelSize = totalLabel.getPreferredSize();
        labelSize.width += 10;
        statusBox.add(createStatusLine(minutesLabel,   labelSize, mTimeInRaidLabel));
        statusBox.add(createStatusLine(
                mStandardDurationNameLabel, labelSize, mStandardDurationPointsLabel));
        mPostThresholdContainer = 
            createStatusLine(thresholdLabel, labelSize, mThresholdDurationPointsLabel);
        statusBox.add(mPostThresholdContainer);
        statusBox.add(createStatusLine(rollCallLabel,  labelSize, mRollCallPointsLabel));
        statusBox.add(createStatusLine(totalLabel,     labelSize, mTotalTimePointsLabel));
        statusBox.setBorder(BorderFactory.createEmptyBorder(5, 10, 5, 10));
        
        JPanel mainPanel = new JPanel(new BorderLayout());
        mainPanel.add(new JScrollPane(mTable), BorderLayout.CENTER);
        mainPanel.add(statusBox, BorderLayout.SOUTH);
        mainPanel.setBackground(PointSummaryPane.BG_COLOR);
        
        mTabComponent = mainPanel;
    }
    
    
    private JComponent createStatusLine(JLabel descriptionLabel, Dimension size, JLabel label) {
        descriptionLabel.setPreferredSize(size);
        descriptionLabel.setMaximumSize(size);
        Box statusLineBox = Box.createHorizontalBox();
        statusLineBox.add(descriptionLabel);
        statusLineBox.add(Box.createHorizontalStrut(10));
        statusLineBox.add(label);
        statusLineBox.add(Box.createHorizontalGlue());
        JPanel statusLinePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 1));
        statusLinePanel.add(statusLineBox);
        statusLinePanel.setBackground(PointSummaryPane.BG_COLOR);
        return statusLinePanel;
    }
    
    public void sectionChange() {
        updateTimeInfo();
        updateStatusInfo();
    }
    
    private void updateTimeInfo() {
        IPointSystem pointSystem = mSummaryPane.getCurrentPointSystem();
        Map rollCallDateMap = 
            pointSystem.getCurrentRaidData().getRollCallDateMap();
        List rollCalls = new ArrayList();
        int i = 0;
        for (Iterator iter=rollCallDateMap.keySet().iterator(); iter.hasNext(); i++) {
            String name = (String) iter.next();
            Date date = (Date) rollCallDateMap.get(name);
            RollCall rollCall = pointSystem.getRollCall(name);
            if (rollCall.getRecurrence() == 0) {
                rollCalls.add(new RollCallDateItem(name, date));
            } else {
                rollCalls.add(new RollCallDateItem(name, rollCall.getRecurrence()));
            }
        }
        Collections.sort(rollCalls);
        RollCallDateItem[] tableItems = (RollCallDateItem[]) rollCalls.toArray(
                    new RollCallDateItem[rollCalls.size()]);
        mTable.setModel(new RollCallTableModel(tableItems));
    }
    
    private void updateStatusInfo() {
        IPointSystem pointSystem = mSummaryPane.getCurrentPointSystem();
        Map rollCallDateMap = 
            pointSystem.getCurrentRaidData().getRollCallDateMap();
        Date startDate = (Date) rollCallDateMap.get(RollCall.RAID_START_NAME);
        Date endDate   = (Date) rollCallDateMap.get(RollCall.RAID_END_NAME);
        if        (startDate == null && endDate != null) {
            setStatusMessage("enter raid start time");
        } else if (startDate != null && endDate == null) {
            setStatusMessage("enter raid end time");
        } else if (startDate == null && endDate == null) {
            setStatusMessage("enter raid start/end times");
        } else {
            if (endDate.compareTo(startDate) <= 0) {
                setStatusMessage("invalid raid start/end times");
                return;
            }
            TimeDKPCalculation summary = new TimeDKPCalculation();
            summary.initialize(pointSystem, startDate, endDate);
            summary.calculateMaximumPointsAvailable();
            String raidMinutesStr = String.valueOf(summary.getRaidMinutes());
            String standardDurationStr = 
                FormatConstants.NUMBER_FORMAT.format(summary.getMaximumStandardDurationPoints());
            String thresholdDurationStr = 
                FormatConstants.NUMBER_FORMAT.format(summary.getMaximumThresholdDurationPoints());
            String rollCallStr = 
                FormatConstants.NUMBER_FORMAT.format(summary.getMaximumRollCallPoints());
            String totalPossibleStr = 
                FormatConstants.NUMBER_FORMAT.format(summary.getMaximumPossiblePoints());
            Color summaryColor = Color.BLACK;
            setLabelStatus(mTimeInRaidLabel,              raidMinutesStr,       summaryColor);
            setLabelStatus(mStandardDurationPointsLabel,  standardDurationStr,  summaryColor);
            setLabelStatus(mThresholdDurationPointsLabel, thresholdDurationStr, summaryColor);
            setLabelStatus(mRollCallPointsLabel,          rollCallStr,          summaryColor);
            setLabelStatus(mTotalTimePointsLabel,         totalPossibleStr,     summaryColor);
            
            if (summary.hasThreshold()) {
                mStandardDurationNameLabel.setText(TEXT_DURATION_PRE);
            } else {
                mStandardDurationNameLabel.setText(TEXT_DURATION);
            }
            mPostThresholdContainer.setVisible(summary.hasThreshold());
        }
    }
    
    private void setStatusMessage(String message) {
        Color messageColor = Color.GRAY;
        setLabelStatus(mTimeInRaidLabel, message, messageColor);
        setLabelStatus(mStandardDurationPointsLabel, message, messageColor);
        setLabelStatus(mThresholdDurationPointsLabel, message, messageColor);
        setLabelStatus(mRollCallPointsLabel, message, messageColor);
        setLabelStatus(mTotalTimePointsLabel, message, messageColor);
    }
    
    private void setLabelStatus(JLabel label, String message, Color color) {
        label.setText(message);
        label.setForeground(color);
    }
    
    
    private class RollCallTableModel extends AbstractTableModel {

        private RollCallDateItem[] mRollCallData;
        
        public RollCallTableModel(RollCallDateItem[] rollCallData) {
            mRollCallData = rollCallData;
        }
        
        public Class getColumnClass(int columnIndex) {
            return RollCallDateItem.class;
        }
        
        public boolean isCellEditable(int rowIndex, int columnIndex) {
            RollCallDateItem item = mRollCallData[rowIndex];
            return !item.isRecurringItem();
        }
        
        public int getColumnCount() {
            return 1;
        }

        public int getRowCount() {
            return mRollCallData.length;
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            return mRollCallData[rowIndex];
        }
        
        public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
            Date newDate = (Date) aValue;
            RollCallDateItem rollCallDateItem = mRollCallData[rowIndex];
            Date oldDate = rollCallDateItem.getDate();
            if (newDate == null && oldDate == null) {
                // both null, no change
                return;
            } else if (newDate == null || oldDate == null) {
                // one is null, the other is not
                rollCallDateItem.setDate(newDate);
            }
            if (rollCallDateItem.equals(newDate)) {
                // no change
                return;
            } else {
                // different dates
                rollCallDateItem.setDate(newDate);
            }
            
            // save changes
            IPointSystem pointSystem = mSummaryPane.getCurrentPointSystem();
            pointSystem.getCalculator().calculate();
            Map rollCallDateMap = 
                pointSystem.getCurrentRaidData().getRollCallDateMap();
            rollCallDateMap.put(rollCallDateItem.getName(), newDate);
            pointSystem.saveRaid(mSummaryPane.getCurrentRaid());
            AutoDKPView.getInstance().reloadRaids();
        }
    }
}
