package org.wow.autodkp.view.table.players;

import java.awt.Component;
import java.awt.Font;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import org.wow.autodkp.model.Logger;
import org.wow.autodkp.model.Loot;
import org.wow.autodkp.model.PlayerData;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.model.points.Adjustment;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.IPointSystemPlayerData;
import org.wow.autodkp.view.GeneralProperties;
import org.wow.autodkp.view.SaveOnExitProperty;
import org.wow.autodkp.view.filters.ItemFilter;
import org.wow.autodkp.view.html.HTMLGenerator;
import org.wow.autodkp.view.table.DateListRenderer;
import org.wow.autodkp.view.table.ISortListener;
import org.wow.autodkp.view.table.ITableColumn;
import org.wow.autodkp.view.table.LootRenderer;
import org.wow.autodkp.view.table.TableSorter;


/**
 * 
 * 
 * @author jsandusky
 */
public class PlayerTable {

    private JScrollPane mComponent;
    private JTable mTable;
    private TableSorter mSortingModel;
    private PlayerTableModel mTableModel;
    private SeparatorColumn mSeparatorColumn;
    private IPointSystem mCurrentSystem;
    private TableCellRenderer mHeaderRenderer;
    private ISortListener mSortListener;
    private boolean mIsStarted;
    private Raid mCurrentRaid;
    private PlayerData[] mPlayerData;
    private LootRenderer mLootRenderer;
    private DefaultCellEditor mClassCellEditor;
    

    public PlayerTable() {
        mTable = new JTable();
        
        mTable.getTableHeader().setReorderingAllowed(false);
        mTable.getTableHeader().setDefaultRenderer(getHeaderRenderer());
        mTable.getTableHeader().setFont(new Font("Dialog", Font.BOLD, 12));
        
        mTable.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        mComponent = new JScrollPane(mTable);
        mComponent.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createEmptyBorder(5,5, 5, 5),
                BorderFactory.createRaisedBevelBorder()));
        
        JLabel label = (JLabel) mTable.getDefaultRenderer(String.class);
        label.setVerticalAlignment(SwingConstants.NORTH);
        
        mSeparatorColumn = new SeparatorColumn();
        
        mClassCellEditor = new DefaultCellEditor(new JComboBox(PlayerData.CLASSES));
        mClassCellEditor.setClickCountToStart(2);
        
        mSortListener = new ISortListener() {
            public void sorted() {
                adjustRowHeights();
            }
        };
    }
    
    public String generateHTMLReport() {
        if (mTableModel != null && mCurrentSystem != null) {
            return HTMLGenerator.generateHTMLReport(
                    mCurrentRaid,
                    mCurrentSystem,
                    mPlayerData,
                    mTable.getColumnModel(),
                    mSortingModel,
                    mLootRenderer);
        }
        return null;
    }
    
    public Component getComponent() {
        return mComponent;
    }
    
    private void restoreColumnInfo() {
        GeneralProperties props = GeneralProperties.getDefault();
        int count = mTable.getColumnModel().getColumnCount();
        String propName = count == PlayerTableModel.SIZE_DEFAULT ? 
                props.RAID_COLUMN_WIDTH_SHORT :
                props.RAID_COLUMN_WIDTH_LONG;
        for (int i=count - 1; i >= 0; i--) {
            TableColumn iterColumn = mTable.getColumnModel().getColumn(i);
            Integer widthProp = props.getIntegerProperty(propName + i);
            if (widthProp != null) {
                iterColumn.setWidth(widthProp.intValue());
                iterColumn.setPreferredWidth(widthProp.intValue());
            }
            final TableColumn finalColumn = iterColumn;
            props.addSaveOnExitProperty(new SaveOnExitProperty(propName + i) {
                public Object getValue() { return new Integer(finalColumn.getWidth()); }
            });
        }
        
        props.addSaveOnExitProperty(new SaveOnExitProperty(props.RAID_COLUMN_SORTING) {
            public Object getValue() { return generateSortInfoString(mSortingModel); }
        });
    }
    
    private static void restoreSortInfo(final TableSorter sorter) {
        GeneralProperties props = GeneralProperties.getDefault();
        String sortInfoString = props.getStringProperty(props.RAID_COLUMN_SORTING);
        if (sortInfoString != null) {
            Map sortInfo = parseSortInfoString(sortInfoString);
            sorter.loadSortingInfo(sortInfo);
        }
    }
    
    private void storeColumnInfo() {
        GeneralProperties props = GeneralProperties.getDefault();
        int count = mTable.getColumnModel().getColumnCount();
        String propName = count == PlayerTableModel.SIZE_DEFAULT ? 
                props.RAID_COLUMN_WIDTH_SHORT :
                props.RAID_COLUMN_WIDTH_LONG;
        for (int i=count - 1; i >= 0; i--) {
            TableColumn iterColumn = mTable.getColumnModel().getColumn(i);
            props.setIntegerProperty(propName + i, iterColumn.getWidth());
        }
    }
    
    private void storeSortInfo() {
        GeneralProperties props = GeneralProperties.getDefault();
        String sortInfoString = generateSortInfoString(mSortingModel);
        if (sortInfoString != null) {
            props.setStringProperty(props.RAID_COLUMN_SORTING, sortInfoString);
        }
    }
    
    private static Map parseSortInfoString(String sortInfoString) {
        // To preserve sort priority, we use a LinkedHashMap.
        Map sortInfo = new LinkedHashMap();
        StringTokenizer sortTok = new StringTokenizer(sortInfoString, ";");
        while (sortTok.hasMoreTokens()) {
            String pair = sortTok.nextToken();
            StringTokenizer pairTok = new StringTokenizer(pair, "=");
            if (pairTok.countTokens() == 2) {
                try {
                    Integer columnIndex = new Integer(pairTok.nextToken());
                    Boolean isDescending = new Boolean(pairTok.nextToken());
                    sortInfo.put(columnIndex, isDescending);
                } catch (NumberFormatException nfe) {
                    // ignore sort pair
                    Logger.log(nfe);
                }
            }
        }
        
        return sortInfo;
    }
    
    private static String generateSortInfoString(TableSorter sorter) {
        if (sorter != null) {
            Map sortInfo = sorter.getSortingInfo();
            StringBuffer buf = new StringBuffer();
            for (Iterator iter=sortInfo.keySet().iterator(); iter.hasNext();) {
                Integer iterColumn = (Integer) iter.next();
                Boolean iterStatus = (Boolean) sortInfo.get(iterColumn);
                buf.append(iterColumn.toString() + "=" + iterStatus.toString());
                if (iter.hasNext()) {
                    buf.append(";");
                }
            }
            return buf.toString();
        }
        return null;
    }
    
    public PlayerData[] getOnlyRaidMembers(PlayerData[] playerData) {
        List newPlayerData = new ArrayList();
        for (int i=0; i < playerData.length; i++) {
            PlayerData data = playerData[i];
            if (!data.isNonRaidMember()) {
                newPlayerData.add(data);
            }
        }
        return (PlayerData[]) newPlayerData.toArray(new PlayerData[newPlayerData.size()]);
    }
    
    public void switchRaid(Raid currentRaid, 
                           IPointSystem currentSystem, 
                           boolean isStarted,
                           boolean showNonRaidMembers,
                           boolean showLootTimes)
    {
        mCurrentSystem = currentSystem;
        mIsStarted = isStarted;
        mCurrentRaid = currentRaid;
        
        // Ensure we save any changes to column widths before recreating the columns.
        storeColumnInfo();
        
        List columnDirectory = buildPointColumnDirectory(currentRaid, currentSystem);
        
        getLootRenderer().setIsShowingLootTimes(showLootTimes);
        
        PlayerData playerData[] = null;
        if (currentSystem != null) {
            playerData = currentSystem.getNonRemovedPlayerData();
        } else {
            playerData = mCurrentRaid.getPlayerData();
        }
        if (
                !showNonRaidMembers || 
                mCurrentSystem == null || 
                mCurrentSystem.getCurrentRaidData() == null) {
            mPlayerData = getOnlyRaidMembers(playerData);
        }
        else {
            mPlayerData = playerData;
        }
        mTableModel = new PlayerTableModel(
                mCurrentRaid, 
                mCurrentSystem, 
                mPlayerData, 
                columnDirectory);
        
        storeSortInfo();
        mSortingModel = new TableSorter();
        mSortingModel.setTableModel(mTableModel);
        restoreSortInfo(mSortingModel);
        
        mTable.setModel(mSortingModel);
        
        mSortingModel.setTableHeader(mTable.getTableHeader());
        mSortingModel.setSortListener(mSortListener);
        
        TableColumn playerColumn = mTable.getColumnModel().getColumn(PlayerTableModel.COLUMN_PLAYER);
        TableColumn classColumn  = mTable.getColumnModel().getColumn(PlayerTableModel.COLUMN_CLASS);
        TableColumn joinColumn   = mTable.getColumnModel().getColumn(PlayerTableModel.COLUMN_TIME_JOIN);
        TableColumn leaveColumn  = mTable.getColumnModel().getColumn(PlayerTableModel.COLUMN_TIME_LEAVE);
        TableColumn lootColumn   = mTable.getColumnModel().getColumn(PlayerTableModel.COLUMN_LOOT);
        
        classColumn.setCellEditor(mClassCellEditor);
        
        JLabel label = (JLabel) mTable.getDefaultRenderer(String.class);
        label.setHorizontalAlignment(JLabel.CENTER);
        
        joinColumn.setCellRenderer(new DateListRenderer());
        leaveColumn.setCellRenderer(new DateListRenderer());
        lootColumn.setCellRenderer(getLootRenderer());
        
        adjustRowHeights();
        
        playerColumn.setPreferredWidth(100);
        classColumn.setPreferredWidth(80);
        joinColumn.setPreferredWidth(110);
        leaveColumn.setPreferredWidth(110);
        lootColumn.setPreferredWidth(200);

        initializeConstraintsColumns(columnDirectory);
        restoreColumnInfo();
        
        GeneralProperties props = GeneralProperties.getDefault();
        setHeaderValue(props.COLUMN_TRANSLATION_NAME,   "Player", playerColumn);
        setHeaderValue(props.COLUMN_TRANSLATION_CLASS,  "Class",  classColumn);
        setHeaderValue(props.COLUMN_TRANSLATION_JOINS,  "Joins",  joinColumn);
        setHeaderValue(props.COLUMN_TRANSLATION_LEAVES, "Leaves", leaveColumn);
        setHeaderValue(props.COLUMN_TRANSLATION_LOOT,   "Loot",   lootColumn);
    }
    
    private void setHeaderValue(String propertyName, String defaultName, TableColumn column) {
        GeneralProperties props = GeneralProperties.getDefault();
        String altName = props.getStringProperty(propertyName);
        if (altName != null && altName.length() > 0) {
            column.setHeaderValue(altName);
        } else {
            column.setHeaderValue(defaultName);
        }
    }
    
    public void adjustRowHeights() {
        int rowHeight = 17;
        
        for (int viewRow=0; viewRow < mTable.getRowCount(); viewRow++) {
            int modelRow = mSortingModel.modelIndex(viewRow);
            int maxSize = 0;
            for (int i=0; i < mTableModel.getColumnCount(); i++) {
                Object value = mTableModel.getValueAt(modelRow, i);
                
                if (i == PlayerTableModel.COLUMN_LOOT) {
                    // Loot may be filtered, so determine how many
                    // loot items are not filtered.
                    ItemFilter filter = mLootRenderer.getLootFilter();
                    if (value instanceof List) {
                        List list = (List) value;
                        if (filter != null) {
                            int count = 0;
                            for (Iterator iter=list.iterator(); iter.hasNext();) {
                                Loot loot = (Loot) iter.next();
                                if (!filter.isFiltered(loot)) {
                                    count++;
                                }
                            }
                            maxSize = Math.max(maxSize, count);
                        } else {
                            maxSize = Math.max(maxSize, list.size());
                        }
                    }
                    continue;
                }
                
                if (value instanceof List) {
                    maxSize = Math.max(maxSize, ((List) value).size());
                }
            }
            
            if (mCurrentSystem != null) {
                PlayerData playerData = mTableModel.getRaidPlayerDataForRow(modelRow);
                if (playerData != null) {
                    Map map = mCurrentSystem.getCurrentRaidData().getPointSystemPlayerDataMap();
                    if (map != null) {
                        IPointSystemPlayerData constraintData = 
                            (IPointSystemPlayerData) map.get(playerData.getPlayerName());
                        if (constraintData != null) {
                            int formulaLineCount = 
                                FormulaUtilities.getLineCount(constraintData.getFormula());
                            maxSize = Math.max(maxSize, formulaLineCount);
                        }
                    }
                }
            }

            int iterHeight = maxSize * rowHeight;
            if (iterHeight == 0) {
                iterHeight = rowHeight;
            }
            mTable.setRowHeight(viewRow, iterHeight);
        }
    }
    
    public void fireTableDataChanged() {
        if (mCurrentSystem != null && mIsStarted) {
            mCurrentSystem.getCalculator().calculate();
            mTableModel.fireTableRowsUpdated(0, mTableModel.getRowCount());
        }
    }
    
    public void setLootFilter(ItemFilter filter) {
        getLootRenderer().setLootFilter(filter);
    }
    
    private ITableColumn[] buildPointColumns(IPointSystem pointSystem, Raid raid) {
        List columns = new ArrayList();
        columns.add(new ColumnIncomingTotal(pointSystem, raid));
        columns.add(new ColumnFormula(pointSystem, raid));
        Map adjustmentIDToNameMap = pointSystem.getAdjustmentMap();
        for (Iterator iter=adjustmentIDToNameMap.keySet().iterator(); iter.hasNext();) {
            String adjustmentID = (String) iter.next();
            Adjustment adjustment = (Adjustment) adjustmentIDToNameMap.get(adjustmentID);
            columns.add(new ColumnAdjustment(pointSystem, raid, adjustmentID, adjustment.getName()));
        }
        columns.add(new ColumnOutgoingTotal(pointSystem, raid));
        return (ITableColumn[]) columns.toArray(new ITableColumn[columns.size()]);
    }
    
    private List buildPointColumnDirectory(Raid currentRaid, IPointSystem currentSystem) {
        List columnDirectory = new ArrayList();
        columnDirectory.add(mSeparatorColumn);
        if (currentSystem != null && mIsStarted) {
            ITableColumn[] columns = buildPointColumns(currentSystem, currentRaid);
            if (columns != null && columns.length > 0) {
                if (columnDirectory.size() > 1) {
                    columnDirectory.add(mSeparatorColumn);
                }
                for (int j=0; j < columns.length; j++) {
                    columnDirectory.add(columns[j]);
                }
            }
        }
        return columnDirectory;
    }
    
    private TableCellRenderer getHeaderRenderer() {
        if (mHeaderRenderer == null) {
            mHeaderRenderer = new PlayerTableHeaderRenderer();
        }
        return mHeaderRenderer;
    }
    
    private LootRenderer getLootRenderer() {
        if (mLootRenderer == null) {
            mLootRenderer = new LootRenderer();
        }
        return mLootRenderer;
    }
    
    private void initializeConstraintsColumns(List columnDirectory) {
        for (int i=0; i < columnDirectory.size(); i++) {
            ITableColumn columnDef = (ITableColumn) columnDirectory.get(i);
            TableColumn columnView = 
                mTable.getColumnModel().getColumn(PlayerTableModel.COLUMN_SEPARATOR + i);
            columnView.setHeaderValue(columnDef.getColumnName());
            columnView.setPreferredWidth(columnDef.getPreferredWidth());
            if (columnDef.getCellRenderer() != null) {
                columnView.setCellRenderer(columnDef.getCellRenderer());
            }
            if (columnDef.getCellEditor() != null) {
                columnView.setCellEditor(columnDef.getCellEditor());
            }
            columnView.setResizable(columnDef.isResizable());
            if (columnDef instanceof SeparatorColumn) {
                columnView.setMinWidth(SeparatorColumn.WIDTH);
                columnView.setMaxWidth(SeparatorColumn.WIDTH);
            }
        }
    }
    
    public boolean removeSelectedPlayers() {
        Map constraintData = mCurrentSystem.getCurrentRaidData().getPointSystemPlayerDataMap();
        int selectedRows[] = mTable.getSelectedRows();
        if (selectedRows.length > 0) {
            List playersToRemove = new ArrayList();
            for (int i = 0; i < selectedRows.length; i++) {
                int row = selectedRows[i];
                PlayerData data = mTableModel.getRaidPlayerDataForRow(mSortingModel.modelIndex(row));
                playersToRemove.add(data.getPlayerName());
            }

            for (Iterator removeIter = playersToRemove.iterator(); removeIter.hasNext();) {
                String name = (String)removeIter.next();
                if (constraintData.containsKey(name)) {
                    // player is in raid, remove from constraints list
                    constraintData.remove(name);
                }
            }

            mCurrentSystem.saveRaid(mCurrentRaid);
            return true;
        }
        
        return false;
    }
    
    public void updatePlayerRow(String playerName) {
        int rowUpdated = mTableModel.updateRowForPlayer(playerName);
        if (rowUpdated >= 0) {
            int viewRow = mSortingModel.viewIndex(rowUpdated);
            if (viewRow >= 0) {
                mTable.getSelectionModel().setSelectionInterval(viewRow, viewRow);
                mTable.scrollRectToVisible(mTable.getCellRect(viewRow, 0, true));
            }
        }
    }
}
